//*****************************************************************************
//*
//*
//*		ColorListBox.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	This code creates a Color-List-Box for MFC
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<Afx.h>
#include	<tchar.h>
#include	"ColorListBox.h"



IMPLEMENT_DYNAMIC(CColorListBox,CListBox)


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CColorListBox::CColorListBox()
{


}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Creates a new Color-List-Box
//	dwStyle		: Style of the Window
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd	: Pointer to parent window
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CColorListBox::Create(DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

	ColorListBoxRegister(AfxGetInstanceHandle());


return CWnd::Create(_T("ColorListBox"),NULL,dwStyle,sRect,pParentWnd,nId);
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Color-List-Box
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd  : Pointer to parent window
//	nId         : Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CColorListBox::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

    ColorListBoxRegister(AfxGetInstanceHandle());


return CWnd::CreateEx(dwExStyle,_T("ColorListBox"),NULL,dwStyle,sRect,pParentWnd,nId);
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control over a dialog item
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//	uItem       : Is the ID of the dialog item
//	pDlg		: Pointer to parent dialog
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CColorListBox::CreateEx(DWORD dwExStyle,DWORD dwStyle,UINT uItem,CDialog *pDlg,UINT nId)
{
CPoint	sPoint(0,0);
CRect	sRect;
CWnd   *pWnd;


	pWnd=pDlg->GetDlgItem(uItem);
	ASSERT(pWnd!=0);


	pWnd->GetWindowRect (&sRect);
	pDlg->ClientToScreen(&sPoint);

	sRect.left	  -= sPoint.x;
	sRect.right   -= sPoint.x;
	sRect.top	  -= sPoint.y;
	sRect.bottom  -= sPoint.y;


	if(!CColorListBox::CreateEx(dwExStyle,dwStyle|WS_CHILD|WS_VISIBLE,sRect,pDlg,nId))
		{
		return FALSE;
		}

	SetWindowPos(pWnd,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOREDRAW);


return TRUE;
}


//*****************************************************************************
//*
//*		SelectWord
//*
//*****************************************************************************
//	Whlt das Wort aus auf dem der Cursor gerade steht
//	Ergibt die Anzahl der Markierten Buchstaben
int	CColorListBox::SelectWord()
{
int		iMarkStartX;
int		iMarkStopX;


	ASSERT(::IsWindow(m_hWnd));

	SendMessage(WM_KEYDOWN,VK_CLEAR,1);
	SendMessage(WM_KEYUP  ,VK_CLEAR,1);

	GetMark(iMarkStartX,iMarkStopX);

return (iMarkStopX<iMarkStartX)? iMarkStartX-iMarkStopX:iMarkStopX-iMarkStartX;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	sString		: Zeiger auf den Textpuffer in dem der Text abgelegt werden soll.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetText(int nIndex,CString &sString)
{
CLB_ITEM	sItem;
int			iSize;


	ASSERT(::IsWindow(m_hWnd));

	iSize			= GetTextLen(nIndex);
	if(iSize<0)return LB_ERR;



	sItem.uMask		= CLBF_TEXT;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= iSize+1;
	sItem.pText		= sString.GetBuffer(iSize);

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		sString.ReleaseBuffer(0);
		return LB_ERR;
		}

	sString.ReleaseBuffer(sItem.iTextMax);


return sItem.iTextMax;
}



//*****************************************************************************
//*
//*		GetTextColor
//*
//*****************************************************************************
//	Die Textfarben aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der die Farben geholt werden sollen.
//	pBuffer		: Zeiger auf den Farbpuffer in dem die Farben abgelegt werden sollen.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetTextColor(int nIndex,LPVOID pBuffer)
{
CLB_ITEM	sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.uMask		= CLBF_COLOR;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= 0x4000;
	sItem.pColor	= (LPBYTE)pBuffer;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		return LB_ERR;
		}


return sItem.iTextMax;
}

//*****************************************************************************
//*
//*		GetTextColor
//*
//*****************************************************************************
//	Die Textfarben aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der die Farben geholt werden sollen.
//	pBuffer		: Zeiger auf den Farbpuffer in dem die Farben abgelegt werden sollen.
//	iSize		: Ist die Gre des Puffers in Byte.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetTextColor(int nIndex,LPVOID pBuffer,int iSize)
{
CLB_ITEM	sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.uMask		= CLBF_COLOR;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= iSize;
	sItem.pColor	= (LPBYTE)pBuffer;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		iSize = 0;
		return LB_ERR;
		}


return sItem.iTextMax;
}

//*****************************************************************************
//*
//*		GetTextColor
//*
//*****************************************************************************
//	Die Textfarben aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	iSize		: Hier wird die Lnge des Textes gespeichert.
//	Ergibt einen Zeiger auf den Text, oder 0 bei einem Fehler.
const BYTE *CColorListBox::GetTextColor(int nIndex,int &iSize)
{
CLB_ITEM	sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.uMask	= CLBF_COLORPTR|CLBF_COLOR;
	sItem.iItem	= nIndex;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		iSize = 0;
		return 0;
		}

	iSize = sItem.iTextMax;


return sItem.pColor;
}

//*****************************************************************************
//*
//*		GetTextColor
//*
//*****************************************************************************
//	Die Textfarben aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	Ergibt einen Zeiger auf den Text, oder 0 bei einem Fehler.
const BYTE *CColorListBox::GetTextColor(int nIndex)
{
CLB_ITEM	sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.uMask	= CLBF_COLORPTR|CLBF_COLOR;
	sItem.iItem	= nIndex;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		return 0;
		}



return sItem.pColor;
}


//*****************************************************************************
//*
//*		GetSelectedText
//*
//*****************************************************************************
//	Holt den ausgewhlten Text aus dem Fenster
//	sText		: Hier wird der Text gespeichert
//	iMode		: Bei 0 wird "\r\n" an Zeilenenden angehngt, ansonsten "\n"
//	Ergibt die Art der Marke:
//					0 = keine Markierung
//					1 = einzeilige Markierung
//					2 = mehrzeilige Markierung
int CColorListBox::GetSelectedText(CString &sText,int iMode)
{
LPTSTR	pPtr;
LPCTSTR	pEnd;
LPCTSTR	pText;
int		iStartX;
int		iStartY;
int		iStopX;
int		iStopY;
int		iSize;
int		iRet;
int		iMax;


	if(sText.IsEmpty()==0)sText.Empty();


	iRet = GetMark(iStartX,iStartY,iStopX,iStopY);

	if(iRet==1)											// Einzeiliger Text
		{
		iSize  = iStopX-iStartX;
		pText  = GetText(iStartY);
		pPtr   = sText.GetBufferSetLength(iSize);
		memcpy(pPtr,pText+iStartX,iSize*sizeof(TCHAR));
		sText.ReleaseBuffer(iSize);
		}
	else if(iRet==2)									// Mehrzeiliger Text
		{
		pEnd   = (iMode)? _T("\n"):_T("\r\n");
		pText  = GetText(iStartY);
		sText  = pText+iStartX;
		sText += pEnd;

		for(iStartY++;iStartY<iStopY;iStartY++)
			{
			sText += GetText(iStartY);
			sText += pEnd;
			}

		iSize  = sText.GetLength();
		pText  = GetText(iStopY,iMax);
		pPtr   = sText.GetBufferSetLength(iSize+iStopX+2);
		memcpy(pPtr+iSize,pText,iStopX*sizeof(TCHAR));

		if(iMax<=iStopX)
			{
			memcpy(pPtr+iSize+iStopX,pEnd,sizeof(TCHAR)*2);
			iStopX += (iMode)? 1:2;
			}

		sText.ReleaseBuffer(iSize+iStopX);
		}


return iRet;
}

//*****************************************************************************
//*
//*		HitTest
//*
//*****************************************************************************
//	Prft was auf eine Koordinate im Fenster ist
//	sPoint	: Ist die Klienz Koortinate im Fenster
//	pFlags	: Hier wird gespeichert wo der Treffer war. Bei NULL kein Test.
//				CLBT_NOWHERE       0x00000001	Keine Treffer
//				CLBT_ONITEMCHECK   0x00000002	Auf Checkbox
//				CLBT_ONITEMLABEL   0x00000004	Auf Text
//				CLBT_ABOVE         0x00000008	ber den Zeilen
//				CLBT_BELOW         0x00000010	Unter den Zeilen
//				CLBT_TORIGHT       0x00000020	Rechts vom Text
//				CLBT_TOLEFT        0x00000040	Links vom Text
//				CLBT_SELECTED	   0x00000080	Auf ausgewhlten Text
//				CLBT_ONITEMICON	   0x00000100	Auf Icon
//				CLBT_ONITEMRIGHT   0x00080000	Koordinate ist rechts vom Text eines Eintrages
//	Ergibt die Zeile des Treffers, oder -1 wenn es keine Zeile wahr
int CColorListBox::HitTest(CPoint sPoint,unsigned *pFlags)
{
CLB_HITTESTINFO		sInfo;


	sInfo.sPoint.x	= sPoint.x;
	sInfo.sPoint.y	= sPoint.y;

	ColorListBox_HitTest(m_hWnd,&sInfo);

	if(pFlags)*pFlags=sInfo.uFlags;


return sInfo.iItem;
}

//*****************************************************************************
//*
//*		HitTest
//*
//*****************************************************************************
//	Prft was auf eine Koordinate im Fenster ist
//	iPosX/Y	: Ist die Client-Koortinate im Fenster
//	pFlags	: Hier wird gespeichert wo der Treffer war. Bei NULL kein Test.
//				CLBT_NOWHERE       0x00000001	Keine Treffer
//				CLBT_ONITEMCHECK   0x00000002	Auf Checkbox
//				CLBT_ONITEMLABEL   0x00000004	Auf Text
//				CLBT_ABOVE         0x00000008	ber den Zeilen
//				CLBT_BELOW         0x00000010	Unter den Zeilen
//				CLBT_TORIGHT       0x00000020	Rechts vom Text
//				CLBT_TOLEFT        0x00000040	Links vom Text
//				CLBT_SELECTED	   0x00000080	Auf ausgewhlten Text
//				CLBT_ONITEMICON	   0x00000100	Auf Icon
//				CLBT_ONITEMRIGHT   0x00080000	Koordinate ist rechts vom Text eines Eintrages
//	Ergibt die Zeile des Treffers, oder -1 wenn es keine Zeile wahr
int CColorListBox::HitTest(int iPosX,int iPosY,unsigned *pFlags)
{
CLB_HITTESTINFO		sInfo;


	sInfo.sPoint.x	= iPosX;
	sInfo.sPoint.y	= iPosY;

	ColorListBox_HitTest(m_hWnd,&sInfo);

	if(pFlags)*pFlags=sInfo.uFlags;


return sInfo.iItem;
}
