//*****************************************************************************
//*
//*
//*		FastListCtrl.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	This code creates a Tree-List-Control for MFC
//
#include	<Afx.h>
#include	<tchar.h>
#include	"ColorListBoxCtrl.h"



IMPLEMENT_DYNAMIC(CColorListBox,CListBox)


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CColorListBox::CColorListBox()
{


}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Creates a new Tree-List-Control
//	dwStyle		: Style of the Window
//					LVS_CHECKBOXES
//					LVS_DISABLEDRAGDROP
//					LVS_EDITLABELS
//					LVS_FULLROWSELECT
//					LVS_HASBUTTONS
//					LVS_HASLINES
//					LVS_INFOTIP
//					LVS_LINESATROOT
//					LVS_NONEVENHEIGHT
//					LVS_NOSCROLL
//					LVS_NOTOOLTIPS
//					LVS_RTLREADING
//					LVS_SHOWSELALWAYS
//					LVS_SINGLEEXPAND
//					LVS_TRACKSELECT
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd	: Pointer to parent window
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CColorListBox::Create(DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

	ColorListBoxRegister(AfxGetInstanceHandle());


return CWnd::Create(_T("ColorListBox"),NULL,dwStyle,sRect,pParentWnd,nId);
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//                	LVS_CHECKBOXES
//                	LVS_DISABLEDRAGDROP
//                	LVS_EDITLABELS
//                	LVS_FULLROWSELECT
//                	LVS_HASBUTTONS
//                	LVS_HASLINES
//                	LVS_INFOTIP
//                	LVS_LINESATROOT
//                	LVS_NONEVENHEIGHT
//                	LVS_NOSCROLL
//                	LVS_NOTOOLTIPS
//                	LVS_RTLREADING
//                	LVS_SHOWSELALWAYS
//                	LVS_SINGLEEXPAND
//                	LVS_TRACKSELECT
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd  : Pointer to parent window
//	nId         : Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CColorListBox::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

    ColorListBoxRegister(AfxGetInstanceHandle());


return CWnd::CreateEx(dwExStyle,_T("ColorListBox"),NULL,dwStyle,sRect,pParentWnd,nId);
}


//*****************************************************************************
//*
//*		EnableSel
//*
//*****************************************************************************
//	Ermglicht oder sperrt das Auswahlfeld eines Eintrages.
//	nIndex		: Ist die Zeilennummer vom Text.
//	bEnable		: Soll das Auswahlfeld eingeblendet werden
//	Ergibt			0       Wenn der Eintrag existiert
//					LB_ERR	Wenn der Eintrag nicht existiert
int CColorListBox::EnableSel(int nIndex,BOOL bEnable)
{
CLB_ITEM	sItem; 

	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask		 = CLBF_STATE;
	sItem.uState	 = (bEnable)? 0:CLBS_NOBOX;
	sItem.uStateMask = CLBS_NOBOX;
	sItem.iItem		 = nIndex;
	
	
return ColorListBox_SetItem(m_hWnd,&sItem,0);
}

//*****************************************************************************
//*
//*		SetFont
//*
//*****************************************************************************
//	Die Schriftart fr die Listbox einstellen.
//	pFont		: Ist der Zeiger auf den Font
//	Ergibt TRUE wenn der Font eingestellt wurde.
BOOL CColorListBox::SetFont(CFont *pFont)
{


	if(!pFont           )return FALSE;
	if(!pFont->m_hObject)return FALSE;


return SetFont((HFONT)pFont->m_hObject);
}

//*****************************************************************************
//*
//*		SetColor
//*
//*****************************************************************************
//	Eine Farbe einstellen.
//	iNr			: Ist der Index der Farbe die eingestellt werden soll.
//					1 = Farbe fr ausgewhlte Zeile (mit  Focus)
//					2 = Farbe fr ausgewhlte Zeile (ohne Focus)
//					3 = Farbe fr ausgewhlte Textmarke
//	cRgbText	: Ist die Textfarbe fr die Farbe.
//					0xFFFFFFFF steht fr keine nderung.
//					Bei den Farben 1 bis 3 kann mit 0xFF000000 bestimmt
//				    werden, das die ursprngliche Textfarbe fr Marken
//					verwendet werden soll.
//	cRgbHg		: Ist die Hintergrundfarbe fr die Farbe.
//					0xFFFFFFFF steht fr die aktuelle Hintergrund Farbe
//	Ergibt den Index der eingestllten Farbe, bzw. LB_ERR bei einem Fehler.
int CColorListBox::SetColor(int iNr,COLORREF cRgbText,COLORREF cRgbHg)
{

	ASSERT(::IsWindow(m_hWnd));

	if((unsigned)iNr>=CLB_MAX_COLOR+CLB_ADD_COLOR)
		{
		return LB_ERR;
		}

	if(cRgbText!=0xFFFFFFFF)
		{
		ColorListBox_SetColor(m_hWnd,iNr,cRgbText);
		}


	ColorListBox_SetColor(m_hWnd,iNr+CLB_MAX_COLOR+CLB_ADD_COLOR,cRgbHg);



return iNr;
}


//*****************************************************************************
//*
//*		SetFlags
//*
//*****************************************************************************
//	Stellt einige Optionen fr das Fenster ein
//	iFlags		: Sind die einzelnen Optionen
//					CLB_FLAG_MARKOFF		Keine       Markierungen
//					CLB_FLAG_MARKSINGLE		Mehrzeilige Markierungen
//					CLB_FLAG_MARKMULTI		Einzeilige  Markierungen
//					CLB_FLAG_MARKFULL		Ganzzeilige Markierungen
//					CLB_FLAG_LINES			Farben fr ungerade Zeilen
//					CLB_FLAG_ALTERNATED		Abwechselnde Faren fr die Zeilen
//					CLB_FLAG_SELMARK		Ausgewhlte Zeile markieren
//					CLB_FLAG_SELFIELD		Auswahlfelder einblenden
//					CLB_FLAG_SELDBLCLK		Auswahl mit Doppelcklick
//					CLB_FLAG_SELSPACE		Auswahl mit SPACE-Taste
//					CLB_FLAG_SELRETURN		Auswahl mit SPACE-Taste
//					CLB_FLAG_CURSOR			Cursor ein bzw. ausblenden
//					CLB_FLAG_WORDDBLCLK		Wortauswahl mit Doppelcklick
//					CLB_FLAG_TABEXIT		Bei TAB den Focus ans Elternfenster geben
//					CLB_FLAG_NOCONTEXT		Kein Kontextmen
//					CLB_FLAG_NOCHGNOTIFY 	Keine Nachrichten beim Whlen eines Eintrages	
//					CLB_FLAG_NOCHARSELCET	Keine Auswahl von Zeilen ber Texteingabe
//	iMask		: Ist die Maske der Optionen die gendert werden sollen
//					CLB_FLAG_MARKMASK		Maske fr Markierungsart
//					CLB_FLAG_LINES			Farben fr ungerade Zeilen
//					CLB_FLAG_ALTERNATED		Abwechselnde Faren fr die Zeilen
//					CLB_FLAG_SELMARK		Ausgewhlte Zeile markieren
//					CLB_FLAG_SELFIELD		Auswahlfelder einblenden
//					CLB_FLAG_SELDBLCLK		Auswahl mit Doppelcklick
//					CLB_FLAG_SELSPACE		Auswahl mit SPACE-Taste
//					CLB_FLAG_SELRETURN		Auswahl mit SPACE-Taste
//					CLB_FLAG_CURSOR			Cursor ein bzw. ausblenden
//					CLB_FLAG_WORDDBLCLK		Wortauswahl mit Doppelcklick
//					CLB_FLAG_TABEXIT		Bei TAB den Focus ans Elternfenster geben
//					CLB_FLAG_NOCONTEXT		Kein Kontextmen
//					CLB_FLAG_NOCHGNOTIFY 	Keine Nachrichten beim Whlen eines Eintrages	
//					CLB_FLAG_NOCHARSELCET	Keine Auswahl von Zeilen ber Texteingabe
//	Ergibt die alten Optionen
int CColorListBox::SetFlags(int iFlags,int iMask)
{
int		iOld;


	ASSERT(::IsWindow(m_hWnd));
	
	iOld =	ColorListBox_GetExtendedStyle  (m_hWnd);
			ColorListBox_SetExtendedStyleEx(m_hWnd,iFlags,iMask);


return iOld;
}

//*****************************************************************************
//*
//*		InsertString
//*
//*****************************************************************************
//	Fgt einen Text in die Listbox ein.
//	nIndex		: Ist die Zeilennummer bei der der Text eingefgt werden soll.
//				  Wenn die Zeilennummer kleiner 0 ist, wird der Text am Ende
//				  eingefgt.
//	pText		: Ist der Zeiger auf den einzufgenden Text.
//	pColor		: Ist ein Zeiger auf die Farbnummern fr den Text.
//				  Die Gre dieses Arrays muss mindestens so lange sein
//				  wie die Lnge des Textes.
//	iLen		: Ist die Lnge des Textes in Zeichen. Bei -1 wird die
//				  Lnge mit strlen(pItem) ermittelt.
//	iState		: Ist der Status fr den Eintag
//					CLBS_SELECTED	0x00000001	Auswahlbit setzen	
//					CLBS_NOBOX		0x00000002	Keine Auswahlbox zeichen
//					CLBF_FAST		0x10000000	Schnell einfgen
//					CLBF_ENDSCROLL	0x20000000	Einfgeposition muss sichtbar bleiben
//	iMask		: Bestimmt welche Statusbits gltig sind.
//	Ergibt die Zeilennummer bei der der Text eingefgt wurde, bzw. LB_ERR bei
//	einem Fehler. Wenn zuwenig Speicher vorhanden ist, wird LB_ERRSPACE zurckgegeben.
int CColorListBox::InsertString(int nIndex,LPCTSTR pText,const BYTE *pColor,int iLen,int iState,int iMask)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));

	sItem.uMask		 = CLBF_TEXT|CLBF_COLOR|CLBF_STATE;
	sItem.iTextMax	 = iLen; 
	sItem.pColor	 = (LPBYTE)pColor; 
	sItem.pText		 = (LPTSTR)pText; 
	sItem.uState	 = iState;
	sItem.uStateMask = iMask;
	sItem.iItem		 = nIndex;
	
	
return ColorListBox_InsertItem(m_hWnd,&sItem,iState&(CLBF_FAST|CLBF_ENDSCROLL));
}

//*****************************************************************************
//*
//*		InsertString
//*
//*****************************************************************************
//	Fgt einen Text in die Listbox ein.
//	nIndex		: Ist die Zeilennummer bei der der Text eingefgt werden soll.
//				  Wenn die Zeilennummer kleiner 0 ist, wird der Text am Ende
//				  eingefgt.
//	pText		: Ist der Zeiger auf den einzufgenden Text.
//	pColor		: Ist ein Zeiger auf die Farbnummern fr den Text.
//				  Die Gre dieses Arrays muss mindestens so lange sein
//				  wie die Lnge des Textes.
//	iLen		: Ist die Lnge des Textes in Zeichen. Bei -1 wird die
//				  Lnge mit strlen(pItem) ermittelt.
//	iState		: Ist der Status fr den Eintag
//					CLBS_SELECTED	0x00000001	Auswahlbit setzen	
//					CLBS_NOBOX		0x00000002	Keine Auswahlbox zeichen
//					CLBF_FAST		0x10000000	Schnell einfgen
//					CLBF_ENDSCROLL	0x20000000	Einfgeposition muss sichtbar bleiben
//	iMask		: Bestimmt welche Statusbits gltig sind.
//	Ergibt die Zeilennummer bei der der Text eingefgt wurde, bzw. LB_ERR bei
//	einem Fehler. Wenn zuwenig Speicher vorhanden ist, wird LB_ERRSPACE zurckgegeben.
int CColorListBox::InsertString(int nIndex,LPARAM lParam,LPCTSTR pText,const BYTE *pColor,int iLen,int iState,int iMask)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask		 = CLBF_TEXT|CLBF_COLOR|CLBF_STATE|CLBF_PARAM;
	sItem.iTextMax	 = iLen; 
	sItem.lParam	 = lParam;
	sItem.pColor	 = (LPBYTE)pColor; 
	sItem.pText		 = (LPTSTR)pText; 
	sItem.uState	 = iState;
	sItem.uStateMask = iMask;
	sItem.iItem		 = nIndex;
	
	
return ColorListBox_InsertItem(m_hWnd,&sItem,iState&(CLBF_FAST|CLBF_ENDSCROLL));
}

//*****************************************************************************
//*
//*		SetTextColor
//*
//*****************************************************************************
//	Stellt die Farben eines Textes ein. 
//	nIndex		: Ist die Zeilennummer vom Text. 
//	pColor		: Ist ein Zeiger auf die Farbnummern. Die Gre dieses Arrays muss mindestens so lange sein wie die Lnge des Textes. Fr jeden Buchstaben kann eine eigene Farbe vergeben werden. Es sind Farbnummern zwischen 0 und CLB_MAX_COLOR erlaubt.
//					0	  Standartfarbe  
//					1	  Ausgewhlter Text mit Fokus  
//					2	  Ausgewhlter Text ohne Fokus  
//					3	  Mit Cursor markierte Text  
//					4...  Frei verwendbare Farben 
//	Ergibt die Zeilennummer vom Text, bzw. LB_ERR bei einem Fehler. 
int CColorListBox::SetTextColor(int nIndex,const BYTE *pColor) 
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask		 = CLBF_COLOR;
	sItem.pColor	 = (LPBYTE)pColor; 
	
	
return ColorListBox_SetItem(m_hWnd,&sItem,0);
}

//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
//	Stellt den Text und die Farben einer Zeile ein. 
//	nIndex		: Ist die Zeilennummer vom Text. 
//	pText		: Ist der Zeiger auf den neuen Text.
//	iLen		: Ist die Lnge des Textes, bei -1 wir strlen(...) benutzt.
//	pColor		: Ist ein Zeiger auf die Farbnummern. Die Gre dieses Arrays muss mindestens so lange sein wie die Lnge des Textes. Fr jeden Buchstaben kann eine eigene Farbe vergeben werden. Es sind Farbnummern zwischen 0 und CLB_MAX_COLOR erlaubt.
//					0	  Standartfarbe  
//					1	  Ausgewhlter Text mit Fokus  
//					2	  Ausgewhlter Text ohne Fokus  
//					3	  Mit Cursor markierte Text  
//					4...  Frei verwendbare Farben 
//	Ergibt die Zeilennummer vom Text, bzw. LB_ERR bei einem Fehler. 
int CColorListBox::SetText(int nIndex,LPCTSTR pText,int iLen,const BYTE *pColor)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	if(iLen<0)iLen=_tcslen(pText);

	sItem.uMask		 = CLBF_COLOR|CLBF_TEXT;
	sItem.pColor	 = (LPBYTE)pColor; 
	sItem.pText		 = (LPTSTR)pText;
	sItem.iTextMax	 = iLen;
	
	
return ColorListBox_SetItem(m_hWnd,&sItem,0);
}

//*****************************************************************************
//*
//*		SelectWord
//*
//*****************************************************************************
//	Whlt das Wort aus auf dem der Cursor gerade steht
//	Ergibt die Anzahl der Markierten Buchstaben
int	CColorListBox::SelectWord()
{
int		iMarkStartX;
int		iMarkStopX;


	ASSERT(::IsWindow(m_hWnd));

	SendMessage(WM_KEYDOWN,VK_CLEAR,1);
	SendMessage(WM_KEYUP  ,VK_CLEAR,1);

	GetMark(iMarkStartX,iMarkStopX);

return (iMarkStopX<iMarkStartX)? iMarkStartX-iMarkStopX:iMarkStopX-iMarkStartX;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	iSize		: Hier wird die Lnge des Textes gespeichert.
//	Ergibt einen Zeiger auf den Text, oder 0 bei einem Fehler.
LPCTSTR	CColorListBox::GetText(int nIndex,int &iSize)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask	= CLBF_TEXTPTR|CLBF_TEXT;
	sItem.iItem	= nIndex;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		iSize = 0;
		return 0;
		}
	
	
	iSize = sItem.iTextMax;


return sItem.pText;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	Ergibt einen Zeiger auf den Text, oder 0 bei einem Fehler.
LPCTSTR	CColorListBox::GetText(int nIndex)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask	= CLBF_TEXTPTR|CLBF_TEXT;
	sItem.iItem	= nIndex;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		return 0;
		}
	


return sItem.pText;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	pBuffer		: Zeiger auf den Textpuffer in dem der Text abgelegt werden soll.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetText(int nIndex,LPTSTR pBuffer)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask		= CLBF_TEXT;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= 0x4000;
	sItem.pText		= pBuffer;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		return LB_ERR;
		}
	

return sItem.iTextMax;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	pBuffer		: Zeiger auf den Textpuffer in dem der Text abgelegt werden soll.
//	iSize		: Ist die Gre des Textpuffers in Zichen.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetText(int nIndex,LPTSTR pBuffer,int iSize)
{
CLB_ITEM	sItem; 

	
	ASSERT(::IsWindow(m_hWnd));
	
	sItem.uMask		= CLBF_TEXT;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= iSize;
	sItem.pText		= pBuffer;

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		iSize = 0;
		return LB_ERR;
		}
	

return sItem.iTextMax;
}


//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Einen Text aus einer Zeile auslesen
//	nIndex		: Ist die Zeilennummer von der der Text geholt werden soll.
//	sString		: Zeiger auf den Textpuffer in dem der Text abgelegt werden soll.
//	Ergibt die Lnge des Textes in Zeichen bzw. LB_ERR bei einem Fehler.
int CColorListBox::GetText(int nIndex,CString &sString)
{
CLB_ITEM	sItem; 
int			iSize;

	
	ASSERT(::IsWindow(m_hWnd));
	
	iSize			= GetTextLen(nIndex);
	if(iSize<0)return LB_ERR;

	

	sItem.uMask		= CLBF_TEXT;
	sItem.iItem		= nIndex;
	sItem.iTextMax	= iSize+1;
	sItem.pText		= sString.GetBuffer(iSize);

	if(!ColorListBox_GetItem(m_hWnd,&sItem))
		{
		sString.ReleaseBuffer(0);
		return LB_ERR;
		}
	
	sString.ReleaseBuffer(sItem.iTextMax);


return sItem.iTextMax;
}

