//*****************************************************************************
//*
//*
//*		ColorListBoxWnd.cpp
//*
//*
//*****************************************************************************
//
//	Dieser Code erstellt ein farbiges List-Fenster.
//
//
//	Copyright (C) Anton Zechner 2010
//
//	ColorListBoxWnd is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
//	Sourcecode which use ColorListBoxWnd must be published. Commercial users
//	must published their code too, or make an licence agreement with me.
//
//
//	ColorListBoxWnd wird unter GNU GENERAL PUBLIC LICENSE (GPL) vertreiben.
//	Sourcecode welcher ColorListBoxWnd verwendet muss verffentlicht werden.
//	Komerzielle Nutzer mssen ihren Code ebenfalls verffentlichen, oder
//	eine Nutzungsvereinbarung mit mir treffen.
//
//	az_software@inode.at
//
//	Version:	2.40
//
#ifdef		UNICODE
#ifndef	   _UNICODE
#define    _UNICODE
#endif
#endif

#include	<tchar.h>
#include	<stdio.h>
#include	<string.h>
#include	<malloc.h>
#include	<windows.h>
#include	<winuser.h>
#include	<commctrl.h>

#include	"WinExtension.h"
#include	"ColorListBoxWnd.h"

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		UNICODE
#define 	str_len				(unsigned)wcslen
#define 	str_cmp				wcscmp
#define 	str_ncpy			wcsncpy
#define 	str_ncmp			wcsncmp
#define 	str_icmp			wcsicmp
#define		CF_TEXT_EX			CF_UNICODETEXT
#else
#define 	str_len				(unsigned)strlen
#define 	str_cmp				strcmp
#define 	str_ncpy			strncpy
#define 	str_ncmp			strncmp
#define 	str_icmp		   _stricmp
#define		CF_TEXT_EX			CF_TEXT
#endif
#define		MAX_COLORS			(CLB_MAX_COLOR+CLB_ADD_COLOR)
#define 	ID_SCROLL			0x3910
#define 	ID_UPDATE			0x3911
#define 	ID_MARKALL			0x3912
#define 	ID_SELALL			0x3913
#define 	ID_SELNONE			0x3914
#define 	ID_COPY				0x3915
#define 	ID_CLEARALL			0x3916
#define 	FIRST_LINE			0xFFFFFFFE
#define 	U(h)				((unsigned)(h))
#define 	GetHandle(h)		((ListBoxData*)GetWindowLongPtr(h,0))
#define 	SYNCH_TYPE			CRITICAL_SECTION
#define 	SYNCH_CREATE(d)		InitializeCriticalSection(&d->sSynch);
#define 	SYNCH_DELETE(d)		DeleteCriticalSection(&d->sSynch);
#define 	UNLOCK(d)			LeaveCriticalSection(&d->sSynch)
#define 	LOCK(d)				EnterCriticalSection(&d->sSynch)
#define 	BPBF_COMPATIBLEBM	0
#define 	ICONS_ADD			4
#define 	SELECT_SIZE			24
#define 	SELECT_HEIGHT		14
#ifndef		BP_CHECKBOX
#define		BP_CHECKBOX			3
#endif
#ifndef		CBS_UNCHECKEDNORMAL
#define		CBS_UNCHECKEDNORMAL 1
#endif
#ifndef		CBS_UNCHECKEDHOT
#define		CBS_UNCHECKEDHOT	2
#endif
#ifndef		CBS_UNCHECKEDPRESSED
#define		CBS_UNCHECKEDPRESSED 3
#endif
#ifndef		CBS_UNCHECKEDDISABLED
#define		CBS_UNCHECKEDDISABLED 4
#endif
#ifndef		CBS_CHECKEDNORMAL
#define		CBS_CHECKEDNORMAL	5
#endif
#ifndef		CBS_CHECKEDHOT
#define		CBS_CHECKEDHOT		6
#endif
#ifndef		CBS_CHECKEDPRESSED
#define		CBS_CHECKEDPRESSED	7
#endif
#ifndef		CBS_CHECKEDDISABLED
#define		CBS_CHECKEDDISABLED	8
#endif
#ifndef		GET_X_LPARAM
#define		GET_X_LPARAM(x)		((short)LOWORD(x))
#endif
#ifndef		GET_Y_LPARAM
#define		GET_Y_LPARAM(x)		((short)HIWORD(x))
#endif
#ifdef		DrawText
#undef 		DrawText
#define 	DrawText(h,p,n,r,f)	DrawTextEx(h,(LPTSTR)(p),n,r,f,0)
#endif



const int	iKeyMaskFirst       = (30<< 0)|(28<< 5)|(24<<10)|(16<<15);
const int	iKeyMaskDefault		= (28<< 0)|(28<< 5)|( 3<<10)|( 3<<15)|(3<<20);
const int	iKeyMaskDefaultUl	= (24<< 0)|(24<< 5)|(24<<10)|( 7<<15)|(7<<20);
const int	iKeyMaskExtentedUl	= ( 6<< 0)|( 1<< 5)|( 1<<10);
const int	iKeyMaskExtented	= ( 2<< 0)|( 1<< 5);
const int	iKeyMaskUpperCase   = (16<<10)|(16<<15);


typedef struct
	{
	LPARAM		lParam;										// Ist der LPARAM Wert fr den Eintrag
	WORD		wLineLen;									// Zeilenlnge
	WORD		wLineSize;									// Zeilen-Speicherlnge
	WORD		wLineSel;									// Ist Zeile aus gewhlt
	WORD		wIdent;										// Ist der Einschub fr die Zeile
	BYTE	   *pTextData;									// Zeilendaten
	}BaseItem;

typedef struct
	{
	HWND		hWnd;										// Fenster Handle
	SYNCH_TYPE	sSynch;										// Zugriffsbeschgrnkung
	LPVOID		hTheme;										// Handle fr benutztes Thema
	HIMAGELIST	hStates;									// Handle der Icon-Liste fr States und Overlay
	HIMAGELIST	hIcons;										// Handle der Icon-Liste fr Icons
	HFONT		hFontL;										// Der letzte zugewiesene Font auf Index [0]
	HFONT		aFonts[4];									// Ist die Liste mit den Zeichenstzen (Bit0=bold Bit1=fixed)
	COLORREF	aColors[MAX_COLORS*2];						// 0=Hintergrundfarbe  1=Abwechselnte Farbe  2=Farbe fr Trennlinien  3=Textfarbe
	PFCLBLINECB	pLineCallback;								// Callback um Zeilendaten zu holen
	LPVOID		pLineCbParam;								// Parameter fr den Zeilendaten Callback
	unsigned	uLastDblClick;								// Wann war der letzte Doppelklick
	int			iLastDblClickX;								// Koordinaten des letzten Doppelklicks
	int			iLastDblClickY;								//		*
	int			iFontHeight;								// Ist die Hhe des Fonts
	int			iFontWidth;									// Schriftbreite in Pixel
	int			iFontLine;									// Ist die Position der Linie beim unterstreichen
	int			iFontOff;									// Ist die Position um der ein Text horizontal verschoben wird
	int			iStatesXsize;								// Breite der States und Overlay Icons
	int			iStatesYsize;								// Hhe   der States und Overlay Icons
	int			iIconsXsize;								// Breite der Icons
	int			iIconsYsize;								// Hhe   der Icons
	int			iRowHeight;									// Ist die Hhe einer Zeile
	int			iRowPost;									// Sind die Pixelreihen unter einer Zeile
	int			iRowPre;									// Sind die Pixelreihen ober einer Zeile
	int			iMaxSizeX;									// Die Gre des breitesten sichtbaren Eintrages
	int			iMaxX;										// Maximale X-Gre in Zeichen
	int			iCursorX;									// Aktuelle Cursor Position
	int			iCursorY;									//		*
	int			iCaretX;									// Koordinaten fr Caret
	int			iCaretY;									//		*
	int			iCaretExist;								// Wurde das Caret erzeugt
	int			iCaretOn;									// Ist der Cursor sichtbar
	int			iDragRow;									// Merker fr Drag Funktion
	int			iDragItem;									//		*
	int			iDragPosX;									// 		*
	int			iDragPosY;									//		*
	int			iDragFlags;									//		*
	int			iMarkStartX;								// Beginn einer Marke
	int			iMarkStartY;								//		*
	int			iMarkStopX;									// Ende einer Marke
	int			iMarkStopY;									//		*
	int			iMarkMove;									// Wurde die Marke verschoben
	int			iMarkMode;									// Ist eine Marke vorhanden 0=Keine 1=Ziehen 2=Fertig
	int			iMarkType;									// Erlaubte Markierungsart (0=Keine 1=Einzeilig 2=Mehzeilig 3=Nur ganze Zeilen)
	int			iLastStartX;								// Speicher fr Beginn einer Marke
	int			iLastStartY;								//		*
	int			iLastStopX;									// Speicher fr Ende einer Marke
	int			iLastStopY;									//		*
	int			iLastMode;									// Speicher fr ist eine Marke vorhanden 0=Keine 1=Ziehen 2=Fertig
	int			aTabPos[CLB_MAX_TABS];						// Tabulatorpositionen
	int			iTabCount;									// Anzhalt der Eintrge in aTabPos[]
	int			iTabSize;									// Ist die Tab-Breite in Pixel
	int			iTabSteps;									// Ist die Tab-Breite in Buchstaben
	int			iKeyStepMask;								// Maske fr Wortsprnge mit der Tastatur
	BaseItem   *pListItems;									// Zeiger auf Item Zeiger
	unsigned	uListItemsMax;								// Gre der Liste mit den vorhanden Eintrge (allokiert um 1 grer)
	unsigned	uListItemsCount;							// Anzahl der vorhanden Eintrge
	unsigned	uListItemsCountTmp;							// Anzahl der vorhanden Eintrge frs Schnelleinfgen
	unsigned	uLastScrolY;								// Wann wurde zuletzt die Y-Scrollbar gezeichnet
	unsigned	uScrollYtmp;								// Aktuelle Y-Scroll-Position fr schnell einfgen
	unsigned	uScrollY;									// Aktuelle Y-Scroll-Position
	unsigned	uScrollX;									// Aktuelle X-Scroll-Position
	unsigned	uSizeX;										// Aktuelle X-Fenster-Gre
	unsigned	uSizeY;										// Aktuelle Y-Fenster-Gre
	unsigned	uSizeCharX;									// Aktuelle X-Fenster-Gre in Buchstaben
	unsigned	uStyle;										// Ist der aktuelle Style des Fensters
	unsigned	uStyleEx;									// Erweiterte Sytle-Flags (siehe CLB_EX_???)
	unsigned	uStartPixel;								// Ist die Y-Koordinate bei der der erste Eintrag beginnt
	unsigned	uMaxEnties;									// Anzahl der sichtbaren Eintrge (inkl. halbsichtbare)
	unsigned	uPageEnties;								// Anzahl der sichtbaren Eintrge (ohne  halbsichtbare)
	unsigned	uCheckedDown;								// Ist die Checkbox gerade gedrckt
	unsigned	uCheckedLine;								// Ist die Checkbox ber der der Cursor steht
	unsigned	uSelectedCount;								// Anzahl der ausgewhlten Eintrge
	unsigned	uSelectedBase;								// Ist der Eintrag ab dem gewhlt wurde
	unsigned	uSelectedLine;								// Ist der Eintrag der gerade gewhlt ist
	unsigned	uStartText;									// Pixel bei dem der Text beginnt
	unsigned	uOldXPage;									// Alte Werte fr X-Scroll-Bar
	unsigned	uOldXCount;									//			*
	unsigned	uOldYPage;									// Alte Werte fr Y-Scroll-Bar
	unsigned	uOldYCount;									//			*
	char		aFontAscent[4];								// Verschub in der Hhe des Fonts
	char		aFontOpaque[4];								// Hintergrund des Font tranparent zeichnen
	TCHAR		cTempText [2048];							// Zeiter Textpuffer fr Callbacks
	BYTE		aTempColor[2048];							// Zeiter Farbpuffer fr Callbacks
	char		cColorChanged[MAX_COLORS*2];				// Welche Farbe wurden verndert
	char		cUpdateScrollX;								// Die Scrollposition aktualisieren
	char		cUpdateScrollY;								// Die Scrollposition aktualisieren
	char		cUpdateTimer;								// Ist der Update-Timer an
	char		cLockChanges;								// Sperren von Fensternderungen
	char		cUpdateCaret;								// Den Cursor aktualisieren
	char		cUpdateView;								// Das Fenster aktualisieren
	char		cStateFrame;								// Extra Hhe fr State und Overlay Bilder
	char		cStateColor;								// Hintergund fr Overlay-Icons (-1=erste aus Zeile)
	char 		cStatesMode;								// Die hStates Image-Liste wurde fr die Checkboxen erzeugt
	char		cRedrawFlag;								// Das neuzeichnen sperren
	char		cIconsMode;									// Die hIcons Image-Liste lschen
	char		cIconColor;									// Hintergund fr Icons (-1=erste aus Zeile)
	char		cIconFrame;									// Extra Hhe fr Icons
	char		cIsEnabled;									// Ist das Fenster freigegeben
	char		cUseTheme;									// Checkboxen als Thema zeichnen (0=not init = 1=nein 2=ja)
	char		cDragFlag;									// Gab es einen LB-Click fr Drag (0=nein 1=item 2=text)
	char		cFastFlag;									// Wurde etwas schnell eingefgt
	char		cHasFocus;									// Hat das Fenster den Fokus
	char		cHasCaret;									// Hat das Fenster einen Cursor
	char		cReSelect;									// Soll die Auswahl neu selektiert werden
	char 		cTimerOn;									// Ist der Scrolltimer ein
	WORD		wIdent;										// Standart-Text-Einrckung 
	}ListBoxData;

typedef	HRESULT (WINAPI *SetWindowThemeT )(HWND,LPCWSTR,LPCWSTR);
typedef	HRESULT (WINAPI *EndBufferedPtT  )(HANDLE,BOOL);
typedef	HANDLE  (WINAPI *BeginBufferedPnT)(HDC,RECT*,DWORD,LPVOID,HDC*);
typedef	HRESULT (WINAPI *BufferedPtInitT )(VOID);
typedef	HRESULT (WINAPI *BufferedPtInitT )(VOID);
typedef	LPVOID  (WINAPI *OpenThemeDataT  )(HWND,LPCWSTR);
typedef	HRESULT (WINAPI *CloseThemeDataT )(LPVOID);
typedef	HRESULT (WINAPI *DrawThemeBackgT )(LPVOID,HDC,int,int,const RECT*,const RECT*);
typedef	HRESULT (WINAPI *GetThemeBackgRcT)(LPVOID,HDC,int,int,LPCRECT,LPRECT);
typedef	HRESULT (WINAPI *GetThemePartSzT )(LPVOID,HDC,int,int,LPCRECT,int,LPSIZE);

static HPEN				hPen1			= NULL;
static HPEN				hPen2			= NULL;
static HMODULE			hUxThemeDll		= NULL;
static SetWindowThemeT	pSetWindowTheme	= NULL;
static EndBufferedPtT	pEndBufferedPt  = NULL;
static BeginBufferedPnT	pBeginBufferedPt= NULL;
static BufferedPtInitT	pBufferedPtInit = NULL;
static BufferedPtInitT	pBufferedPtExit = NULL;
static OpenThemeDataT	pOpenThemeData	= NULL;
static CloseThemeDataT	pCloseThemeData	= NULL;
static DrawThemeBackgT	pDrawThemeBackg	= NULL;
static GetThemeBackgRcT	pGetThemeBackgRc= NULL;
static GetThemePartSzT	pGetThemePartSz = NULL;
static HINSTANCE		hMainInstance	= NULL;
static HFONT			hDefaultFontN	= NULL;
static HFONT			hDefaultFontF	= NULL;
static LONG				lWindowCount	= -1;
static SIZE				sThemeSize		= {16,16};
static TCHAR			cKeyData[16];
static unsigned 		uKeyLast;
static unsigned 		uKeyPos;
static int		 		iKeyItem;
static void				ColorListBoxDraw		(HWND hWnd,HDC hDc,RECT *pRect);
static LRESULT CALLBACK ColorListBoxProc		(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
static int				ColorListBoxGetItemRect (ListBoxData *pData,unsigned uLine,unsigned uFlags,RECT *pRect);
static int				ColorListBoxSelectItem  (ListBoxData *pData,unsigned uLine,BOOL bRedraw=TRUE);
static void				ColorListBoxMarkAll		(ListBoxData *pData);



//*****************************************************************************
//*
//*		ColorListBoxRegister
//*
//*****************************************************************************
//	Registiert das ColorListBox Fenster.
//	Ergibt 1 wenn das Fenster erfolgreich registiert wurde.
int ColorListBoxRegister(HINSTANCE hInstance)
{
static int		iIsRegistered=FALSE;
WNDCLASSEX		sClass;


    if(iIsRegistered)return TRUE;


    memset(&sClass,0,sizeof(sClass));

    sClass.cbSize           = sizeof(sClass);
    sClass.style            = CS_DBLCLKS;
    sClass.lpfnWndProc      = ColorListBoxProc;
    sClass.cbClsExtra       = 0;
    sClass.cbWndExtra       = sizeof(ListBoxData*);
    sClass.hInstance        = hInstance;
    sClass.hIcon            = NULL;
    sClass.hCursor          = LoadCursor(NULL,IDC_ARROW);
    sClass.hbrBackground	= NULL;
    sClass.lpszMenuName     = NULL;
    sClass.hIconSm          = NULL;
    sClass.lpszClassName	= _T("ColorListBox");

    if(!RegisterClassEx(&sClass))return 0;

	hMainInstance			= hInstance;
    iIsRegistered			= TRUE;


return TRUE;
}

//*****************************************************************************
//*
//*		GlobalInit
//*
//*****************************************************************************
static void GlobalInit()
{
LPVOID			hTheme;
LONG			lCount;
RECT			sRect;
LOGBRUSH		sLog;


	   lCount = InterlockedIncrement(&lWindowCount);
	if(lCount>0)return;

	sLog.lbColor = GetSysColor(COLOR_BTNSHADOW);
	sLog.lbStyle = PS_SOLID;
	sLog.lbHatch = 0;
	hPen1		 = CreatePen(PS_SOLID,2,RGB(0,128,0));
	hPen2		 = CreatePen(PS_SOLID,1,RGB(0,255,0));


	if(!hUxThemeDll)
		{
		   hUxThemeDll = LoadLibrary(_T("UxTheme.dll"));
		if(hUxThemeDll)
			{
			pSetWindowTheme  = (SetWindowThemeT )GetProcAddress(hUxThemeDll,"SetWindowTheme"     );
			pEndBufferedPt   = (EndBufferedPtT  )GetProcAddress(hUxThemeDll,"EndBufferedPaint"   );
			pBeginBufferedPt = (BeginBufferedPnT)GetProcAddress(hUxThemeDll,"BeginBufferedPaint" );
			pBufferedPtInit	 = (BufferedPtInitT	)GetProcAddress(hUxThemeDll,"BufferedPaintInit"  );
			pBufferedPtExit  = (BufferedPtInitT	)GetProcAddress(hUxThemeDll,"BufferedPaintUnInit");
			pOpenThemeData	 = (OpenThemeDataT	)GetProcAddress(hUxThemeDll,"OpenThemeData"		 );
			pCloseThemeData	 = (CloseThemeDataT	)GetProcAddress(hUxThemeDll,"CloseThemeData"	 );
			pDrawThemeBackg	 = (DrawThemeBackgT )GetProcAddress(hUxThemeDll,"DrawThemeBackground");
			pGetThemePartSz  = (GetThemePartSzT	)GetProcAddress(hUxThemeDll,"GetThemePartSize"   );
			pGetThemeBackgRc = (GetThemeBackgRcT)GetProcAddress(hUxThemeDll,"GetThemeBackgroundContentRect");

			hTheme = pOpenThemeData(NULL,L"BUTTON");

			sRect.left		 = 0;
			sRect.right		 = 3;
			sRect.top		 = 0;
			sRect.bottom	 = 3;

			pGetThemePartSz(hTheme,NULL,BP_CHECKBOX,CBS_UNCHECKEDNORMAL,&sRect,1,&sThemeSize);
			pCloseThemeData(hTheme);
			}
		}

	if(pBufferedPtInit)
		{
		pBufferedPtInit();
		}

	if(hDefaultFontN==NULL)		
		{
		hDefaultFontN  = GetDefaultFont();
		}

}


//*****************************************************************************
//*
//*		GlobalDeinit
//*
//*****************************************************************************
static void GlobalDeinit()
{
int	lCount;


	   lCount=InterlockedDecrement(&lWindowCount);
	if(lCount>=0)return;

	if(hPen1)
		{
		DeleteObject(hPen1);
		hPen1 = NULL;
		}

	if(hPen2)
		{
		DeleteObject(hPen2);
		hPen2 = NULL;
		}

	if(pBufferedPtExit)
		{
		pBufferedPtExit();
		}

}

//*****************************************************************************
//*
//*		InitColors
//*
//*****************************************************************************
//	Initialisiert die Farben
static void InitColors(ListBoxData *pData)
{


	memset(pData->aColors+MAX_COLORS,0xFF,sizeof(pData->aColors)/2);
	memset(pData->aColors           ,0xFF,sizeof(pData->aColors)/2);
	memset(pData->cColorChanged+4,1,CLB_MAX_COLOR-4);


	if(CLB_MAX_COLOR>=16)
		{
		pData->aColors[ 4] = RGB(  0,  0,  0);
		pData->aColors[ 5] = RGB(  0,  0,128);
		pData->aColors[ 6] = RGB(  0,128,  0);
		pData->aColors[ 7] = RGB(  0,128,128);
		pData->aColors[ 8] = RGB(128,  0,  0);
		pData->aColors[ 9] = RGB(128,  0,128);
		pData->aColors[10] = RGB(128,128,  0);
		pData->aColors[11] = RGB(128,128,128);
		pData->aColors[12] = RGB(192,192,192);
		pData->aColors[13] = RGB(  0,  0,255);
		pData->aColors[14] = RGB(  0,255,  0);
		pData->aColors[15] = RGB(  0,255,255);
		pData->aColors[16] = RGB(255,  0,  0);
		pData->aColors[17] = RGB(255,  0,255);
		pData->aColors[18] = RGB(255,255,  0);
		pData->aColors[19] = RGB(255,128,128);

		memset(pData->cColorChanged+4,1,16);
		}


}

//*****************************************************************************
//*
//*		SaveMark
//*
//*****************************************************************************
//	Speichert die aktuelle Marken Position
//	pData		: Zeiger auf die Fensterdaten
static inline void SaveMark(ListBoxData *pData)
{

	pData->iLastStartX = pData->iMarkStartX;
	pData->iLastStartY = pData->iMarkStartY;
	pData->iLastStopX  = pData->iMarkStopX;
	pData->iLastStopY  = pData->iMarkStopY;
	pData->iLastMode   = pData->iMarkMode;

}

//*****************************************************************************
//*
//*		SetClipboardText
//*
//*****************************************************************************
//	Speichert einen Text in die Zwischenablage
//	pText	: Ist der Zeiger auf den Text
//	iSize	: Ist die maximale Lnge vom Text
//			  Wenn iSize kleine Null is wird die Lnge mit strlen bestimmt.
//	Ergibt die Lnge des gespeicherten Textes bzw. 0 wenn kein Text gespeichert wurde.
static int SetClipboardText(LPCTSTR pText,int iSize)
{
TCHAR	   *pNew;
int			iOk=0;
HGLOBAL		hGlobal;


	if(iSize< 0)iSize=(int)_tcslen(pText);
	if(iSize<=0)return 0;

	hGlobal=GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,(iSize+1)*sizeof(pText[0]));
	pNew=(TCHAR*)GlobalLock(hGlobal);
	memcpy(pNew,pText,iSize*sizeof(TCHAR));
	pNew[iSize]=0;

	GlobalUnlock(hGlobal);

	if(OpenClipboard(0))
		{
		EmptyClipboard();
		if(!SetClipboardData(CF_TEXT_EX,hGlobal))GlobalFree(hGlobal);
		CloseClipboard();
		iOk=iSize;
		}
	else{
		GlobalFree(hGlobal);
		}


return iOk;
}

//*****************************************************************************
//*
//*		IsChar
//*
//*****************************************************************************
//	Prft ob ein Zeichen ein Buchstabe ist
static int IsChar(TCHAR cChar)
{
unsigned	uVal;

	if(sizeof(TCHAR)==1)
			uVal = (unsigned char)cChar;
	else	uVal = cChar;

	if(uVal>='a')
		{
		if(uVal<='z')return 1;
		if(uVal>=192)return 1;
		return 1;
		}
	else{
		if(uVal=='_')return 1;
		if(uVal> 'Z')return 0;
		if(uVal>='A')return 1;
		if(uVal< '0')return 0;
		if(uVal> '9')return 0;
		}

return 1;
}

//*****************************************************************************
//*
//*		IsSign
//*
//*****************************************************************************
//	Prft ob ein Zeichen ein Spezialzeichen ist
static int IsSign(TCHAR cChar)
{
unsigned	uVal;

	if(sizeof(TCHAR)==1)
			uVal = (unsigned char)cChar;
	else	uVal = cChar;

	if(uVal<=' ')return 0;

	if(uVal>='a')
		{
		if(uVal<='z')return 0;
		if(uVal>=192)return 0;
		return 1;
		}
	else{
		if(uVal=='_')return 0;
		if(uVal> 'Z')return 1;
		if(uVal>='A')return 0;
		if(uVal< '0')return 1;
		if(uVal> '9')return 1;
		}

return 0;
}

//*****************************************************************************
//*
//*		CharType
//*
//*****************************************************************************
//	Holt den Typ von einem Zeichen
//	cChar		; Ist das Zeichen das analysiert werden soll
//	Ergibt den Typ des Zeichens
//				0 = Leerzeichen
//				1 = Diverse Zeichen
//				2 = Das '_' Zeichen
//				3 = Kleinbuchstaben oder Zahlen
//				4 = Grosbuchstaben
static inline int CharType(TCHAR cChar)
{
unsigned	uVal;


	if(sizeof(TCHAR)==1)
			uVal = (unsigned char)cChar;
	else	uVal = cChar;

	if(uVal>'Z')
		{
		if(uVal< 'a')
			{
			if(uVal=='_')return 2;
			return 1;
			}

		if(uVal<='z')return 3;
		if(uVal>=221)return 3;
		if(uVal>=192)return 4;
		return 0;
		}
	else{
		if(uVal>='A')return 4;
		if(uVal<=' ')return 0;
		if(uVal< '0')return 1;
		if(uVal> '9')return 1;
		}


return 3;
}

//*****************************************************************************
//*
//*		GetDataEntry
//*
//*****************************************************************************
//	Holt die Daten von einen einzelnen Spalteneintrag
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	pEntry		: Hier werden die Daten gsepeichert
static inline void GetDataEntry(ListBoxData *pData,unsigned uLine,CLB_ENTRY *pEntry)
{
BaseItem   *pTemp;



	pTemp = pData->pListItems+uLine;

	if(pData->pLineCallback)							// Texte ber Callback holen
		{
		pEntry->iIdent	= pTemp->wIdent;
		pEntry->iSize	= sizeof(pData->cTempText)/sizeof(TCHAR)-1;
		pEntry->pText	= pData->cTempText;
		pEntry->pColor	= pData->aTempColor;
		pData->pLineCallback(pData->hWnd,pTemp->lParam,pEntry,pData->pLineCbParam);
		}
	else{
		pEntry->iIdent	= pTemp->wIdent;
		pEntry->iSize	= pTemp->wLineLen;
		pEntry->pText	= (TCHAR*)pTemp->pTextData;
		pEntry->pColor	= pTemp->pTextData+pTemp->wLineSize*sizeof(TCHAR);
		}

}

//*****************************************************************************
//*
//*		GetDataEntry
//*
//*****************************************************************************
//	Holt die Daten von einen einzelnen Spalteneintrag
//	pData		: Zeiger auf die Fensterdaten
//	pTemp		: Ist der Zeiger auf den Zeileneintrag
//	pEntry		: Hier werden die Daten gsepeichert
static inline void GetDataEntry(ListBoxData *pData,BaseItem *pTemp,CLB_ENTRY *pEntry)
{

	if(pData->pLineCallback)							// Texte ber Callback holen
		{
		pEntry->iIdent	= pTemp->wIdent;
		pEntry->iSize	= sizeof(pData->cTempText)/sizeof(TCHAR)-1;
		pEntry->pText	= pData->cTempText;
		pEntry->pColor	= pData->aTempColor;
		pData->pLineCallback(pData->hWnd,pTemp->lParam,pEntry,pData->pLineCbParam);
		}
	else{
		pEntry->iIdent	= pTemp->wIdent;
		pEntry->iSize	= pTemp->wLineLen;
		pEntry->pText	= (TCHAR*)pTemp->pTextData;
		pEntry->pColor	= pTemp->pTextData+pTemp->wLineSize*sizeof(TCHAR);
		}

}

//*****************************************************************************
//*
//*		PointOfRow
//*
//*****************************************************************************
//	Holt die X-Koordinate von einem Zeichen in einer Zeile
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer der Zeile
//	iPos		: Buchstaben-Position in der Zeile
//	Ergibt die X-Koordinate auf der der Buchstaben beginnt.
static int PointOfRow(ListBoxData *pData,unsigned uLine,int iPos=0x7FFFFFFF,HDC hDc=NULL)
{
CLB_ENTRY	sData;
DWORD		uSize;
HDC			hFree;



	if(uLine>=pData->uListItemsCount)						// Gibt es die Zeile
		{
		return 0;
		}

	GetDataEntry(pData,uLine,&sData);
	
	if(iPos<=0)												// Vor dem ersten Buchstaben
		{
		return sData.iIdent;
		}

	if(sData.iSize<=0)										// Keine Buchstaben in der Zeile
		{
		return 0;
		}

	if(iPos>sData.iSize)									// Offset im Text prfen
		{
		iPos=sData.iSize;
		}

	
	if(!hDc)
		{
		hDc   = GetDC(pData->hWnd);
		hFree = hDc;
		}
	else{
		hFree = NULL;
		}

	
	if(sData.iIdent || (pData->uStyleEx&CLB_EX_MULITFONT))	// Werden mehrere Fonts benutzt
		{
		uSize = GetTabbedSizeEx(hDc,sData.iIdent,sData.pText,sData.pColor,iPos,pData->iTabCount,pData->aTabPos,0,pData->aFonts);
		}
	else{
		uSize = GetTabbedTextExtent(hDc,sData.pText,iPos,pData->iTabCount,pData->aTabPos);
		}


	if(hFree)												// DC freigeben
		{
		ReleaseDC(pData->hWnd,hFree);
		}


return (uSize&0xFFFF)+sData.iIdent;
}

//*****************************************************************************
//*
//*		RowOfPoint
//*
//*****************************************************************************
//	Holt die Textspalte von einem Punkt in einer Zeile
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer der Zeile
//	iPosX		: Pixel-Position in der Zeile
//	iAddHalf	: Eine Position weiter wenn ber die Buchstabenhlfte
//	Ergibt die Textspalte auf die der Punkt zeigt.
static int RowOfPoint(ListBoxData *pData,unsigned uLine,int iPosX,int iAddHalf=0)
{
CLB_ENTRY	sData;
SIZE		sSize;
BYTE		bLast;
BYTE		bMask;
UINT		uFont;
INT			iRow;
INT			iPos;
INT			iEnd;
INT			iOld;
INT			iOff;
INT			iTab;
HDC			hDc;



	if(uLine>=pData->uListItemsCount)						// Gibt es die Zeile
		{
		return 0;
		}


	GetDataEntry(pData,uLine,&sData);


	iPos	= 0;
	iOff	= 0;
	iTab	= 0;
	iPosX  += pData->uScrollX;
	iPosX  -= sData.iIdent; 
	hDc     = GetDC(pData->hWnd);
	bMask   = (pData->uStyleEx&CLB_EX_MULITFONT)? 0xC0:0x00;
	uFont   = 0;
	bLast   = 0;

	if(iPosX<0)iPosX=0;

	SelectObject(hDc,pData->aFonts[0]);						// Schrift auswhlen

	for(;;)
		{
		if(iPos<sData.iSize)
			{
			bLast = sData.pColor[iPos];
			}

		for(iEnd=iPos;iEnd<sData.iSize;iEnd++)				// Suche nchstes TAB
			{
			if( sData.pText [iEnd]=='\t')break;
			if((sData.pColor[iEnd]^bLast)&bMask)break;
			}

		iRow = iEnd-iPos;

		if((UINT)((bLast&bMask)>>6)!=uFont)					// Andere Schrift whlen
			{
			uFont = bLast>>6;
			SelectObject(hDc,pData->aFonts[uFont]);
			}

		if(iRow>0)
			{
			GetTextExtentExPoint(hDc,sData.pText+iPos,iRow,iPosX-iOff,&iRow,NULL,&sSize);
			iRow += iPos;
			}
		else{
			iRow     = iPos;
			sSize.cx = 0;
			}

		if(iRow<iEnd)										// X-Koordinate berschritten
			{
			if(!iAddHalf)break;

			iOff = iPosX-PointOfRow(pData,uLine,iRow,hDc)+sData.iIdent;
			GetTextExtentExPoint(hDc,sData.pText+iRow,1,1000,NULL,NULL,&sSize);

			if(iOff*2>=sSize.cx)iRow++;
			break;
			}

		if(iEnd>=sData.iSize)break;
		

		iOff += sSize.cx;
		iOld  = iOff;
															// Ist ein Tab am Ende
		if(sData.pText[iEnd]=='\t')
			{
			for(;;iTab++)
				{
				if(iTab>=pData->iTabCount)					// Fixe Tabulatorabstnde
					{
					iOff += pData->iTabSize;
					iOff /= pData->iTabSize;
					iOff *= pData->iTabSize;
					break;
					}

				if(iOff<pData->aTabPos[iTab])
					{
					iOff = pData->aTabPos[iTab];
					break;
					}
				}

			iEnd += 1;
			}

		if(iPosX<iOff)										// X-Koordinate berschritten
			{
			if(!iAddHalf)break;
			if((iPosX-iOld)*2>=iOff-iOld)iRow++;
			break;
			}

		if(iEnd>=sData.iSize)break;

		iPos = iEnd;
		}


	ReleaseDC(pData->hWnd,hDc);



return iRow;
}


//*****************************************************************************
//*
//*		RowOfNewLine
//*
//*****************************************************************************
//	Berechnet die Reihe in einer neuen Zeile auf der gleichen X-Koordinate.
//	pData		: Zeiger auf die Fensterdaten
//	uLineNew	: Neue Zeilenposition
//	uLineNow	: Aktuelle Zeilenposition
//	iPosNow		: Aktuelle Reihe in der Zeile
//	Ergibt die Reihe in der neuen Zeile
static int RowOfNewLine(ListBoxData *pData,unsigned uLineNew,unsigned uLineNow,int iPosNow)
{
int		iPos;
int		iRow;


	if(uLineNew>=pData->uListItemsCount)return 0;
	if(uLineNow>=pData->uListItemsCount)return 0;

	iPos = PointOfRow(pData,uLineNow,iPosNow);
	iRow = RowOfPoint(pData,uLineNew,iPos+pData->iFontWidth/4);


return iRow;
}


//*****************************************************************************
//*
//*		SendNotify
//*
//*****************************************************************************
//	Sendet eine WM_NOTIFY Nachricht an das Elternfenster
//	pData		: Zeiger auf die Fensterdaten
//	pNotify		: Zeiger auf die Notify-Daten
//	Ergibt den Rckgabewert der WM_NOTIFY Nachrich
static LRESULT SendNotify(ListBoxData *pData,NMHDR	*pNotify)
{
LRESULT		lRet;


	UNLOCK(pData);

	pNotify->hwndFrom	= pData->hWnd;
    pNotify->idFrom		= GetWindowLong(pNotify->hwndFrom,GWL_ID);

	lRet = 	SendMessage(GetParent(pNotify->hwndFrom),WM_NOTIFY,pNotify->idFrom,(LPARAM)pNotify);

	LOCK(pData);


return lRet;
}


//*****************************************************************************
//*
//*		UpdateStart
//*
//*****************************************************************************
//	Startet ein 200ms verzgertes Update
//	pData		: Zeiger auf die Fensterdaten
static void UpdateStart(ListBoxData *pData)
{

	if(pData->cUpdateTimer)return;

	pData->cUpdateTimer = 1;

	if(!PostMessage(pData->hWnd,WM_COMMAND,ID_UPDATE,0))
		{
		Sleep(10);
		PostMessage(pData->hWnd,WM_COMMAND,ID_UPDATE,0);
		}

}

//*****************************************************************************
//*
//*		UpdateColors
//*
//*****************************************************************************
//	Aktualisiert alle Farben
//	pData		: Zeiger auf die Fensterdaten
static void UpdateColors(ListBoxData *pData)
{
unsigned	uColOdd;
unsigned 	uColor;
unsigned	uColEx;
int 		iDiff;
int 		iSum;



	if(!pData->cColorChanged[CLB_COLOR_FRAME_ODD])
		{
		pData->aColors[CLB_COLOR_FRAME_ODD] = GetSysColor(COLOR_3DFACE);
		}

	if(!pData->cColorChanged[CLB_COLOR_FRAME_EVEN])
		{
		pData->aColors[CLB_COLOR_FRAME_EVEN] = GetSysColor(COLOR_3DFACE);
		}

	if(!pData->cColorChanged[CLB_COLOR_EVENWND])
		{
		pData->aColors[CLB_COLOR_EVENWND]=GetSysColor(COLOR_WINDOW);
		}

	if(!pData->cColorChanged[CLB_COLOR_ODDWND])
		{
		uColOdd = GetSysColor(COLOR_INFOBK);
		iDiff   = ((uColOdd    )&0xFF)-((pData->aColors[CLB_COLOR_EVENWND]    )&0xFF); iSum  = iDiff*iDiff;
		iDiff   = ((uColOdd>> 8)&0xFF)-((pData->aColors[CLB_COLOR_EVENWND]>> 8)&0xFF); iSum += iDiff*iDiff;
		iDiff   = ((uColOdd>>16)&0xFF)-((pData->aColors[CLB_COLOR_EVENWND]>>16)&0xFF); iSum += iDiff*iDiff;

		if(iSum<64)												// Ist die alternierente Farbe fast gleich ?
			{
			uColOdd  =   pData->aColors[CLB_COLOR_EVENWND]&0x0000FFFF;
			uColOdd |= ((pData->aColors[CLB_COLOR_EVENWND]&0x00FF0000)-0x00080000)&0x00FF0000;
			}

		pData->aColors[CLB_COLOR_ODDWND] = uColOdd;
		}

	if(!pData->cColorChanged[CLB_COLOR_ODDSEL])
		{
		uColEx = GetSysColor(COLOR_HIGHLIGHT);
		uColor = pData->aColors[CLB_COLOR_ODDWND];
		uColor = ((uColor>>2)&0x3F3F3F)*2;
		uColor+= ((uColEx>>2)&0x3F3F3F)*2;

		pData->aColors[CLB_COLOR_ODDSEL] = uColor;
		}

	if(!pData->cColorChanged[CLB_COLOR_EVENSEL])
		{
		uColEx = GetSysColor(COLOR_HIGHLIGHT);
		uColor = pData->aColors[CLB_COLOR_EVENWND];
		uColor = ((uColor>>2)&0x3F3F3F)*2;
		uColor+= ((uColEx>>2)&0x3F3F3F)*2;

		pData->aColors[CLB_COLOR_EVENSEL] = uColor;
		}

	if(!pData->cColorChanged[CLB_COLOR_ODDFOCUS])
		{
		uColEx = GetSysColor(COLOR_HIGHLIGHT);
		uColor = pData->aColors[CLB_COLOR_ODDWND];
		uColor = ((uColor>>2)&0x3F3F3F)*3;
		uColor+= ((uColEx>>2)&0x3F3F3F);

		pData->aColors[CLB_COLOR_ODDFOCUS] = uColor;
		}

	if(!pData->cColorChanged[CLB_COLOR_EVENFOCUS])
		{
		uColEx = GetSysColor(COLOR_HIGHLIGHT);
		uColor = pData->aColors[CLB_COLOR_EVENWND];
		uColor = ((uColor>>2)&0x3F3F3F)*3;
		uColor+= ((uColEx>>2)&0x3F3F3F);

		pData->aColors[CLB_COLOR_EVENFOCUS] = uColor;
		}

	if(!pData->cColorChanged[MAX_COLORS+3])pData->aColors[MAX_COLORS+3] = GetSysColor(COLOR_HIGHLIGHT);
	if(!pData->cColorChanged[0           ])pData->aColors[0           ] = GetSysColor(COLOR_WINDOWTEXT);
	if(!pData->cColorChanged[1           ])pData->aColors[1           ] = GetSysColor(COLOR_WINDOWTEXT)|0xFF000000;
	if(!pData->cColorChanged[2           ])pData->aColors[2           ] = GetSysColor(COLOR_WINDOWTEXT)|0xFF000000;
	if(!pData->cColorChanged[3           ])pData->aColors[3           ] = GetSysColor(COLOR_HIGHLIGHTTEXT);

	if(pData->cUseTheme==0 && pOpenThemeData)			// ber Thema zeichnen
		{
		if(!pData->hTheme)
			{
			pData->hTheme = pOpenThemeData(pData->hWnd,L"BUTTON");
			}

		if(pData->uStyleEx&CLB_EX_NOTHEMES)
			{
			pData->cUseTheme = 1;
			}
		else{
			pData->cUseTheme = (pData->hTheme)? 2:1;
			}
		}


}

//*****************************************************************************
//*
//*		UpdateCaret
//*
//*****************************************************************************
//	Aktualisiert den Blink-Cursor
//	pData		: Zeiger auf die Fensterdaten
static void UpdateCaret(ListBoxData *pData)
{
int		iPosX;
int		iPosY;


	if(pData->cHasCaret && pData->cHasFocus)
		{
		pData->iCaretX  = pData->uStartText;
		pData->iCaretX += PointOfRow(pData,pData->iCursorY,pData->iCursorX);
		pData->iCaretY  = pData->iCursorY *pData->iRowHeight;

		if(!pData->iCaretExist)
			{
			pData->iCaretExist = 1;
			CreateCaret(pData->hWnd,NULL,2,pData->iFontHeight-1);
			}

		if(pData->iCaretX>=(int)pData->uScrollX && pData->iCaretX<(int)(pData->uScrollX+pData->uSizeX))
			{
			iPosX = pData->iCaretX-pData->uScrollX;
			iPosY = pData->iCaretY-pData->uScrollY*pData->iRowHeight+pData->iRowPre;

			SetCaretPos(iPosX,iPosY+1);

			if(!pData->iCaretOn)
				{
				pData->iCaretOn=1;
				ShowCaret(pData->hWnd);
				}

			return;
			}
		}

	if(pData->iCaretOn)
		{
		pData->iCaretOn=0;
		HideCaret(pData->hWnd);
		}

}

//*****************************************************************************
//*
//*		UpdateHeight
//*
//*****************************************************************************
//	Prft ob die Zeilenhhe sich gendert hat
//	pData		: Zeiger auf die Fensterdaten
//	Ergibt 1 bei nderungen sonst 0
static int UpdateHeight(ListBoxData *pData)
{
int		iHeight;
int		iFrame;
UINT	uRows;
RECT	sRect;


	iHeight = 10;
	iFrame	= 0;

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)
		{
		if(pData->hStates)
			{
			if(iHeight<pData->iStatesYsize+pData->cStateFrame)
				{
				iHeight = pData->iStatesYsize+pData->cStateFrame;
				}
			}
		else{
			if(iHeight<15 && pData->cUseTheme==2)iHeight=15;
			if(iHeight<13)iHeight=13;
			}
		}

	if(pData->hIcons)
		{
		if(iHeight<pData->iIconsYsize+pData->cIconFrame)
			{
			iHeight = pData->iIconsYsize+pData->cIconFrame;
			}
		}

	if(iHeight<pData->iFontHeight)
		{
		iHeight = pData->iFontHeight;
		}

	if(pData->uStyleEx&CLB_EX_GRIDLINES)
		{
		if(pData->iFontHeight==iHeight)iHeight++;
		iHeight++;
		iFrame=1;
		}

	if(pData->iRowHeight==iHeight)
		{
		uRows	= pData->uSizeY;
		uRows  -= pData->uStartPixel;
		uRows  += pData->iRowHeight-1;
		uRows  /= pData->iRowHeight;

		if(uRows==pData->uMaxEnties)
			{
			return 0;
			}
		}		

	if(pData->iFontOff && pData->uStartText<=1)
		{
		pData->uStartText = 1;
		}
	else if(pData->uStartText==1)
		{
		pData->uStartText = 0;
		}


	pData->iRowHeight	 =  iHeight;
	pData->uMaxEnties	 =  pData->uSizeY;
	pData->uMaxEnties	-=  pData->uStartPixel;
	pData->uPageEnties	 =  pData->uMaxEnties;
	pData->uMaxEnties	+=  pData->iRowHeight-1;
	pData->uMaxEnties   /=  pData->iRowHeight;
	pData->uPageEnties  /=  pData->iRowHeight;
	pData->iRowPre		 = (pData->iRowHeight-pData->iFontHeight-iFrame)/2;
	pData->iRowPost		 = (pData->iRowHeight-pData->iFontHeight-iFrame-pData->iRowPre);

	GetClientRect (pData->hWnd,&sRect);
	InvalidateRect(pData->hWnd,&sRect,FALSE);


return 1;
}

//*****************************************************************************
//*
//*		UpdateRect
//*
//*****************************************************************************
//	Zeichnet einen Text eines Eintrags neu
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	uStart		: Ist der Start-Buchstabe
//	uEnd		: Ist der End-Buchstabe
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateRect(ListBoxData *pData,unsigned uLine,unsigned uStart,unsigned uEnd=0x7FFF)
{
RECT		sRect;
unsigned	uPos;



	if(uLine>=pData->uListItemsCount)return 0;				// Ist der Eintrag gltig

	   uPos = uLine-pData->uScrollY;
	if(uPos>=pData->uMaxEnties)return 0;					// Eintrag im Fenster sichtbar

	sRect.left    = 0;
//	sRect.left   -= pData->uScrollX;
	sRect.right	  = pData->uSizeX;
//	sRect.right  -= pData->uScrollX;
	sRect.top     = pData->uStartPixel;
	sRect.top    += pData->iRowHeight*uPos;
	sRect.bottom  = pData->iRowHeight+sRect.top;

	InvalidateRect(pData->hWnd,&sRect,FALSE);



return 1;
}

//*****************************************************************************
//*
//*		UpdateBox
//*
//*****************************************************************************
//	Zeichnet die Checkbox eines Eintrags neu
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateBox(ListBoxData *pData,unsigned uLine)
{
RECT		sRect;
unsigned	uFrom;
unsigned	uEnd;
unsigned	uPos;


	if(uLine>=pData->uListItemsCount)return 0;				// Ist der Eintrag gltig

	   uPos = uLine-pData->uScrollY;
	if(uPos>= pData->uMaxEnties)return 0;					// Eintrag im Fenster sichtbar

	if(pData->hIcons)
		{
		uEnd  = pData->iIconsXsize+SELECT_SIZE;
		uFrom = pData->iIconsXsize;
		}
	else{
		uEnd  = SELECT_SIZE;
		uFrom = 0;
		}

	if(pData->uScrollX>=uEnd)return 0;

	sRect.left	  = uFrom;
	sRect.right	  = uEnd;
	sRect.top     = pData->uStartPixel;
	sRect.top    += pData->iRowHeight*uPos;
	sRect.bottom  = pData->iRowHeight+sRect.top;

	InvalidateRect(pData->hWnd,&sRect,FALSE);



return 1;
}

//*****************************************************************************
//*
//*		UpdateIcon
//*
//*****************************************************************************
//	Zeichnet die Checkbox eines Eintrags neu
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateIcon(ListBoxData *pData,unsigned uLine)
{
RECT		sRect;
unsigned	uPos;



	if(uLine>=pData->uListItemsCount)return 0;				// Ist der Eintrag gltig

	   uPos = uLine-pData->uScrollY;
	if(uPos>= pData->uMaxEnties              )return 0;		// Eintrag im Fenster sichtbar
	if(pData->uScrollX>=U(pData->iIconsXsize))return 0;

	sRect.left    = 0;
	sRect.right	  = pData->iIconsXsize;
	sRect.top     = pData->uStartPixel;
	sRect.top    += pData->iRowHeight*uPos;
	sRect.bottom  = pData->iRowHeight+sRect.top;


	InvalidateRect(pData->hWnd,&sRect,FALSE);



return 1;
}


//*****************************************************************************
//*
//*		UpdateRows
//*
//*****************************************************************************
//	Zeichnet einen Eintrag ber die ganze Zeile neu
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateRows(ListBoxData *pData,unsigned uLine,unsigned uCount=1)
{
RECT		sRect;
unsigned	uNum;
unsigned	uPos;



	if(uLine>=pData->uListItemsCount+pData->uMaxEnties/2)	// Ist der Eintrag gltig
		{
		return 0;
		}

	if(uLine<pData->uScrollY)								// Vor erster sichtbarer Zeile
		{
		   uNum = pData->uScrollY-uLine;
		if(uNum>=uCount)return 0;

		uCount -= uNum;
		uLine   = pData->uScrollY;
		uPos	= 0;
		}
	else{
		   uPos = uLine-pData->uScrollY;
		if(uPos>=pData->uMaxEnties)return 0;				// Eintrag im Fenster sichtbar
		}

	if(uLine+uCount>pData->uListItemsCount+pData->uMaxEnties/2)
		{
		uCount  = pData->uListItemsCount+pData->uMaxEnties/2-uLine;
		}

	if(uPos+uCount>pData->uMaxEnties)
		{
		uCount  = pData->uMaxEnties-uPos;
		}

	if(uCount<=0)return 0;

	sRect.left    = 0;
	sRect.right   = pData->uSizeX;
	sRect.top     = pData->uStartPixel;
	sRect.top    += pData->iRowHeight*uPos;
	sRect.bottom  = pData->iRowHeight*uCount+sRect.top;
	InvalidateRect(pData->hWnd,&sRect,FALSE);



return 1;
}

//*****************************************************************************
//*
//*		UpdateMark
//*
//*****************************************************************************
//	Zeichnet die nderungen der Marke neu. Die vorherige Position der Marke
//	muss vorher mit SaveMark(...) gespeichert worden sein.
//	pData		: Zeiger auf die Fensterdaten
static void UpdateMark(ListBoxData *pData)
{
int		iPos1;
int		iPos2;
int		iPos3;
int		iPos4;
int		iPosY;

															// nderungen auf einer Zeile
	if(pData->iMarkMode==0 || pData->iMarkStartY==pData->iMarkStopY)
	if(pData->iLastMode==0 || pData->iLastStartY==pData->iLastStopY)
		{
		iPosY = pData->iMarkStartY;

		if(pData->iMarkMode)
			{
			if(pData->iMarkStartX<=pData->iMarkStopX)
				{
				iPos1 = pData->iMarkStartX;
				iPos2 = pData->iMarkStopX;
				}
			else{
				iPos2 = pData->iMarkStartX;
				iPos1 = pData->iMarkStopX;
				}
			}
		else{
			iPos1 = pData->iLastStartX;
			iPos2 = pData->iLastStartX;
			}

		if(pData->iLastMode)
			{
			iPosY = pData->iLastStartY;

			if(pData->iLastStartX<=pData->iLastStopX)
				{
				iPos3 = pData->iLastStartX;
				iPos4 = pData->iLastStopX;
				}
			else{
				iPos4 = pData->iLastStartX;
				iPos3 = pData->iLastStopX;
				}
			}
		else{
			iPos3 = pData->iMarkStartX;
			iPos4 = pData->iMarkStartX;
			}

		     if(iPos1<iPos3)UpdateRect(pData,iPosY,iPos1,iPos3);
		else if(iPos1>iPos3)UpdateRect(pData,iPosY,iPos3,iPos1);

		     if(iPos2<iPos4)UpdateRect(pData,iPosY,iPos2,iPos4);
		else if(iPos2>iPos4)UpdateRect(pData,iPosY,iPos4,iPos2);

		return;
		}


	if(pData->iMarkStartX==pData->iLastStartX)				// Ist die Startposition gleich
	if(pData->iMarkStartY==pData->iLastStartY)
	if(pData->iMarkMode  ==pData->iLastMode  )
		{
		if(pData->iMarkStopY==pData->iLastStopY)			// nderung in X
			{
			if(pData->iMarkStopX<pData->iLastStopX)
				{
				UpdateRect(pData,pData->iMarkStopY,pData->iMarkStopX,pData->iLastStopX);
				}
			else if(pData->iMarkStopX>pData->iLastStopX)
				{
				UpdateRect(pData,pData->iMarkStopY,pData->iLastStopX,pData->iMarkStopX);
				}
			}
		else if(pData->iMarkStopY!=pData->iLastStopY)		// nderung in Y
			{
			if(pData->iMarkStopY<pData->iLastStopY)
				{
				UpdateRows(pData,pData->iMarkStopY,pData->iLastStopY-pData->iMarkStopY+1);
				}
			else{
				UpdateRows(pData,pData->iLastStopY,pData->iMarkStopY-pData->iLastStopY+1);
				}
			}

		return;
		}



	if(pData->iMarkMode)									// Groflchige nderung
		{
		if(pData->iMarkStartY<pData->iMarkStopY)
			{
			iPos1 = pData->iMarkStartY;
			iPos2 = pData->iMarkStopY;
			}
		else{
			iPos2 = pData->iMarkStartY;
			iPos1 = pData->iMarkStopY;
			}
		}
	else{
		iPos1 = pData->iLastStartY;
		iPos2 = pData->iLastStartY;
		}

	if(pData->iLastMode)
		{
		if(pData->iLastStartY<pData->iLastStopY)
			{
			if(pData->iLastStartY<iPos1)iPos1=pData->iLastStartY;
			if(pData->iLastStopY >iPos2)iPos2=pData->iLastStopY;
			}
		else{
			if(pData->iLastStartY>iPos2)iPos2=pData->iLastStartY;
			if(pData->iLastStopY <iPos1)iPos1=pData->iLastStopY;
			}
		}


	UpdateRows(pData,iPos1,iPos2-iPos1+1);


}


//*****************************************************************************
//*
//*		UpdateView
//*
//*****************************************************************************
//	Zeichnet das ganze Fenster neu
//	pData		: Zeiger auf die Fensterdaten
//	Ergibt 1 wenn der Eintrag sichtbar war
static void UpdateView(ListBoxData *pData)
{
RECT	sRect;


	GetClientRect (pData->hWnd,&sRect);
	sRect.top =    pData->uStartPixel;
	InvalidateRect(pData->hWnd,&sRect,FALSE);

}

//*****************************************************************************
//*
//*		UpdateScrollX
//*
//*****************************************************************************
//	Aktualisiert die X-Scroolbar
//	pData		: Zeiger auf die Fensterdaten
//	Ergibt 1 wenn der sich Einstellungen verndert haben
static void UpdateScrollX(ListBoxData *pData)
{
SCROLLINFO  sInfo;
unsigned	uSize;



	if(pData->uStyle&CLBS_NOSCROLLX)
		{
		uSize  = 0;
		}
	else{
		uSize  = pData->iMaxSizeX+10;
		uSize += pData->uStartText;
		}

	if(pData->uOldXCount==uSize)
	if(pData->uOldXPage ==pData->uSizeX)
		{
		return;
		}

	pData->uOldXPage	= pData->uSizeX;
	pData->uOldXCount	= uSize;

	UNLOCK(pData);

	sInfo.cbSize	= sizeof(SCROLLINFO);
	sInfo.fMask		= SIF_ALL;
	sInfo.nMin		= 0;
	sInfo.nMax  	= uSize;
	sInfo.nPage 	= pData->uSizeX;
	sInfo.nPos  	= pData->uScrollX;
	sInfo.nTrackPos	= 0;



	if((int)sInfo.nPage>=sInfo.nMax && pData->uScrollX>0)
		{
		sInfo.nPos      = 0;
		pData->uScrollX = 0;

		UpdateView(pData);
		}

	SetScrollInfo(pData->hWnd,SB_HORZ,&sInfo,TRUE);


	LOCK(pData);

}

//*****************************************************************************
//*
//*		UpdateScrollY
//*
//*****************************************************************************
//	Aktualisiert die Y-Scroolbar
//	pData		: Zeiger auf die Fensterdaten
//	bForce		: Das Fenster immer neu Zeichen
//	Ergibt 1 wenn der sich Einstellungen verndert haben
static void UpdateScrollY(ListBoxData *pData,BOOL bForce=FALSE)
{
SCROLLINFO  sInfo;
unsigned	uTick;
unsigned	uDiff;
BOOL		bDraw;
int			iMax;



	if(pData->uOldYCount==pData->uListItemsCount)
	if(pData->uOldYPage ==pData->uPageEnties    )
		{
		return;
		}

	pData->uOldYPage	= pData->uPageEnties;
	pData->uOldYCount	= pData->uListItemsCount;

	UNLOCK(pData);

	sInfo.cbSize	= sizeof(SCROLLINFO);
	sInfo.fMask		= SIF_ALL;
	sInfo.nMin		= 0;
	sInfo.nMax  	= pData->uListItemsCount;
	sInfo.nPage 	= pData->uPageEnties;
	sInfo.nPos  	= pData->uScrollY;
	sInfo.nTrackPos	= 0;

	if(pData->uStyle&CLBS_NOSCROLLY)
		{
		sInfo.nMax = 0;
		}


	uTick = GetTickCount();

	if(bForce)												// Immer neu zeichen
		{
		bDraw = TRUE;
		}
	else{
		   uDiff = uTick-pData->uLastScrolY;
		if(uDiff>200)										// Nur alle 200 ms neuzeichnen
			{
			bDraw				= TRUE;
			pData->uLastScrolY	= uTick;
			}
		else{
			bDraw = FALSE;
			}
		}


	iMax  = pData->uListItemsCount;
	iMax -=	pData->uPageEnties/2-1;

	if(sInfo.nPos>=iMax)sInfo.nPos=iMax;
	if(sInfo.nPos<0)sInfo.nPos=0;
	if(sInfo.nPos!=(int)pData->uScrollY)
		{
		pData->uScrollY = sInfo.nPos;

		if((int)sInfo.nPage>=sInfo.nMax && pData->uScrollY>0)
			{
			sInfo.nPos      = 0;
			pData->uScrollY = 0;
			}

		if(bDraw)
			{
			UpdateView (pData);
			}
		else{
			pData->cUpdateView = 1;
			UpdateStart(pData);
			}
		}


	if((int)sInfo.nPage>=sInfo.nMax && pData->uScrollY>0)
		{
		sInfo.nPos      = 0;
		pData->uScrollY = 0;

		if(bDraw)
			{
			UpdateView (pData);
			}
		else{
			pData->cUpdateView = 1;
			UpdateStart(pData);
			}
		}


	SetScrollInfo(pData->hWnd,SB_VERT,&sInfo,bDraw);

	if(bDraw)
		{
		pData->uLastScrolY	= uTick;
		}
	else{
		pData->cUpdateScrollY = 1;
		UpdateStart(pData);
		}


	LOCK(pData);

}

//*****************************************************************************
//*
//*		UpdateScrollBars
//*
//*****************************************************************************
//	Die Scroolbars neu Zeichen
//	pData		: Zeiger auf die Fensterdaten
static void UpdateScrollBars(ListBoxData *pData)
{
SCROLLINFO  sInfo;


	GetScrollInfo(pData->hWnd,SB_VERT,&sInfo);
	SetScrollInfo(pData->hWnd,SB_VERT,&sInfo,TRUE);

	GetScrollInfo(pData->hWnd,SB_HORZ,&sInfo);
	SetScrollInfo(pData->hWnd,SB_HORZ,&sInfo,TRUE);

}


//*****************************************************************************
//*
//*		UpdateTextSize
//*
//*****************************************************************************
//	Atualisiert die Textbreiten
//	pData			: Zeiger auf die Fensterdaten
static void UpdateTextSize(ListBoxData *pData)
{
BaseItem   *pEntry;
CLB_ENTRY	sData;
unsigned	uSize;
unsigned	uLine;
unsigned 	uMax;
HDC			hDc;



	uMax   = 0;
	hDc    = GetDC(pData->hWnd);
	pEntry = pData->pListItems;
	SelectObject(hDc,pData->aFonts[0]);

	if(pData->pLineCallback)
		{
		for(uLine=pData->uListItemsCount;uLine>0;pEntry++)
			{
			uLine -= 1;

			GetDataEntry(pData,pEntry,&sData);

			uSize  = GetTabbedTextExtent(hDc,sData.pText,sData.iSize,pData->iTabCount,pData->aTabPos);
			uSize &= 0xFFFF;
			if(uSize<=uMax)continue;
			uMax   = uSize;
			}
		}
	else{
		for(uLine=pData->uListItemsCount;uLine>0;pEntry++)
			{
			uLine -= 1;
			uSize  = GetTabbedTextExtent(hDc,(TCHAR*)pEntry->pTextData,pEntry->wLineLen,pData->iTabCount,pData->aTabPos);
			uSize &= 0xFFFF;
			if(uSize<=uMax)continue;
			uMax   = uSize;
			}
		}

	ReleaseDC(pData->hWnd,hDc);

	if(pData->iMaxSizeX!=(int)uMax)
		{
		pData->iMaxSizeX = uMax;
		UpdateScrollX(pData);
		}

}


//*****************************************************************************
//*
//*		UpdateFont
//*
//*****************************************************************************
//	Prft ob sich ein Font verndert hat und holt die Hhe und Breiten.
//	pData		: Zeiger auf die Fensterdaten
//	iRedraw		: Soll das Fenster neugezeichnet werden
//	Ergibt 1 wenn der Font verndert wurde
static int UpdateFont(ListBoxData *pData)
{
TEXTMETRIC	sMetrics;
int			iRet;
int			iPos;
HDC			hDc;





	if(pData->aFonts[0]==NULL)								// Kein Font vorhanden
		{
		pData->aFonts[0] = hDefaultFontN;
		pData->aFonts[1] = hDefaultFontN;
		pData->aFonts[2] = hDefaultFontN;
		pData->aFonts[3] = hDefaultFontN;
		}

	if(pData->aFonts[0]!=pData->hFontL)						// Ist ein neuer Font vorhanden
		{
		pData->hFontL = pData->aFonts[0];

		hDc = GetDC(NULL);
		SelectObject(hDc,pData->aFonts[0]);
		GetTextMetrics(hDc,&sMetrics);
		pData->iFontHeight =  sMetrics.tmHeight;
		pData->iFontWidth  =  sMetrics.tmAveCharWidth;
		pData->iFontLine   =  sMetrics.tmAscent+1;
		pData->iFontOff    = (sMetrics.tmPitchAndFamily&TMPF_FIXED_PITCH)? -1:0;
		ReleaseDC(NULL,hDc);

		if(pData->iFontWidth<=0)pData->iFontWidth=1;

		iRet=1;
		}
	else{
		iRet=0;
		}



	   iPos=pData->uSizeX/pData->iFontWidth;
	if(iPos<=0)iPos=1;

	pData->uSizeCharX = iPos;

	if(iRet)UpdateTextSize(pData);


return iRet;
}

//*****************************************************************************
//*
//*		UpdateAscent
//*
//*****************************************************************************
//	Erzeugt einen den fetten Font fr das Fenster
//	pData		: Zeiger auf die Fensterdaten
//	iRedraw		: Soll das Fenster neugezeichnet werden
//	Ergibt 1 wenn der Font verndert wurde
static void  UpdateAscent(ListBoxData *pData)
{
LONG		lHeight;
LONG		lAscent;
TEXTMETRIC	sMetrics;
UINT		uPos;
HDC			hDc;




	hDc = GetDC(NULL);

	for(uPos=0;uPos<4;uPos++)
		{	
		SelectObject(hDc,pData->aFonts[uPos]);
		GetTextMetrics(hDc,&sMetrics);

		if(uPos)
			{
			pData->aFontAscent[uPos] = (char)(sMetrics.tmAscent-lAscent);	
			pData->aFontOpaque[uPos] = (lHeight!=sMetrics.tmHeight || pData->aFontAscent[uPos])? 1:0;
			}
		else{
			lHeight =  sMetrics.tmHeight;
			lAscent =  sMetrics.tmAscent;

			pData->aFontAscent[0] = 0;
			pData->aFontOpaque[0] = 0;
			}
		}
	
	ReleaseDC(NULL,hDc);


}

//*****************************************************************************
//*
//*		ColorListBoxGetMark
//*
//*****************************************************************************
//	Holt den aktuellen markierten Bereich. Bei Zeilenmodi (2,3) werden die
//	Y-Koordinaten gespeichert, beim Modus 1 die X-Koordinaten.
//	pData			: Zeiger auf die Fensterdaten
//	pStart		: Hier wird der Anfang der Markierung gespeichert.
//				  Das ist immer der kleinere Koordinatenwert.
//	pStop		: Hier wird das Ende   der Markierung gespeichert.
//				  Das ist immer der grere Koordinatenwert.
//	Ergibt die Art der Marke
//				0 = keine Markierung
//				1 = einzeilige Markierung
//				2 = mehrzeilige Markierung
static int ColorListBoxGetMark(ListBoxData *pData,int *pStart,int *pStop)
{


	if(!pData->iMarkMode)
		{
		*pStart = 0;
		*pStop  = 0;

		return 0;
		}

	if(pData->iMarkType>=2)
		{
		if(pData->iMarkStartY<pData->iMarkStopY || (pData->iMarkStartY==pData->iMarkStopY && pData->iMarkStartX<pData->iMarkStopX))
			{
			*pStart = pData->iMarkStartY;
			*pStop  = pData->iMarkStopY;
			if(pData->iMarkStopX>0)(*pStop)++;
			}
		else{
			*pStart = pData->iMarkStopY;
			*pStop  = pData->iMarkStartY;
			if(pData->iMarkStartX>0)(*pStop)++;
			}
		}
	else{
		if(pData->iMarkStartX<pData->iMarkStopX)
			{
			*pStart = pData->iMarkStartX;
			*pStop  = pData->iMarkStopX;
			}
		else{
			*pStart = pData->iMarkStopX;
			*pStop  = pData->iMarkStartX;
			}
		}


return (pData->iMarkStartY==pData->iMarkStopY)? 1:2;
}


//*****************************************************************************
//*
//*		GetMark
//*
//*****************************************************************************
//	Holt den aktuellen markierten Bereich
//	pData			: Zeiger auf die Fensterdaten
//	iStartX/iStartY	: Hier wird der Anfang der Markierung gespeichert.
//					  Das ist immer der kleinere Koordinatenwert.
//	iStopX /iStopY	: Hier wird das Ende   der Markierung gespeichert.
//					  Das ist immer der grere Koordinatenwert.
//	Ergibt die Art der Marke
//				0 = keine Markierung
//				1 = einzeilige Markierung
//				2 = mehrzeilige Markierung
static int ColorListBoxGetMark(ListBoxData *pData,int &iStartX,int &iStartY,int &iStopX,int &iStopY)
{

	if(!pData->iMarkMode)
		{
		iStartX = 0;
		iStartY = 0;
		iStopX  = 0;
		iStopY  = 0;

		return 0;
		}

	if(pData->iMarkStartY<pData->iMarkStopY || (pData->iMarkStartY==pData->iMarkStopY && pData->iMarkStartX<pData->iMarkStopX))
		{
		iStartX = pData->iMarkStartX;
		iStartY = pData->iMarkStartY;
		iStopX  = pData->iMarkStopX;
		iStopY  = pData->iMarkStopY;
		}
	else{
		iStartX = pData->iMarkStopX;
		iStartY = pData->iMarkStopY;
		iStopX  = pData->iMarkStartX;
		iStopY  = pData->iMarkStartY;
		}


return (iStartY==iStopY)? 1:2;
}

//*****************************************************************************
//*
//*		ColorListBoxGetItemRect
//*
//*****************************************************************************
//	Holt den sichtbaren Bereich eines Eintrages
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	uMode		: Welcher Teil soll geholt werden
//					0 = ganze Zeile
//					1 = nur Text
//					2 = Text mit Checkbox
//					3 = nur	Checkbox
//					4 = nur	Icon
//					5 = Text mit Checkbox und Icons
//	Ergibt 1 wenn der Eintrag sichtbar war
static int ColorListBoxGetItemRect(ListBoxData *pData,unsigned uLine,unsigned uMode,RECT *pRect)
{
BaseItem   *pEntry;
unsigned	uPos;



	if(uLine>pData->uListItemsCount)
		{
		memset(pRect,0,sizeof(RECT));
		return LB_ERR;
		}

	if(uLine>pData->uScrollY+pData->uPageEnties)			// Ist die Zeile sichtbar
		{
		memset(pRect,0,sizeof(RECT));
		return LB_ERR;
		}

	pEntry = pData->pListItems+uLine;

	   uPos = uLine-pData->uScrollY;
	if(uPos>=pData->uMaxEnties)								// Eintrag im Fenster sichtbar
		{
		memset(pRect,0,sizeof(RECT));
		return 0;
		}

	pRect->top     = pData->uStartPixel;
	pRect->top    += pData->iRowHeight*uPos;
	pRect->bottom  = pData->iRowHeight+pRect->top;

	pRect->left    = 0x00000000;
	pRect->right   = 0x7FFFFFFF;

	switch(uMode)
		{
	case 4:	if(pData->hIcons)								// Nur Icons
				{
				pRect->right  = pData->iIconsXsize;
				}
			else{
				pRect->right  = 0;
				}

			break;

	case 3:	pRect->right = pData->uStartText;				// Nur Checkbox
			pRect->left  = pData->uStartText;

			if(pData->uStyleEx&CLB_EX_CHECKBOXES)
				{
				pRect->left	 -= SELECT_SIZE;
				}

			break;

	case 5:	if(pData->hIcons)								// Text mit Checkbox und Icons
				{
				pRect->left  -= pData->iIconsXsize;
				pRect->right -= pData->iIconsXsize;
				}

	case 2:	if(pData->uStyleEx&CLB_EX_CHECKBOXES)			// Text mit Checkbox
				{
				pRect->left	 -= SELECT_SIZE;
				}

	case 1:	pRect->right  = pRect->left;					// Nur Text
			pRect->right += PointOfRow(pData,uLine);

			if(pData->uStyleEx&CLB_EX_CHECKBOXES)
				{
				pRect->left	 += SELECT_SIZE;
				pRect->right += SELECT_SIZE;
				}

			if(pData->hIcons)
				{
				pRect->left  += pData->iIconsXsize;
				pRect->right += pData->iIconsXsize;
				}

			break;

	case 0:	break;											// Ganze Zeile

	default:
			return 0;
		}

	if(pRect->left <(int) pData->uScrollX)
		{
		pRect->left = pData->uScrollX;
		}

	if(pRect->right<(int)(pData->uScrollX+pData->uSizeX))
		{
		pRect->right = pData->uScrollX+pData->uSizeX;
		}



return 1;
}

//*****************************************************************************
//*
//*		ColorListBoxEnsureVisible
//*
//*****************************************************************************
//	Macht einen Eintrag sichtbar
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	uMode		: Bestimmt wir die Eintrag sichtbar sein muss
//					0 = Eintrag muss vollstndig
//					1 = Eintrag kann auch teilweise sichtbar sein
//					2 = Eintrag muss in die erste Zeile
//	Ergibt 1 wenn nur zum Eintrag gescrollt wurde bzw. -1 wenn nicht.
static int ColorListBoxEnsureVisible(ListBoxData *pData,unsigned uLine,unsigned uMode)
{
unsigned	uEnd;
int			iOff;
int			iMax;




	if(uLine>=pData->uListItemsCount)return -1;

	switch(uMode)
		{
	case 2:		uEnd = 1;					break;
	case 1:		uEnd = pData->uMaxEnties;	break;
	default:	uEnd = pData->uPageEnties;	break;
		}

	if(uLine<pData->uScrollY)								// Vor erster Zeile
		{
		pData->uScrollY	= uLine;
		SetScrollPos(pData->hWnd,SB_VERT,uLine,TRUE);
		UpdateView(pData);
		}
	else if(uMode==FIRST_LINE)								// Als ersten Eintrag
		{
		if(uLine==pData->uScrollY)return 0;

		pData->uScrollY	= uLine;
		SetScrollPos(pData->hWnd,SB_VERT,uLine,TRUE);
		UpdateView(pData);

		return 1;
		}
	else if(uLine>=pData->uScrollY+uEnd)					// Nach letzter Zeile
		{
		iOff  = uLine-(pData->uPageEnties-1);
		iMax  = pData->uListItemsCount;
		iMax -=	pData->uPageEnties-1;

		if(iOff>=iMax)iOff=iMax;
		if(iOff<0)iOff=0;
		if(iOff!=(int)pData->uScrollY)
			{
			pData->uScrollY	= iOff;
			SetScrollPos(pData->hWnd,SB_VERT,iOff,TRUE);
			UpdateView(pData);
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		ColorListBoxIsVisible
//*
//*****************************************************************************
//	Prft ob ein Eintrag sichtbar ist
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages
//	Ergibt den Zustand des Eintrages:
//				   -1 = Unbekannter Eintrag
//					0 = Eintrag ist nicht sichtbar
//					1 = Eintrag ist teilweise sichtbar
//					2 = Eintrag ist aufgeklappt und ganz sichtbar
static int ColorListBoxIsVisible(ListBoxData *pData,unsigned uLine)
{



	if(uLine>=pData->uListItemsCount)return -1;


	if(uLine<pData->uScrollY)								// Vor erster Zeile
		{
		return 0;
		}

	if(uLine>=pData->uScrollY+pData->uMaxEnties)			// Nach letzter Zeile
		{
		return 0;
		}

	if(uLine==pData->uScrollY+pData->uPageEnties)			// Auf halbsichtbarer Zeile
		{
		return 1;
		}



return 2;
}

//*****************************************************************************
//*
//*		ColorListBoxDeleteItems
//*
//*****************************************************************************
//	Lscht einen oder mehrere Eintrge aus dem Fenster
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages der gelscht werden soll
//	uCount		: Ist die Anzahl der zu lschenden Eintrge
//	Ergibt 1 wenn die Eintrge gelscht wurden.
static int ColorListBoxDeleteItems(ListBoxData *pData,unsigned uLine,unsigned uCount)
{
CLB_STATUS	sNotify;
BaseItem   *pEntry;
unsigned	uScroll;
unsigned	uDraw;
unsigned	uPos;
unsigned	uNum;
unsigned	uEnd;
LRESULT		lRet;



	if(pData->cLockChanges)return 0;


	if(uLine>=pData->uListItemsCount || uCount==0)			// Prfe den Eintrag
		{
		return 0;
		}


	if(uCount>=0x7FFFFFFF)									// Anzahl prfen
		{
		uCount=0x7FFFFFFF;
		}

	if(uLine+uCount>pData->uListItemsCount)
		{
		uCount = pData->uListItemsCount-uLine;
		}



	uEnd = uLine+uCount;


	for(uPos=uLine;uPos<uEnd;uPos++)						// Ausgewhlte Eintrge abwhlen
		{
		if(uPos>=pData->uListItemsCount)					// Prfen ob der Eintrag noch existiert
			{
			return 0;
			}

		pEntry = pData->pListItems+uLine;

		if(!(pEntry->wLineSel&CLBS_SELECTED))
			{
			continue;
			}

		if(pData->uSelectedCount>0)							// Ausgewhlte Eintrge runterzhlen
			{
			pData->uSelectedCount--;
			}

		if(pData->uSelectedLine==uPos)
			{
			pData->uSelectedLine = 0xFFFFFFFF;
			}

		if(pData->uStyleEx&CLB_EX_NOCHANGENOTIFY)			// Keine nderungsnachrichten
			{
			continue;
			}


		sNotify.sNmHdr.code	= CLB_SELCHANED;
		sNotify.iItem		= uPos;
		sNotify.iState		= pEntry->wLineSel;
		sNotify.lParam		= pEntry->lParam;

		lRet = SendNotify(pData,&sNotify.sNmHdr);			// Bekannt geben das der Eintrag nicht mehr ausgewhlt ist
		}


	if(pData->uSelectedLine>=uLine)
	if(pData->uSelectedLine< uLine+uCount)
		{
		ColorListBoxSelectItem(pData,0xFFFFFFFF,FALSE);
		}


	uScroll = (pData->cFastFlag)? pData->uScrollYtmp:pData->uScrollY;


	if(uScroll>=uLine+uCount)								// Muss die Scroll-Position angepasst werden
		{
		if(pData->cFastFlag)
			{
			pData->uScrollY  = pData->uScrollYtmp;
			pData->cFastFlag = 0;
			}

		pData->uScrollY -= uCount;
		uDraw = 0xFFFFFFFF;
		}
	else if(uScroll+pData->uMaxEnties<=uLine)
		{
		uDraw = 0xFFFFFFFF;
		}
	else if(uScroll>uLine)
		{
		if(pData->cFastFlag)
			{
			pData->uScrollY  = pData->uScrollYtmp;
			pData->cFastFlag = 0;
			}

		pData->uScrollY = uLine;
		uDraw = uLine;
		}
	else{
		uDraw = uLine;
		}


	if(pData->iMarkMode)									// Marke lschen wenn in ihr gelscht wird
		{
		if(pData->iMarkStartY<pData->iMarkStopY)
			{
			uPos = pData->iMarkStartY;
			uNum = pData->iMarkStopY;
			}
		else{
			uNum = pData->iMarkStartY;
			uPos = pData->iMarkStopY;
			}

		if(uNum>=uLine && uPos<uLine+uCount)				// Marke lschen
			{
			SaveMark(pData);
			pData->iMarkMode = 0;
			UpdateMark(pData);
			}
		else if(uPos>=uLine-uCount)							// Marke verschieben
			{
			pData->iMarkStartY -= uCount;
			pData->iMarkStopY  -= uCount;
			}
		}


//******************** Eintrge lschen ***************************************


	pEntry	= pData->pListItems+uLine;

	for(uPos=uLine;uPos<uEnd;uPos++,pEntry++)				// Ausgewhlte Eintrge lschen
		{
		if(!pEntry->pTextData)continue;
		delete pEntry->pTextData;
		}

	memmove(pData->pListItems+uLine,pData->pListItems+uEnd,(pData->uListItemsCount-uEnd)*sizeof(BaseItem));
	pData->uListItemsCount -= uCount;

	UpdateScrollY(pData);

	if(pData->uListItemsCount==0)
		{
		UpdateView(pData);
		}
	else if(uDraw!=0xFFFFFFFF)
		{
		UpdateRows(pData,uDraw,pData->uMaxEnties);
		}


	if(pData->cHasCaret)									// Den Cursor verschieben
		{
		if(pData->iCursorY>=(int)uLine && pData->iCursorY<(int)(uLine+uCount))
			{
			pData->iCursorX	   = 0;
			pData->iCursorY	   = uLine;
			UpdateCaret(pData);
			}
		else if(pData->iCursorY>(int)(uLine+uCount))
			{
			pData->iCursorY	  -= uCount;
			UpdateCaret(pData);
			}
		}



return 1;
}

//*****************************************************************************
//*
//*		ColorListBoxSelectItem
//*
//*****************************************************************************
//	Whlt die aktuelle Zeile aus
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Zeilennummer (0xFFFFFFFF fr keine Zeile)
//	bRedraw		: Bei nderungen Fenster neu Zeichen
//	Ergibt -1 bei einem Fehler, 1 bei nderungen und 0 wenn alles gleich bleibt.
static int ColorListBoxSelectItem(ListBoxData *pData,unsigned uLine,BOOL bRedraw)
{
unsigned	uLast;
unsigned	uVal;



	if(uLine>=pData->uListItemsCount)						// Zeile prfen
	if(uLine!=0xFFFFFFFF)
		{
		return -1;
		}

	if(uLine==pData->uSelectedLine)							// Hat sich die aktuelle Linie gendert
		{
		return 0;
		}

	uLast = pData->uSelectedLine;
	pData->uSelectedLine = uLine;

	if(bRedraw)												// nderungen zeichnen
		{
		if(uLine!=0xFFFFFFFF)UpdateRows(pData,uLine);
		if(uLast!=0xFFFFFFFF)UpdateRows(pData,uLast);
		}

	if(pData->uStyleEx&CLB_EX_NOCHANGENOTIFY)				// Keine nderungsnachrichten
		{
		return 1;
		}

	UNLOCK(pData);
	uVal  = GetWindowLong(pData->hWnd,GWL_ID);
	SendMessage(GetParent(pData->hWnd),WM_COMMAND,MAKELONG(uVal,LBN_SELCHANGE),(LPARAM)pData->hWnd);
	LOCK(pData);


return 1;
}

//*****************************************************************************
//*
//*		ColorListBoxSetSel
//*
//*****************************************************************************
//	Whlt einen Eintrag aus bzw. ab.
//	pData		: Zeiger auf die Fensterdaten
//	uLine		: Ist die Nummer des Eintrages der ausgewhlt werden soll
//	iSelect		: Soll der Eintrag ausgewhlt werden. (0=nein 1=ja 2=xor)
//	Ergibt			0       Wenn der Eintrag existiert
//					LB_ERR	Wenn der Eintrag nicht existiert
static int ColorListBoxSetSel(ListBoxData *pData,unsigned uLine,int iSelect,BOOL bNotify=TRUE)
{
BaseItem	 *pEntry;
CLB_STATUS	  sNotify;
WORD		  wValue;



	if(uLine>=pData->uListItemsCount)
		{
		return LB_ERR;
		}


	pEntry  = pData->pListItems+uLine;

	switch(iSelect)											// Art der Auswahl
		{
	case 2:	wValue = pEntry->wLineSel^ CLBS_SELECTED;	break;
	case 1:	wValue = pEntry->wLineSel| CLBS_SELECTED;	break;
	case 0:	wValue = pEntry->wLineSel&~CLBS_SELECTED;	break;
	default:return LB_ERR;
		}



	if(pEntry->wLineSel!=wValue)							// Hat sich etwas verndert
		{
		pEntry->wLineSel = wValue;

		if(pData->uStyleEx&CLB_EX_CHECKBOXES)
		if(!(pEntry->wLineSel&CLBS_NOBOX))
			{
			UpdateBox(pData,uLine);
			}

		if(pEntry->wLineSel&CLBS_SELECTED)
				pData->uSelectedCount++;
		else	pData->uSelectedCount--;

		if(bNotify)											// Eine Notify-Nachricht verschicken
		if(!(pData->uStyleEx&CLB_EX_NOCHANGENOTIFY))
			{
			sNotify.sNmHdr.code		= CLB_SELCHANED;
			sNotify.iItem			= uLine;
			sNotify.iState			= wValue;
			sNotify.lParam			= pEntry->lParam;

			SendNotify(pData,&sNotify.sNmHdr);
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		ColorListBoxCalcSize
//*
//*****************************************************************************
//	Fgt einen Eintrag ins Fenster ein
//	pData		: Zeiger auf die Fensterdaten
//	pItem		: Zeiger auf die Taten des Eintrages
//	uFlags		: Sind die State-Flags fr das Einfgen
//					CLBF_FAST		= Schnell einfgen
//					CLBF_ENDSCROLL	= Beim Einfgen am sichtbaren Ende weiter scrollen
//					CLBF_SHOWSCROLL	= Beim Einfgen muss der Eintrag sichtbar bleiben
//					CLBS_SELECTED	= Status fr Auswahlbit
//	Ergibt die Einfgeposition des neuen Eintrages oder LB_ERRSPACE bei einem Fehler
static unsigned ColorListBoxCalcSize(ListBoxData *pData,int iPosX,CLB_ITEM *pItem)
{
int		iSize;
int		iLen;
HDC		hDc;


	   iLen = pItem->iTextMax;
	if(iLen<0)iLen=str_len(pItem->pText);

	hDc   = GetDC(pData->hWnd);
	iSize = GetTabbedSizeEx(hDc,iPosX,pItem->pText,pItem->pColor,iLen,pData->iTabCount,pData->aTabPos,0,pData->aFonts);

	ReleaseDC(pData->hWnd,hDc);


return iSize;
}

//*****************************************************************************
//*
//*		ColorListBoxInsertItem
//*
//*****************************************************************************
//	Fgt einen Eintrag ins Fenster ein
//	pData		: Zeiger auf die Fensterdaten
//	pItem		: Zeiger auf die Taten des Eintrages
//	uFlags		: Sind die State-Flags fr das Einfgen
//					CLBF_FAST		= Schnell einfgen
//					CLBF_ENDSCROLL	= Beim Einfgen am sichtbaren Ende weiter scrollen
//					CLBF_SHOWSCROLL	= Beim Einfgen muss der Eintrag sichtbar bleiben
//					CLBS_SELECTED	= Status fr Auswahlbit
//	Ergibt die Einfgeposition des neuen Eintrages oder LB_ERRSPACE bei einem Fehler
static unsigned ColorListBoxInsertItem(ListBoxData *pData,CLB_ITEM *pItem,unsigned uFlags=0)
{
LPBYTE		pMem;
LPCTSTR		pText;
LPBYTE		pColor;
LPARAM		lParam;
BaseItem   *pOld;
BaseItem   *pItems;
BaseItem   *pEntry;
unsigned	uState;
unsigned	uIdent;
unsigned	uLine;
unsigned	uSize;
unsigned	uMax;
unsigned	uLen;
int			iMax;
int			iLen;



	if(pData->cLockChanges)return 0;


	if(pData->uListItemsCount>=pData->uListItemsMax)		// Gre der Liste erhhen
		{
		pOld	= pData->pListItems;
		uMax	= pData->uListItemsMax;
		uMax   += pData->uListItemsMax/2;
		uMax   += 64;
		pItems  = new BaseItem[uMax+1];

		if(!pItems)
			{
			return LB_ERRSPACE;
			}

		memcpy(pItems ,pData->pListItems       ,sizeof(BaseItem)*(     pData->uListItemsMax+1));
		memset(pItems +pData->uListItemsMax+1,0,sizeof(BaseItem)*(uMax-pData->uListItemsMax  ));

		pData->uListItemsMax = uMax;
		pData->pListItems	 = pItems;
		delete pOld;
		}


//******************** Den neuen Eintrag erzeugen *****************************


	uLine	=  pItem->iItem;
	pText	= (pItem->uMask&CLBF_TEXT )? pItem->pText   :_T("");
	iLen	= (pItem->uMask&CLBF_TEXT )? pItem->iTextMax:0;
	lParam	= (pItem->uMask&CLBF_PARAM)? pItem->lParam	:0;
	pColor	= (pItem->uMask&CLBF_COLOR)? pItem->pColor  :0;
	uState	= (pItem->uMask&CLBF_STATE)? pItem->uState&pItem->uStateMask:0;
	uIdent	= (pItem->uMask&CLBF_IDENT)? pItem->uState>>16:pData->wIdent;
	uLen	= (iLen<0)? str_len(pText):iLen;

	if(uLen>=0x4000)										// Ist der String zulang ?
		{
		return LB_ERRSPACE;
		}


	pItems	= pData->pListItems;

	if(pData->pLineCallback)
		{
		pMem	= 0;
		uLen	= 0;
		uSize	= 0;
		}
	else{
		uSize	= (uLen+4)&~3;
		pMem	= new BYTE[uSize*(1+sizeof(TCHAR))];
		if(pMem==NULL)return LB_ERRSPACE;

		memcpy(pMem,pText,uLen*sizeof(TCHAR));
		((TCHAR*)pMem)[uLen] = 0;

		if(((DWORD_PTR)pColor)>0xFF)
				memcpy(pMem+uSize*sizeof(TCHAR),      pColor,uLen);
		else	memset(pMem+uSize*sizeof(TCHAR),(BYTE)pColor,uLen);
		}

	if(uLine>=pData->uListItemsCount)
		{
		uLine = pData->uListItemsCount;

		if(pData->uStyleEx&CLB_EX_AUTOSCROLL)
			{
			uFlags |= CLBF_ENDSCROLL;
			}
		}

															// Erstelle den neuen Eintrag
	memmove(pItems+uLine+1,pItems+uLine,sizeof(BaseItem)*(pData->uListItemsCount-uLine));

	pEntry				= pItems+uLine;
	pEntry->lParam		= lParam;
	pEntry->wIdent		= uIdent;
	pEntry->wLineSel	= uState&((CLBS_NOBOX|CLBS_ICON_MASK|CLBS_OVERLAY)&~CLBS_SELECTED);
	pEntry->wLineLen	= uLen;
	pEntry->wLineSize   = uSize;
	pEntry->pTextData   = pMem;


//******************** Eintrag einfgen ***************************************


	pData->uListItemsCount++;


	iMax = PointOfRow(pData,uLine);

	if(iMax>pData->iMaxSizeX)								// Textbreite aktualisieren
		{
		pData->iMaxSizeX = iMax;

		if(uFlags&CLBF_FAST)								// Schnell einfgen
			{
			UpdateStart(pData);

			if(!pData->cFastFlag)
				{
				pData->cUpdateScrollX = 1;
				}
			}
		else{
			UpdateScrollX(pData);
			}
		}



	if(pData->iMarkMode)									// Marke lschen wenn in ihr eingefgt wird
		{
		if((pData->iMarkStartY<=(int)uLine && pData->iMarkStopY>=(int)uLine) ||
		   (pData->iMarkStartY>=(int)uLine && pData->iMarkStopY<=(int)uLine))
			{
			SaveMark(pData);
			pData->iMarkMode = 0;
			UpdateMark(pData);
			}
		else if(pData->iMarkStartY>=(int)uLine)				// Marke verschieben
			{
			pData->iMarkStartY += 1;
			pData->iMarkStopY  += 1;
			}
		}



	if(uFlags&CLBF_FAST)									// Schnell einfgen
		{
		if(!pData->cFastFlag)
			{
			pData->cFastFlag   = 1;
			pData->uScrollYtmp = pData->uScrollY;
			}

		if(uLine<pData->uScrollYtmp)
			{
			pData->uScrollYtmp++;
			}

		if(uFlags&CLBF_ENDSCROLL)							// Beim Einfgen am Ende muss Eintrag sichtbar bleiben
			{
			if(uLine>=pData->uScrollYtmp+pData->uPageEnties-2)
			if(uLine<=pData->uScrollYtmp+pData->uPageEnties)
				{
				if(pData->uPageEnties>=2)
						pData->uScrollYtmp = uLine-pData->uPageEnties+2;
				else	pData->uScrollYtmp = uLine;

				pData->cUpdateScrollY= 1;
				pData->cUpdateView	 = 1;

				UpdateStart(pData);

				return uLine;
				}
			}

		if(uFlags&CLBF_SHOWSCROLL)							// Einfgeposition muss sichtbar bleiben
			{
			if(uLine<pData->uScrollYtmp || uLine>=pData->uScrollYtmp+pData->uPageEnties)
				{
				if(uLine<pData->uScrollYtmp)
						 pData->uScrollYtmp = uLine;
				else	 pData->uScrollYtmp = uLine-(pData->uPageEnties-1);

				pData->cUpdateScrollY= 1;
				pData->cUpdateView	 = 1;

				UpdateStart(pData);

				return uLine;
				}
			}

		if(uLine<pData->uScrollY+pData->uMaxEnties)
			{
			pData->cUpdateView	= 1;
			UpdateStart(pData);
			}

		if(pData->cHasCaret)
		if(pData->iCursorY>=(int)uLine)						// Cursorposition nach hinten verschieben
			{
			pData->iCursorY++;
			pData->cUpdateCaret	= 1;

			UpdateStart(pData);
			}

		if(pData->uSelectedLine>=uLine)						// Ausgewhle Zeile nach hinten verschieben
			{
			ColorListBoxSelectItem(pData,pData->uSelectedLine+1,FALSE);
			}

		return uLine;
		}

	if(uLine<pData->uScrollY)
		{
		pData->uScrollY++;
		}

	if(uState&CLBS_SELECTED)								// Den Eintrag auch auswhlen
		{
		ColorListBoxSetSel(pData,uLine,1);
		}

	if(uFlags&CLBF_ENDSCROLL)								// Beim Einfgen am Ende muss Eintrag sichtbar bleiben
		{
		if(uLine>=pData->uScrollY+pData->uPageEnties-2)
		if(uLine<=pData->uScrollY+pData->uPageEnties)
			{
			if(pData->uPageEnties>=2)
					pData->uScrollY = uLine-pData->uPageEnties+2;
			else	pData->uScrollY = uLine;

			UpdateView(pData);
			UpdateScrollY(pData);
			return uLine;
			}
		}
	else if(uFlags&CLBF_SHOWSCROLL)							// Einfgeposition muss sichtbar bleiben
		{
		if(ColorListBoxEnsureVisible(pData,uLine,0))
			{
			return uLine;
			}
		}


	if(pData->cHasCaret)
	if(pData->iCursorY>=(int)uLine)							// Cursorposition nach hinten verschieben
		{
		pData->iCursorY++;
		UpdateCaret(pData);
		}

	if(pData->uSelectedLine>=uLine)							// Ausgewhle Zeile nach hinten verschieben
		{
		ColorListBoxSelectItem(pData,pData->uSelectedLine+1);
		}

	if(uLine<pData->uScrollY+pData->uMaxEnties)
		{
		UpdateRows(pData,uLine,pData->uMaxEnties);
		}


	UpdateScrollY(pData);

  	if(pData->iMaxX<(int)uLen)
		{
		pData->iMaxX = uLen;
		}





return uLine;
}

//*****************************************************************************
//*
//*		ColorListBoxSetItem
//*
//*****************************************************************************
//	ndert einen Eintrag im Fenster
//	pData		: Zeiger auf die Fensterdaten
//	pItem		: Zeiger auf die ein zu ndernden Daten
//	uFlags		: Diverse Flags fr das ndern
//					CLBF_FAST	= Schnell einfgen
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
static int ColorListBoxSetItem(ListBoxData *pData,const CLB_ITEM *pItem,unsigned uFlags)
{
LPBYTE			pOld;
LPBYTE			pMem;
BaseItem	   *pEntry;
unsigned		uChange;
unsigned		uMask;
unsigned		uBits;
unsigned		uLine;
unsigned		uSize;
int				iRet;
int				iLen;
int				iBox;
int				iCol;


	uChange = 0;


	   uLine = pItem->iItem;
	if(uLine>=pData->uListItemsCount)return 0;

	pEntry	 = pData->pListItems+uLine;
	uBits	 = pItem->uMask;

	if(uBits&(CLBF_TEXT|CLBF_COLOR))						// Text und Farben ndern
	if(pData->pLineCallback==NULL)
		{
		iLen = pItem->iTextMax;

		if(uBits&CLBF_TEXT)
			{
			if(iLen< 0x0000)iLen=str_len(pItem->pText);
			if(iLen>=0x4000)iLen=0x3FFF;

			if(iLen>pEntry->wLineSize)						// Passt der Text in den Speicher
				{
				uSize	= (iLen+4)&~3;
				pOld	= pEntry->pTextData;
				pMem	= new BYTE[uSize*(1+sizeof(TCHAR))];
				if(!pMem)return 0;
				}
			else{
				uSize	= pEntry->wLineSize;
				pMem	= pEntry->pTextData;
				pOld	= 0;
				}

			memcpy(pMem,pItem->pText,iLen*sizeof(TCHAR));	// Text ndern
			((TCHAR*)pMem)[iLen] = 0;

			if(!(uBits&CLBF_COLOR))
				{
				if(pOld)									// Farben aus alten Puffer holen
					{
					memcpy(pMem+uSize*sizeof(TCHAR),pOld+pEntry->wLineSize*sizeof(TCHAR),pEntry->wLineLen);
					}

				if(iLen>pEntry->wLineLen)					// Farbpuffer auffllen
					{
					memset(pMem+uSize*sizeof(TCHAR)+pEntry->wLineLen,0,iLen-pEntry->wLineLen);
					}
				}

			pEntry->pTextData = pMem;
			pEntry->wLineLen  = iLen;
			pEntry->wLineSize = uSize;

			if(pOld)delete pOld;
			}

															// Farben ndern
		if(uBits&CLBF_COLOR)
			{
			uSize	= pEntry->wLineSize;
			pMem	= pEntry->pTextData;

			if((DWORD_PTR)pItem->pColor<=0xFF)
				{
				iCol = U(pItem->pColor);
				if(iCol!=0xFF)memset(pMem+uSize*sizeof(TCHAR),iCol,pEntry->wLineLen);
				}
			else{
				if(U(iLen)>pEntry->wLineLen)iLen=pEntry->wLineLen;
				memcpy(pMem+uSize*sizeof(TCHAR),pItem->pColor,iLen);
				}
			}

		uChange = 1;
		}

	if(uBits&CLBF_PARAM)									// Eintragsdaten ndern
		{
		pEntry->lParam	= pItem->lParam;
		}


	if(uBits&CLBF_STATE)									// Den Status ndern
		{
		uMask = pItem->uStateMask;
		uBits = uMask&(pEntry->wLineSel^pItem->uState);
		pEntry->wLineSel ^= uBits&~(CLBS_NOBOX|CLBS_SELECTED);
		iBox  = 1;

		if(uBits&CLBS_NOBOX)
			{
			pEntry->wLineSel^=CLBS_NOBOX;
			iBox = 1;
			}

		if(pData->hStates)
		if(uBits&(CLBS_OVERLAY&~CLBS_SELECTED))				// Andere Checkbox einblenden
			{
			iBox = 1;
			}

		if(uBits&CLBS_SELECTED)								// Hat sich die Auswahl verndert
			{
			if(uFlags&CLBF_FAST)
				{
				pEntry->wLineSel ^= CLBS_SELECTED;
				uChange = 1;
				}
			else{
				iRet = ColorListBoxSetSel(pData,uLine,2,TRUE);
				if(iRet>=0)uChange=1;
				iBox = 0;
				}
			}

		if(iBox && (pData->uStyleEx&CLB_EX_CHECKBOXES))		// Auswahlbox neuzeichnen
			{
			if(uFlags&CLBF_FAST)
				{
				uChange = 1;
				}
			else{
				UpdateBox(pData,uLine);
				}
			}

		if(pData->hIcons)
		if(uBits&CLBS_ICON_MASK)							// Hat sich das Icon verndert
			{
			if(uFlags&CLBF_FAST)
				{
				uChange = 1;
				}
			else{
				UpdateIcon(pData,uLine);
				}
			}
		}

	if(uChange)												// Neuzeichnen des Eintrages
		{
		if(uFlags&CLBF_FAST)
			{												// Ist die Zeile sichtbar
			if(uLine>=pData->uScrollY && uLine<pData->uListItemsCount+pData->uMaxEnties/2)
				{
				pData->cUpdateView	= 1;
				UpdateStart(pData);
				}
			}
		else{
			UpdateRows(pData,uLine,1);
			}
		}



return 1;
}


//*****************************************************************************
//*
//*		ColorListBoxGetItem
//*
//*****************************************************************************
//	Daten von einem Eintrag abfragen
//	pData		: Zeiger auf die Fensterdaten
//	pItem		: Zeiger auf die den Datenspeicher
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
static unsigned ColorListBoxGetItem(ListBoxData *pData,CLB_ITEM *pItem)
{
BaseItem	   *pEntry;
CLB_ENTRY		sData;
unsigned		uBits;
unsigned		uLine;




	   uLine = pItem->iItem;
	if(uLine>=pData->uListItemsCount)						// Prfe die Zeilennummer
		{
		pItem->iTextMax	= 0;
		return 0;
		}

	pEntry   = pData->pListItems+uLine;
	uBits    = pItem->uMask;

	if(uBits&CLBF_PARAM)
		{
		pItem->lParam = pEntry->lParam;
		}

	if(uBits&CLBF_STATE)
		{
		pItem->uState = pEntry->wLineSel;
		}

	if(uBits&CLBF_IDENT)
		{
		if(!(uBits&CLBF_STATE))
				pItem->uState  = pEntry->wIdent<<16;
		else	pItem->uState |= pEntry->wIdent<<16;
		}

	if(uBits&(CLBF_TEXT|CLBF_COLOR))						// Text und Farben auslesen
		{
		GetDataEntry(pData,pEntry,&sData);

		if(uBits&CLBF_TEXT)
			{
			if(uBits&CLBF_TEXTPTR)
				{
				pItem->pText	= sData.pText;
				pItem->iTextMax	= sData.iSize;
				}
			else if(pItem->iTextMax>0)
				{
				if(sData.iSize>=pItem->iTextMax)sData.iSize=pItem->iTextMax-1;
				memcpy(pItem->pText,sData.pText,sData.iSize*sizeof(TCHAR));
				pItem->pText[sData.iSize] = 0;
				pItem->iTextMax = sData.iSize;
				}
			}

		if(uBits&CLBF_COLOR)								// Farben auslesen
			{
			if(uBits&CLBF_COLORPTR)
				{
				pItem->pColor	= sData.pColor;
				pItem->iTextMax	= sData.iSize;
				}
			else if(pItem->iTextMax>0)
				{
				if(sData.iSize>pItem->iTextMax)sData.iSize=pItem->iTextMax;
				memcpy(pItem->pColor,sData.pColor,sData.iSize);
				pItem->iTextMax = sData.iSize;
				}
			}
		}


return 1;
}


//*****************************************************************************
//*
//*		ColorListBoxHitTest
//*
//*****************************************************************************
//	Prft wo eine Koordinate im Fenster ist
//	pData		: Zeiger auf die Fensterdaten
//	pInfo		: Zeiger auf die ein zu fgenden Daten
//	Ergibt das Item auf dem die Koordinate zeigt oder 0xFFFFFFFF wenn nicht.
static int ColorListBoxHitTest(ListBoxData *pData,CLB_HITTESTINFO *pInfo)
{
int			iTextPixels;
int			iStartX;
int			iStartY;
int			iStopX;
int			iStopY;
int			iPosX;
int			iPosY;
int			iPosZ;
BaseItem   *pEntry;



	iPosX = pInfo->sPoint.x;
	iPosY = pInfo->sPoint.y;

	if((unsigned)iPosX>=pData->uSizeX)						// Rechts/Links auerhalb vom Fenster
		{
		pInfo->iItem	= -1;
		pInfo->uFlags	= (iPosX<0)? CLBT_TOLEFT:CLBT_TORIGHT;
		return -1;
		}

	iPosY -= pData->uStartPixel;

	if((unsigned)iPosY>=pData->uSizeY)						// Open/Unten auerhalb vom Fenster
		{
		pInfo->iItem	= -1;
		pInfo->uFlags	= (iPosY<0)? CLBT_ABOVE:CLBT_BELOW;
		return -1;
		}


	iPosZ  = iPosY/pData->iRowHeight;
	iPosZ +=       pData->uScrollY;

	if((unsigned)iPosZ>=pData->uListItemsCount)				// Kein Eintrag unter der Position
		{
		pInfo->iItem	= -1;
		pInfo->uFlags	= CLBT_NOWHERE;
		return -1;
		}

	iPosX		+= pData->uScrollX;
	pEntry		 = pData->pListItems+iPosZ;
	pInfo->iItem = iPosZ;
	iTextPixels	 = PointOfRow(pData,iPosZ);


	if(pData->uStyleEx&CLB_EX_GRIDLINES)
		{
		iPosX--;
		}

	if(pData->hIcons)										// Auf deinem Icon
		{
		iPosX -= pData->iIconsXsize;

		if(iPosX<0)
			{
			pInfo->uFlags = CLBT_ONITEMICON;
			return iPosZ;
			}
		}

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)					// Auf der Auswahl-Box
		{
		iPosX -= SELECT_SIZE;

		if(iPosX<0)
			{
			pInfo->uFlags = CLBT_ONITEMCHECK;
			return iPosZ;
			}
		}

	if(iPosX<iTextPixels+5)									// Auf Text
		{
		pInfo->uFlags = CLBT_ONITEMLABEL;

		if(pData->iMarkMode!=0)								// Ist ein Text ausgewhlt
			{
			if(pData->iMarkStartY<pData->iMarkStopY || (pData->iMarkStartY==pData->iMarkStopY && pData->iMarkStartX<=pData->iMarkStopX))
				{
				iStartX = pData->iMarkStartX;
				iStartY = pData->iMarkStartY;
				iStopX  = pData->iMarkStopX;
				iStopY  = pData->iMarkStopY;
				}
			else{
				iStartX = pData->iMarkStopX;
				iStartY = pData->iMarkStopY;
				iStopX  = pData->iMarkStartX;
				iStopY  = pData->iMarkStartY;
				}

			if(iStartY<=iPosZ && iStopY>=iPosZ)
				{
				iStartX = (iStartY<iPosZ)? 0          :PointOfRow(pData,iPosZ,iStartX);
				iStopX  = (iStopY >iPosZ)? iTextPixels:PointOfRow(pData,iPosZ,iStopX );

				if(iPosX>=iStartX && iPosX<=iStopX)
					{
					pInfo->uFlags |= CLBT_SELECTED;
					}
				}
			}
		}
	else{
		pInfo->uFlags = CLBT_ONITEMRIGHT;
		}



return iPosZ;
}

//*****************************************************************************
//*
//*		ColorListBoxGetNextItem
//*
//*****************************************************************************
//	Sucht den nchsten Eintrag
//	pData	: Zeiger auf die Fensterdaten
//	uLine	: Ist die Nummer des Eintrages
//	uFlags	: Bestimmt nach welchem Eintrag gesucht werden soll
//	Ergibt 1 wenn der Eintrag unterstrichen wurde
static int ColorListBoxGetNextItem(ListBoxData *pData,unsigned uLine,unsigned uFlags)
{
BaseItem	   *pEntry;


	switch(uFlags)
		{
	case CLPI_FIRSTVISIBLE:		if(pData->uListItemsCount<=pData->uScrollY)return -1;

								return pData->uScrollY;

	case CLPI_NEXTVISIBLE:		uLine++;
								if(uLine>=pData->uListItemsCount)return -1;
								if(uLine<=pData->uScrollY)return -1;
								if(uLine> pData->uScrollY+pData->uPageEnties)return -1;

								return uLine;

	case CLPI_PREVIOUSVISIBLE:	uLine--;
								if(uLine>=pData->uListItemsCount)return -1;
								if(uLine<=pData->uScrollY)return -1;
								if(uLine> pData->uScrollY+pData->uPageEnties)return -1;

								return uLine;

	case CLPI_LASTVISIBLE:		uLine = pData->uScrollY+pData->uPageEnties;
								if(uLine>=pData->uListItemsCount)return -1;
								if(uLine<=pData->uScrollY)return -1;
								if(uLine> pData->uScrollY+pData->uPageEnties)return -1;

								return uLine;

	case CLPI_CARET:			return pData->uSelectedLine;
		}



	if(uFlags&1)
		{
		uLine++;

		if(uLine>pData->uListItemsCount)return -1;

		pEntry  = pData->pListItems+uLine;

		for(;uLine<pData->uListItemsCount;uLine++,pEntry++)
			{
			if(pEntry->wLineSel&CLBS_SELECTED)return uLine;
			}
		}



return -1;
}


//*****************************************************************************
//*
//*		ColorListBoxFindItem
//*
//*****************************************************************************
//	Sucht einen Eintrag mit einem bestimmten Text
//	pData	: Zeiger auf die Fensterdaten
//	uLine	: Ist die Nummer des Elterneintrages
//	pFind	: Die Struktur mit den Suchparametern
//	Ergibt die Nummer des Eintrages bzw. -1 wenn nicht gefunden wurde
static int ColorListBoxFindItem(ListBoxData *pData,unsigned uLine,CLB_FIND *pFind)
{
CLB_ENTRY		sData;
BaseItem	   *pEntry;
unsigned		uChkParam;
unsigned		uChkState;
unsigned		uChkText;
unsigned		uChkCase;
unsigned		uTextLen;


	uLine++;

	if(uLine>pData->uListItemsCount)						// Ist der Eintrag gltig
		{
		return 0;
		}



	uChkParam = pFind->uFlags&CLB_FIND_PARAM;
	uChkState = pFind->uFlags&CLB_FIND_STATE;
	uChkText  = pFind->uFlags&CLB_FIND_TEXT;
	uChkCase  = pFind->uFlags&CLB_FIND_CASE;
	uTextLen  = 0;


	if(uChkText)
		{
		uTextLen = str_len(pFind->pText);
		}

	pEntry = pData->pListItems+uLine;

	for(;;uLine++,pEntry++)									// Durchlaufe alle Zeilen
		{
		if(uLine>=pData->uListItemsCount)
			{
			return LB_ERR;
			}

		if(uChkParam && pFind->lParam!=pEntry->lParam)		// Vergleiche lParam
			{
			continue;
			}
															// Vergleiche die State-Bits
		if(uChkState && ((pEntry->wLineSel^pFind->uState)&pFind->uStateMask))
			{
			continue;
			}

		if(uChkText==0)break;

		GetDataEntry(pData,pEntry,&sData);

		if(uTextLen!=(UINT)sData.iSize)continue;

		if(uChkCase)										// Zwischen Gro/Kleinbuchstaben unterscheiden
			{
			if(!str_icmp(sData.pText,pFind->pText))break;
			}
		else{
			if(!memcmp(sData.pText,pFind->pText,sData.iSize*sizeof(TCHAR)))break;
			}
		}



return uLine;
}

//*****************************************************************************
//*
//*		ColorListBoxCopy
//*
//*****************************************************************************
//	Kopiert den markierten Text in die Zwischenablage
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxCopy(ListBoxData *pData)
{
CLB_ENTRY	sData;
TCHAR	   *pMem;
int			iStartX,iStopX;
int			iStartY,iStopY;
int			iLen;
int			i;



	if(pData->uListItemsCount<=0)return;


	if(pData->iMarkMode==0)									// Ist etwas ausgewhlt
		{
		iStartX = 0x0000;
		iStopX  = 0x8000;
		iStartY = pData->uSelectedLine;
		iStopY  = iStartY;
		}
	else if(pData->iMarkStartY<pData->iMarkStopY || (pData->iMarkStartY==pData->iMarkStopY && pData->iMarkStartX<=pData->iMarkStopX))
		{
		iStartX = pData->iMarkStartX;
		iStartY = pData->iMarkStartY;
		iStopX  = pData->iMarkStopX;
		iStopY  = pData->iMarkStopY;

		if(iStopY==iStartY && iStopX==iStartX)
			{
			iStartX = 0x0000;
			iStopX  = 0x8000;
			}
		}
	else{
		iStartX = pData->iMarkStopX;
		iStartY = pData->iMarkStopY;
		iStopX  = pData->iMarkStartX;
		iStopY  = pData->iMarkStartY;
		}

	if(iStartY==iStopY)										// Einzeilige Marke
		{
		if(iStartY>=(int)pData->uListItemsCount || iStartY<0)return;

 		GetDataEntry(pData,iStartY,&sData);

		if(iStartX>sData.iSize)iStartX=sData.iSize;
		if(iStopX >sData.iSize)iStopX =sData.iSize;

		iLen  = iStopX-iStartX;
		if(iLen>0)SetClipboardText(sData.pText+iStartX,iLen);

		return;
		}


	if(iStopY>=(int)pData->uListItemsCount)					// Koordinaten prfen
		{
		iStopY  = pData->uListItemsCount-1;
		if(iStopY<0)iStopY=0;
		GetDataEntry(pData,iStopY,&sData);
		iStopX  = sData.iSize;
		}

	if(iStartY<0)
		{
		iStartY = 0;
		iStopX  = 0;
		}


	GetDataEntry(pData,iStartY,&sData);

	iLen  = sData.iSize;									// Lnge der mehrzeiligen Marke bestimmen
	if(iStartX>iLen)iStartX=iLen;
	iLen -= iStartX;
	iLen += 2;

	for(i=iStartY+1;i<iStopY;i++)
		{
		GetDataEntry(pData,i,&sData);
		iLen += sData.iSize+2;
		}


	GetDataEntry(pData,i,&sData);
	if(iStopX>sData.iSize)iStopX=sData.iSize;

		pMem=new TCHAR[iLen+iStopX+1];
	if(!pMem)return;


	GetDataEntry(pData,iStartY,&sData);						// Die Texte kopieren

	iLen  = sData.iSize;
	iLen -= iStartX;
	if(iLen<0)iLen=0;

	memcpy(pMem,sData.pText+iStartX,iLen*sizeof(TCHAR));

	pMem[iLen]='\r';iLen++;
	pMem[iLen]='\n';iLen++;

	for(i=iStartY+1;i<iStopY;i++)
		{
		GetDataEntry(pData,i,&sData);

		memcpy(pMem+iLen,sData.pText,sData.iSize*sizeof(TCHAR));

		iLen += sData.iSize;
		pMem[iLen]='\r';iLen++;
		pMem[iLen]='\n';iLen++;
		}

	GetDataEntry(pData,iStopY,&sData);
	memcpy(pMem+iLen,sData.pText,iStopX*sizeof(TCHAR));
	iLen += iStopX;

	SetClipboardText(pMem,iLen);

	delete pMem;


}

//*****************************************************************************
//*
//*		ColorListBoxLButtonUp
//*
//*****************************************************************************
//	Loslassen der linker Maustatste.
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxLButtonUp(ListBoxData *pData,UINT nFlags,int iPosX,int iPosY)
{
int		iLine;
int		iRow;


	iPosX -= pData->uStartText;
	iLine  = iPosY/pData->iRowHeight+pData->uScrollY;
	iRow   = RowOfPoint(pData,iLine,iPosX,1);

	if(pData->iMarkType==3)									// Nur ganzzeilige Markierungen
		{
		if(pData->iMarkStartY<iLine || (pData->iMarkStartY==iLine && pData->iMarkStartX<iRow))
			{
			iLine	           += 1;
			iRow				= 0;
			pData->iMarkStartX	= 0;
			}
		else{
			iRow				= 0x0000;
			pData->iMarkStartX	= 0x8000;
			}
		}


	if(iLine>(int)pData->uListItemsCount)iLine=pData->uListItemsCount;
	if(iLine<0)iLine=0;
	if(iRow <0)iRow =0;


	if(pData->cTimerOn)										// Den Scroll-Timer abschalten
		{
		pData->cTimerOn=0;
		KillTimer(pData->hWnd,ID_SCROLL);
		}

	if(pData->iMarkType>=1 && pData->iMarkMode==1)
		{
		if(pData->iMarkType>=2)
			{
			pData->iMarkStopY = iLine;
			}

		pData->iMarkStopX  = iRow;							// Beginn einer Marke
		pData->iMarkMode   = 2;
		}


	if(pData->iMarkMove<=0)
		{
		pData->iMarkMode   = 0;
		}

	if(pData->iMarkStopX==pData->iMarkStartX)
	if(pData->iMarkStopY==pData->iMarkStartY)
		{
		pData->iMarkMode   = 0;
		}

	ReleaseCapture();


}

//*****************************************************************************
//*
//*		ColorListBoxLButtonDown
//*
//*****************************************************************************
//	Klick mit linker Maustatste.
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxLButtonDown(ListBoxData *pData,UINT nFlags,int iPosX,int iPosY)
{
BaseItem   *pEntry;
unsigned	uDiff;
int			iLine;
int			iRow;


	if(!pData->cHasFocus)
		{
		UNLOCK(pData);
		SetFocus(pData->hWnd);
		LOCK(pData);
		}

	iLine  = iPosY/pData->iRowHeight+pData->uScrollY;

	if(U(iLine)>=pData->uListItemsCount)return;

	iPosX -= pData->uStartText;

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)					// Umschalten der Auswahl
	if(iPosX<-4 && iPosX>-SELECT_SIZE+4)
		{
		ColorListBoxSetSel(pData,iLine,2,TRUE);
		pData->uCheckedDown=1;
		return;
		}



	if(pData->uStyleEx&CLB_EX_SELWORDDBL)					// Die ganze Zeile auswhlen (3fach Click)
	if(pData->iMarkType==2 && pData->uLastDblClick)
		{
		if(pData->iLastDblClickX!=iPosX || pData->iLastDblClickY!=iPosY)
			{
			pData->uLastDblClick = 0;
			}
		else{
			   uDiff = GetTickCount()-pData->uLastDblClick;
			if(uDiff<300)
				{
				pData->uLastDblClick = 0;
				pEntry = pData->pListItems+iLine;
				iRow   = pEntry->wLineLen;

				if(pData->iMarkStartX!=0     || pData->iMarkStopX!=iRow ||
				   pData->iMarkStartY!=iLine || pData->iMarkStopY!=iLine)
					{
					SaveMark(pData);

					pData->iMarkStartX = 0;
					pData->iMarkStopX  = iRow;
					pData->iMarkStartY = iLine;
					pData->iMarkStopY  = iLine;
					pData->iMarkMode   = 2;
					pData->iMarkMove   = 1;

					UpdateMark(pData);
					return;
					}
				}
			}
		}


	   iRow = RowOfPoint(pData,iLine,iPosX,1);
	if(iRow<0)iRow=0;

															// Auf Drag prfen
	if((pData->uStyleEx&CLB_EX_DRAGITEMS) && pData->cDragFlag==0)
		{
		if(pData->iMarkType!=0 && pData->iMarkMode==2)
		if(pData->iMarkStartY<iLine || (pData->iMarkStartY==iLine && pData->iMarkStartX<=iRow))
		if(pData->iMarkStopY >iLine || (pData->iMarkStopY ==iLine && pData->iMarkStopX > iRow))
			{
			pData->iDragFlags	= nFlags;
			pData->iDragPosX	= iPosX;
			pData->iDragPosY	= iPosY;
			pData->iDragItem	= iLine;
			pData->iDragRow		= iRow;
			pData->cDragFlag	= 2;
			return;
			}

		if(pData->iMarkType==0)								// Nur wenn keine Auswahl erlaubt ist
			{
			pData->iDragFlags	= nFlags;
			pData->iDragPosX	= iPosX;
			pData->iDragPosY	= iPosY;
			pData->iDragItem	= iLine;
			pData->iDragRow		= iRow;
			pData->cDragFlag	= 1;
			return;
			}
		}
	else{
		pData->cDragFlag = 0;
		}


	SetCapture(pData->hWnd);
	SaveMark(pData);

	pData->iMarkMove = 0;

	if(pData->iMarkType>=1)
		{
		pData->iMarkMode   = 1;								// Beginn einer Marke
		pData->iMarkStartX = (pData->iMarkType==3)? 0:iRow;
		pData->iMarkStopX  = (pData->iMarkType==3)? 0:iRow;
		pData->iMarkStartY = iLine;
		pData->iMarkStopY  = iLine;
		}

	if(pData->cHasCaret)
	if(pData->iCursorX!=iRow || pData->iCursorY!=iLine)		// Eine neue Cursorposition setzen
		{
		pData->iCursorX	   = iRow;
		pData->iCursorY	   = iLine;
		UpdateCaret(pData);
		}


	if(U(iLine)!=pData->uSelectedLine)						// Die Zeile auswhlen
		{
		ColorListBoxSelectItem(pData,iLine);
		}


	UpdateMark(pData);

}

//*****************************************************************************
//*
//*		ColorListBoxLButtonDblClk
//*
//*****************************************************************************
//	Doppelklick mit linker Maustatste.
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxLButtonDblClk(ListBoxData *pData,UINT nFlags,int iPosX,int iPosY)
{
CLB_ENTRY	sData;
int			iLine;
int			iPos1;
int			iPos2;
WORD		wId;



	iLine = iPosY/pData->iRowHeight+pData->uScrollY;

	if(U(iLine)<pData->uListItemsCount)
		{
		iPosX -= pData->uStartText;
															// Umschalten der Auswahl
		if((iPosX<-4 && iPosX>-SELECT_SIZE+4) || (pData->uStyleEx&CLB_EX_SELDBLCLICK))
			{
			ColorListBoxSetSel(pData,iLine,2,TRUE);
			pData->uCheckedDown = 1;
			}

		if(pData->uStyleEx&CLB_EX_SELWORDDBL)				// Das Wort auswhlen
		if(pData->iMarkType==1 || pData->iMarkType==2)
		if(iPosX>=0)
			{
			SaveMark(pData);

					GetDataEntry(pData,iLine,&sData);
			iPos1 = RowOfPoint  (pData,iLine, iPosX);
			iPos2 = iPos1;


			while(iPos1>0 && IsChar(sData.pText[iPos1-1]))
				{
				iPos1--;
				}

			while(iPos2<sData.iSize && IsChar(sData.pText[iPos2]))
				{
				iPos2++;
				}

			if(iPos1==iPos2)
				{
				while(iPos1>0 && IsSign(sData.pText[iPos1-1]))
					{
					iPos1--;
					}

				while(iPos2<sData.iSize && IsSign(sData.pText[iPos2]))
					{
					iPos2++;
					}
				}

			if(iPos1!=iPos2)
			if(pData->iMarkStartX!=iPos1 || pData->iMarkStopX!=iPos2 ||
			   pData->iMarkStartY!=iLine || pData->iMarkStopY!=iLine)
				{
				pData->iMarkStartX = iPos1;
				pData->iMarkStopX  = iPos2;
				pData->iMarkStartY = iLine;
				pData->iMarkStopY  = iLine;
				pData->iMarkMode   = 2;
				pData->iMarkMove   = 1;

				UpdateMark(pData);
				}

			pData->uLastDblClick   = GetTickCount();
			pData->iLastDblClickX  = iPosX;
			pData->iLastDblClickY  = iPosY;
			}
		}

	UNLOCK(pData);
	wId = (WORD)GetWindowLong(pData->hWnd,GWL_ID);
	SendMessage(GetParent(pData->hWnd),WM_COMMAND,MAKELONG(wId,LBN_DBLCLK),(LPARAM)pData->hWnd);
	LOCK(pData);


}

//*****************************************************************************
//*
//*		ColorListBoxRButtonDown
//*
//*****************************************************************************
//	Klick mit rechter Maustatste.
//	pData		: Zeiger auf die Fensterdaten
//	Ergibt 1 wenn ein Context-Men angezeigt wird
static int ColorListBoxRButtonDown(ListBoxData *pData,UINT nFlags,int iPosX,int iPosY)
{
HMENU	hMenu;
RECT	sRect;


	if(!pData->cHasFocus)
		{
		UNLOCK(pData);
		SetFocus(pData->hWnd);
		LOCK(pData);
		}

	if(!pData->iMarkMode)
		{
		ColorListBoxLButtonDown(pData,nFlags,iPosX,iPosY);
		ColorListBoxLButtonUp  (pData,nFlags,iPosX,iPosY);
		}

	if(pData->uStyleEx&CLB_EX_NOCONTEXTMENU)
		{
		return 0;
		}


	hMenu = CreatePopupMenu();

	if(!(pData->uStyleEx&CLB_EX_CHECKBOXES) || pData->iMarkType>=1)
		{
		AppendMenu(hMenu,MF_STRING,ID_COPY   ,LoadDefaultStrings(hMainInstance,IDS_TEXT_COPY));
		AppendMenu(hMenu,MF_STRING,ID_MARKALL,LoadDefaultStrings(hMainInstance,IDS_TEXT_MARK));
		EnableMenuItem(hMenu,ID_COPY   ,(pData->iMarkMode!=0                          )? MF_ENABLED:MF_DISABLED|MF_GRAYED);
		EnableMenuItem(hMenu,ID_MARKALL,(pData->iMarkType>=2 && pData->uListItemsCount)? MF_ENABLED:MF_DISABLED|MF_GRAYED);
		}

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)
		{
		AppendMenu(hMenu,MF_STRING,ID_SELALL ,LoadDefaultStrings(hMainInstance,IDS_TEXT_SELALL ));
		AppendMenu(hMenu,MF_STRING,ID_SELNONE,LoadDefaultStrings(hMainInstance,IDS_TEXT_SELNONE));
		}

	if(pData->uStyleEx&CLB_EX_CLEARALL)
		{
		AppendMenu(hMenu,MF_STRING,ID_CLEARALL,LoadDefaultStrings(hMainInstance,IDS_TEXT_CLEAR));
		EnableMenuItem(hMenu,ID_CLEARALL,(pData->uListItemsCount)? MF_ENABLED:MF_DISABLED|MF_GRAYED);
		}

	GetWindowRect(pData->hWnd,&sRect);

	iPosX += sRect.left;
	iPosY += sRect.top;

	UNLOCK(pData);
	TrackPopupMenu(hMenu,TPM_LEFTALIGN,iPosX,iPosY,0,pData->hWnd,NULL);
	LOCK(pData);

	DestroyMenu(hMenu);


return 1;
}


//*****************************************************************************
//*
//*		ColorListBoxMouseMove
//*
//*****************************************************************************
//	Wurde die Maus-Cursor-Position verndert.
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxMouseMove(ListBoxData *pData,UINT nFlags,int iPosX,int iPosY)
{
CLB_HITTESTINFO	sHitInfo;
CLB_DRAG		sNotify;
int				iDiffX;
int				iDiffY;
int				iLine;
int				iPos;
int				iRow;


	if(pData->cDragFlag)									// Auf Drag prfen
		{
		   iDiffX = pData->iDragPosX-iPosX;
		if(iDiffX<0)iDiffX=-iDiffX;
		   iDiffY = pData->iDragPosY-iPosY;
		if(iDiffY<0)iDiffY=-iDiffY;

		if(iDiffX>2 || iDiffY>2)							// Mehr als zwei Pixel-Verschub
			{
			sHitInfo.sPoint.x		= iPosX;
			sHitInfo.sPoint.y		= iPosY;
			ColorListBoxHitTest(pData,&sHitInfo);

			sNotify.sNmHdr.code		= CLB_BEGINDRAG;
			sNotify.sNmHdr.hwndFrom	= pData->hWnd;
			sNotify.sNmHdr.idFrom	= GetWindowLong(pData->hWnd,GWL_ID);
			sNotify.iMode			= pData->cDragFlag;
			sNotify.iItem			= pData->iDragItem;
			sNotify.iRow			= pData->iDragRow;
			pData->cDragFlag		= 0;

			if(SendNotify(pData,&sNotify.sNmHdr))			// Klick nicht weiterfhren
				{
				return;
				}
			}
		}

	iPosX -= pData->uStartText;

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)
	if(pData->cUseTheme==2)									// Mit Thema zeichnen
		{
		iPos = iPosX+pData->uScrollX;

		if(iPos<-4 && iPos>-20)								// Ist der Cursor auf der Checkbox
			{
			iLine  = iPosY/pData->iRowHeight+pData->uScrollY;
			if(U(iLine)>=pData->uListItemsCount)iLine=0xFFFFFFFF;
			}
		else{
			iLine  = 0xFFFFFFFF;
			}

		if(iLine^pData->uCheckedLine)						// Hat sich die selektierte Ckeckbox verndert
			{
			if(pData->uCheckedLine!=0xFFFFFFFF)
				{
				UpdateBox(pData,pData->uCheckedLine);
				}

			pData->uCheckedLine = iLine;

			if(pData->uCheckedLine!=0xFFFFFFFF)
				{
				if(!pData->uCheckedLine)
					{
					TRACKMOUSEEVENT sEvent;

					sEvent.cbSize		= sizeof(TRACKMOUSEEVENT);
					sEvent.dwFlags		= TME_LEAVE;
					sEvent.hwndTrack	= pData->hWnd;

					_TrackMouseEvent(&sEvent);
					}

				UpdateBox(pData,pData->uCheckedLine);
				}
			}
		}


	if(pData->iMarkMode!=1)return;

	iLine  = iPosY/pData->iRowHeight+pData->uScrollY;
	iRow   = RowOfPoint(pData,iLine,iPosX,1);


	SaveMark(pData);
	pData->iMarkMove++;

	if(pData->iMarkType==3)									// Nur ganzzeilige Markierungen
		{
		if(pData->iMarkStartY<iLine || (pData->iMarkStartY==iLine && pData->iMarkStartX<iRow))
			{
			iLine			   += 1;
			iRow				= 0;
			pData->iMarkStartX	= 0;
			}
		else{
			iRow				= 0;
			pData->iMarkStartX	= 0x8000;
			}
		}


	if(iLine>(int)pData->uListItemsCount)iLine=pData->uListItemsCount;
	if(iLine<0)iLine=0;
	if(iRow <0)iRow=0;

	if(pData->cHasCaret)
	if(pData->iCursorX!=iRow || pData->iCursorY!=iLine)		// Eine neue Cursorposition setzen
		{
		pData->iCursorX = iRow;
		pData->iCursorY = iLine;
		UpdateCaret(pData);
		}


	if(!pData->cTimerOn)									// Scroll Timer starten
		{
		iPosX += pData->uStartText;

		if(U(iLine)>=pData->uPageEnties || U(iPosX)>=pData->uSizeX)
			{
			pData->cTimerOn=1;
			SetTimer(pData->hWnd,ID_SCROLL,100,NULL);
			}
		}


	if(pData->iMarkType>=2 && pData->iMarkStopY!=iLine)pData->iMarkStopY=iLine;
	if(pData->iMarkType>=1 && pData->iMarkStopX!=iRow )pData->iMarkStopX=iRow;


	UpdateMark(pData);


}

//*****************************************************************************
//*
//*		OnMouseWheel
//*
//*****************************************************************************
//	Fenster ber Mausrad scrollen.
//	pData		: Zeiger auf die Fensterdaten
static BOOL ColorListBoxMouseWheel(ListBoxData *pData,UINT nFlags,short iDelta)
{
POINT	sPoint;
int		iCount;


	pData->iMarkMove++;

	if(nFlags&MK_CONTROL)
		{
		if(iDelta<0)ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_PAGEDOWN,iDelta),0);
		else		ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_PAGEUP  ,iDelta),0);
		}
	else{
		iCount=(nFlags&MK_SHIFT)? 9:3;

		for(;iCount>0;iCount--)
			{
			if(iDelta<0)ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_LINEDOWN,iDelta),0);
			else		ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_LINEUP  ,iDelta),0);
			}
		}

	if(pData->iMarkMode==1)
		{
		GetCursorPos(&sPoint);
		ScreenToClient(pData->hWnd,&sPoint);
		ColorListBoxMouseMove(pData,nFlags,sPoint.x,sPoint.y);
		}



return TRUE;
}


//*****************************************************************************
//*
//*		ColorListBoxScrollTimer
//*
//*****************************************************************************
//	Scroller fr eine Markierung
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxScrollTimer(ListBoxData *pData)
{
POINT	sPoint;
int		iLine;
int		iDiff;
int		iRow;



	if(pData->iMarkMode!=1)return;


	GetCursorPos(&sPoint);
	ScreenToClient(pData->hWnd,&sPoint);


	iRow   = (sPoint.x+pData->iFontWidth/2)/pData->iFontWidth;
	iLine  = (sPoint.y+pData->iRowHeight/2)/pData->iRowHeight;


	if(iRow<=0 && pData->uScrollX)
		{
		iDiff = (-iRow)/4;

		if(iDiff>1 || (iDiff==1 && (pData->cTimerOn&1)) || (iDiff<=0 && (pData->cTimerOn&3)==2))
			{
			ColorListBoxProc(pData->hWnd,WM_HSCROLL,MAKELONG(SB_LINEUP,1),0);
			}
		}

	if(iRow>=(int)pData->uSizeCharX && (int)pData->uScrollX<pData->iMaxSizeX+10-pData->uSizeX)
		{
		iDiff = (iRow-pData->uSizeCharX)/4;

		if(iDiff>1 || (iDiff==1 && (pData->cTimerOn&1)) || (iDiff<=0 && (pData->cTimerOn&3)==2))
			{
			ColorListBoxProc(pData->hWnd,WM_HSCROLL,MAKELONG(SB_LINEDOWN,1),0);
			}
		}

	if(iLine<=0 && pData->uScrollY)
		{
		iDiff = (-iLine)/4;

		if(iDiff>1 || (iDiff==1 && (pData->cTimerOn&1)) || (iDiff<=0 && (pData->cTimerOn&3)==2))
			{
			ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_LINEUP,-1),0);
			}
		}

	if(iLine>=(int)pData->uPageEnties)
		{
		iDiff = (iLine-pData->uPageEnties)/4;

		if(iDiff>1 || (iDiff==1 && (pData->cTimerOn&1)) || (iDiff<=0 && (pData->cTimerOn&3)==2))
			{
			ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKELONG(SB_LINEDOWN,1),0);
			}
		}


	ColorListBoxMouseMove(pData,0,sPoint.x,sPoint.y);


	if(pData->cTimerOn)pData->cTimerOn++;

}

//*****************************************************************************
//*
//*		ColorListBoxChar
//*
//*****************************************************************************
//	Eingabe von Buchstaben ber die WM_CHAR Nachricht
//	pData	: Zeiger auf die Fensterdaten
//	nChar	: Ist das Zeichen das eingegeben wurde
static void ColorListBoxChar(ListBoxData *pData,UINT nChar)
{
CLB_ENTRY		sData;
unsigned		uDelta;
unsigned 		uTick;
unsigned		uPos;
unsigned		uVal;
LPTSTR			pName;
int				iLine;
int				iNum;
int				iMax;




	if(nChar>=' ')
		{
		if(nChar==' ' && (pData->uStyleEx&CLB_EX_SELSPACE))	// Auswahl mit der SPACE-Taste
			{
			ColorListBoxSetSel(pData,pData->uSelectedLine,2,TRUE);
			return;
			}

		   iMax = pData->uListItemsCount;
		if(iMax<=0)return;

		if(!(pData->uStyleEx&CLB_EX_NOCHARSELCET))			// Suche Anfangsbuchstaben
			{
			uTick  = GetTickCount();
			uDelta = (uKeyPos>0)? 750:500;

			if(uKeyPos>=3           )uDelta=1000;
			if(uTick-uKeyLast>uDelta)uKeyPos=0;
			if(uKeyPos>=16          )uKeyPos=0;
			if(uKeyPos==0			)						// Hole die Startzeile
				{
				   iKeyItem = pData->uSelectedLine;
				if(iKeyItem<0)iKeyItem=0;
				}

			if((nChar>='a' && nChar<='z') ||				// In Grosbuchstaben umwandeln
			   (nChar>=224 && nChar<=254))
				{
				nChar-='a'-'A';
				}

			cKeyData[uKeyPos] = (TCHAR)nChar;
			uKeyLast          = uTick;
			uKeyPos          += 1;
			iNum			  =-1;

			for(iLine=iKeyItem+1;iLine!=iKeyItem;iLine++)	// Suche bereinstimmung
				{
				if(iLine>=iMax){iLine=-1;continue;}

				GetDataEntry(pData,iLine,&sData);

				pName = sData.pText;

				for(uPos=0;uPos<uKeyPos;uPos++)				// Vergleiche die Texte
					{
					#if UNICODE
					uVal = pName[uPos];
					#else
					uVal = ((unsigned char*)pName)[uPos];
					#endif

					if((uVal>='a' && uVal<='z') ||			// In Grosbuchstaben umwandeln
					   (uVal>=224 && uVal<=254))
						{
						uVal-='a'-'A';
						}

					if(cKeyData[uPos]!=(TCHAR)uVal)
						{
						break;
						}
					}

				if(uPos<uKeyPos)
					{
					if(iNum==-1 && uPos>0)iNum=iLine;		// Ersten Anfangsbuchstaben speichern
					continue;
					}

				if(pData->iMarkMode)						// Lschen der Marke
					{
					SaveMark(pData);

					pData->iMarkMode   = 0;
					pData->iMarkStartX = 0;
					pData->iMarkStopX  = 0;
					pData->iMarkStartY = 0;
					pData->iMarkStopY  = 0;

					UpdateMark(pData);
					}

				ColorListBoxEnsureVisible(pData,iLine,0);
				ColorListBoxSelectItem   (pData,iLine  );

				return;
				}

			if(iNum==-1)return;

			for(uPos=1;uPos<uKeyPos;uPos++)					// Anfangsbuchstaben durchlaufen
				{
				nChar = cKeyData[uPos];

				for(iLine=iNum+1;iLine!=iNum;iLine++)		// Suche Anfangsbuchstaben
					{
					if(iLine>=iMax){iLine=-1;continue;}

					GetDataEntry(pData,iLine,&sData);

					pName = sData.pText;

					#if UNICODE
					uVal = pName[0];
					#else
					uVal = ((unsigned char*)pName)[0];
					#endif

					if((uVal>='a' && uVal<='z') ||			// In Grosbuchstaben umwandeln
					   (uVal>=224 && uVal<=254))
						{
						uVal-='a'-'A';
						}

					if(cKeyData[uPos]==(TCHAR)uVal)
						{
						iNum = iLine;
						break;
						}
					}
				}

			if(pData->iMarkMode)							// Lschen der Marke
				{
				SaveMark(pData);

				pData->iMarkMode   = 0;
				pData->iMarkStartX = 0;
				pData->iMarkStopX  = 0;
				pData->iMarkStartY = 0;
				pData->iMarkStopY  = 0;

				UpdateMark(pData);
				}

			ColorListBoxEnsureVisible(pData,iNum,0);
			ColorListBoxSelectItem   (pData,iNum  );

			return;
			}
		}


}

//*****************************************************************************
//*
//*		ColorListBoxKeyDown
//*
//*****************************************************************************
//	Soll ein Tastendruck ausgefhrt werden. ACHTUNG UNLOCK wird ausgefhrt
//	pData	: Zeiger auf die Fensterdaten
//	wParam	: WPARAM des Mausklicks
//	lParam	: LPARAM des Mausklicks
//	Ergibt 0 wenn die Taste verarbeitet wurde
static int ColorListBoxKeyDown(ListBoxData *pData,WPARAM wParam,LPARAM lParam)
{
CLB_ENTRY	sData;
NMKEY		sNotify;
UINT		nChar;
UINT		nFlags;
UINT		nRepCnt;
int			iScrollNewX;
int			iScrollNewY;
int			iCurSelNew;
int			iCursorNew;
int			iChange;
int			iShift;
int			iType1;
int			iType2;
int			iTemp;
int			iPosX;
int			iPosY;
int			iLock;
int			iBit;
int			iRet;



	iRet		= 0;
	iLock		= 0;
	nChar		= (int)wParam;
	nRepCnt		= LOWORD(lParam);
	nFlags		= HIWORD(lParam);

	if(nChar==VK_CONTROL)return 1;

	iScrollNewX = pData->uScrollX;
	iScrollNewY = pData->uScrollY;
	iCursorNew  = pData->iCursorX;
	iCurSelNew  = (pData->uSelectedLine<0)? pData->iCursorY:pData->uSelectedLine;
	iShift      = GetAsyncKeyState(VK_SHIFT)&0x8000;


	for(;nRepCnt;nRepCnt--)
	switch(nChar)
		{
	case VK_RETURN:	if(!(pData->uStyleEx&CLB_EX_SELRETURN))	// Auswahl mit RETURN Taste
						{
						break;
						}

					ColorListBoxSetSel(pData,pData->uSelectedLine,2,TRUE);
					return 0;

	case VK_HOME:	if(pData->cHasCaret)					// An den Anfang der Liste
					if(!(GetAsyncKeyState(VK_CONTROL)&0x8000))
						{
						iCursorNew = 0;
						break;
						}

					iCurSelNew=0;
					iCursorNew=0;
					break;

	case VK_END:	if(pData->cHasCaret)					// Ans Ende der Liste
					if(!(GetAsyncKeyState(VK_CONTROL)&0x8000))
						{
						if(U(iCurSelNew)>=pData->uListItemsCount)
							{
							iCursorNew = 0;
							break;
							}

						GetDataEntry(pData,iCurSelNew,&sData);
						iCursorNew	= sData.iSize;
						break;
						}

					iCurSelNew = pData->uListItemsCount-1;

					if(U(iCurSelNew)<pData->uListItemsCount)
						{
						GetDataEntry(pData,iCurSelNew,&sData);
						iCursorNew	= sData.iSize;
						}
					else{
						iCursorNew=0;
						}

					break;

	case VK_UP:												// Eine Zeile aufwrts
					if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKEWPARAM(SB_THUMBTRACK,pData->uScrollY-1),0);
						break;
						}
															// Marke lschen
					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						ColorListBoxGetMark(pData,iPosX,iPosY,iTemp,iTemp);

						if(iCurSelNew>iPosY || (iCurSelNew==iPosY && iCursorNew>iPosX))
							{
							iCursorNew=iPosX;
							iCurSelNew=iPosY;
							break;
							}
						}

					if(iCurSelNew<=0)break;

					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						iPosY = (pData->iMarkStartY<pData->iMarkStopY)? pData->iMarkStartY:pData->iMarkStopY;

						if(iCurSelNew>iPosY)
							{
							iCurSelNew=iPosY;
							}
						}

					iCurSelNew--;
					iCursorNew = RowOfNewLine(pData,iCurSelNew,pData->iCursorY,pData->iCursorX);
					break;

	case VK_DOWN:											// Eine Zeile abwrts
					if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKEWPARAM(SB_THUMBTRACK,pData->uScrollY+1),0);
						break;
						}
															// Marke lschen
					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						ColorListBoxGetMark(pData,iTemp,iTemp,iPosX,iPosY);

						if(iCurSelNew<iPosY || (iCurSelNew==iPosY && iCursorNew<iPosX))
							{
							iCursorNew = iPosX;
							iCurSelNew = iPosY;
							break;
							}
						}

					if(U(iCurSelNew+1)>=pData->uListItemsCount)break;

					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						iPosY = (pData->iMarkStartY>pData->iMarkStopY)? pData->iMarkStartY:pData->iMarkStopY;

						if(iCurSelNew<iPosY)
							{
							iCurSelNew=iPosY;
							}
						}

					iCurSelNew++;
					iCursorNew = RowOfNewLine(pData,iCurSelNew,pData->iCursorY,pData->iCursorX);
					break;

	case VK_PRIOR:											// Eine Seite aufwrts
					if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKEWPARAM(SB_THUMBTRACK,pData->uScrollY-pData->uPageEnties),0);
						break;
						}

					iScrollNewY -= pData->uPageEnties-1;
					iCurSelNew  -= pData->uPageEnties-1;
					iCursorNew   = RowOfNewLine(pData,iCurSelNew,pData->iCursorY,pData->iCursorX);
					break;

	case VK_NEXT:											// Eine Seite abwrts
					if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKEWPARAM(SB_THUMBTRACK,pData->uScrollY+pData->uPageEnties),0);
						break;
						}

					iScrollNewY += pData->uPageEnties-1;
					iCurSelNew  += pData->uPageEnties-1;
					iCursorNew   = RowOfNewLine(pData,iCurSelNew,pData->iCursorY,pData->iCursorX);
					break;

	case VK_LEFT:											// Cursor nach Links
					if(!pData->cHasCaret)break;

															// Marke lschen
					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						ColorListBoxGetMark(pData,iPosX,iPosY,iTemp,iTemp);

						if(iCurSelNew>iPosY || (iCurSelNew==iPosY && iCursorNew>iPosX))
							{
							iCursorNew=iPosX;
							iCurSelNew=iPosY;
							break;
							}
						}

					if(iCursorNew==0 || U(iCurSelNew)>=pData->uListItemsCount)
						{
						iCurSelNew--;

						if(iCurSelNew<0)
							{
							iCurSelNew = 0;
							iCursorNew = 0;
							}
						else{
							GetDataEntry(pData,iCurSelNew,&sData);
							iCursorNew	= sData.iSize;
							}
						}
					else{
						GetDataEntry(pData,iCurSelNew,&sData);

						if(GetAsyncKeyState(VK_CONTROL)&0x8000)
							{
							iType1 = CharType(sData.pText[iCursorNew-1]);

							for(iCursorNew--;iCursorNew>0;iCursorNew--)
								{
								iType2 = CharType(sData.pText[iCursorNew-1]);
								iBit   = 1<<(iType1+iType2*5);
								if(pData->iKeyStepMask&iBit)break;
								iType1 = iType2;
								}
							}
						else{
							iCursorNew--;

							}
						}

					break;

	case VK_RIGHT:										// Cursor nach Rechts
					if(!pData->cHasCaret)break;
														// Marke lschen
					if(!iShift && pData->iMarkMode && pData->iMarkType!=3)
						{
						ColorListBoxGetMark(pData,iTemp,iTemp,iPosX,iPosY);

						if(iCurSelNew<iPosY || (iCurSelNew==iPosY && iCursorNew<iPosX))
							{
							iCursorNew = iPosX;
							iCurSelNew = iPosY;
							break;
							}
						}

					if(U(iCurSelNew)>=pData->uListItemsCount)
						{
						iCursorNew=0;
						}
					else{
						GetDataEntry(pData,iCurSelNew,&sData);

						if(iCursorNew>=sData.iSize)
							{
							if(U(iCurSelNew+1)>=pData->uListItemsCount)
								{
								iCursorNew = sData.iSize;
								break;
								}

							iCursorNew = 0;
							iCurSelNew++;
							break;
							}

						if(GetAsyncKeyState(VK_CONTROL)&0x8000)
							{
							iType1 = CharType(sData.pText[iCursorNew]);

							for(iCursorNew++;iCursorNew<sData.iSize;iCursorNew++)
								{
								iType2 = CharType(sData.pText[iCursorNew]);
								iBit   = 1<<(iType1*5+iType2);
								if(pData->iKeyStepMask&iBit)break;
								iType1 = iType2;
								}
							}
						else{
							iCursorNew++;
							}
						}

					break;


	case VK_CLEAR:	if(pData->cHasCaret==0)break;			// Ein Wort auswhlen
					if(pData->iMarkType==0)break;
					if(pData->iMarkType==3)break;
					if(U(iCurSelNew)>=pData->uListItemsCount)break;

					GetDataEntry(pData,iCurSelNew,&sData);
					iPosX		= pData->iCursorX;
					iCursorNew	= iPosX;
					iShift		= 1;

					while(iPosX>0 && IsChar(sData.pText[iPosX-1]))
						{
						iPosX--;
						}

					while(iCursorNew<sData.iSize && IsChar(sData.pText[iCursorNew]))
						{
						iCursorNew++;
						}

					if(iPosX==iCursorNew)
						{
						while(iPosX>0 && IsSign(sData.pText[iPosX-1]))
							{
							iPosX--;
							}

						while(iCursorNew<sData.iSize && IsSign(sData.pText[iCursorNew]))
							{
							iCursorNew++;
							}
						}

					if(iPosX!=iCursorNew)
					if(pData->iMarkStartX!=iPosX      || pData->iMarkStopX!=iCursorNew ||
					   pData->iMarkStartY!=iCurSelNew || pData->iMarkStopY!=iCurSelNew)
						{
						pData->iMarkMode = 0;
						pData->iCursorX  = iPosX;
						}

					break;

	case VK_TAB:

					if(pData->uStyleEx&CLB_EX_TABEXIT)
						{
						UNLOCK(pData);
						SetFocus(GetParent(pData->hWnd));
						LOCK(pData);
						}

					iLock = 1;

					break;

	case VK_INSERT:											// Markierten Text kopieren
	case 'C':		if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxCopy(pData);
						}

					iLock = 1;

					break;

	case 'A':												// Alles markieren
					if(GetAsyncKeyState(VK_CONTROL)&0x8000)
						{
						ColorListBoxMarkAll(pData);
						}

					iLock = 1;

					break;

	default:		iLock = 1;								// Taste wird nicht verarbeitet
					iRet  = 1;

					break;
		}


	sNotify.hdr.code		= NM_KEYDOWN;
	sNotify.hdr.hwndFrom	= pData->hWnd;
	sNotify.hdr.idFrom		= GetWindowLong(pData->hWnd,GWL_ID);
	sNotify.nVKey			= nChar;
	sNotify.uFlags			= (nFlags<<16)|(nRepCnt);

	if(SendNotify(pData,&sNotify.hdr))
		{
		iRet = 1;
		}


//*****************************************************************************


	if(pData->cHasCaret && !iLock)							// Eine neue Cursorposition setzen
	if(pData->iCursorX!=iCursorNew || pData->iCursorY!=iCurSelNew)
		{
		if(iShift && pData->iMarkType)
			{												// Marke nachziehen
			SaveMark(pData);

			iChange = 0;

			switch(pData->iMarkType)
				{
			case 3:	if(pData->iMarkStopY!=iCurSelNew)		// Nur ganzzeilige Marken
						{
						pData->iMarkStopY=iCurSelNew;
						iChange=1;
						}

					if(pData->iMarkMode==0)
						{
						if(pData->iMarkStartY!=pData->iCursorY){pData->iMarkStartY=pData->iCursorY;iChange=1;}
						}

					if(pData->iMarkStartY<=pData->iMarkStopY)
						{
						if(pData->iMarkStartX!=0x0000){pData->iMarkStartX=0x0000;iChange=1;}
						if(pData->iMarkStopX !=0x8000){pData->iMarkStopX =0x8000;iChange=1;}
						}
					else{
						if(pData->iMarkStartX!=0x8000){pData->iMarkStartX=0x8000;iChange=1;}
						if(pData->iMarkStopX !=0x0000){pData->iMarkStopX =0x0000;iChange=1;}
						}

					break;

			case 2:											// Mehrzeilige Marken
					if(pData->iMarkStopX!=iCursorNew){pData->iMarkStopX=iCursorNew;iChange=1;}
					if(pData->iMarkStopY!=iCurSelNew){pData->iMarkStopY=iCurSelNew;iChange=1;}

					if(pData->iMarkMode==0)
						{
						if(pData->iMarkStartX!=pData->iCursorX){pData->iMarkStartX=pData->iCursorX;iChange=1;}
						if(pData->iMarkStartY!=pData->iCursorY){pData->iMarkStartY=pData->iCursorY;iChange=1;}
						}

					break;

			case 1:	if(pData->iMarkStopX!=iCursorNew)		// Einzeilige Marken
						{
						pData->iMarkStopX=iCursorNew;
						iChange=1;
						}

					if(pData->iMarkMode==0)
						{
						if(pData->iMarkStartX!=pData->iCursorX){pData->iMarkStartX=pData->iCursorX;iChange=1;}
						if(pData->iMarkStartY!=iCurSelNew){pData->iMarkStartY=iCurSelNew;iChange=1;}
						if(pData->iMarkStopY !=iCurSelNew){pData->iMarkStopY =iCurSelNew;iChange=1;}
						}

					break;
				}

			pData->iCursorX = iCursorNew;
			pData->iCursorY = iCurSelNew;

			if(iChange)										// Hat sich die Marke verndert
				{
				if(pData->iMarkStartY!=pData->iMarkStopY || pData->iMarkStartX!=pData->iMarkStopX)
					{
					pData->iMarkMode=2;
					}
				else{
					pData->iMarkMode=0;
					}

				UpdateMark(pData);
				}
			else{
				UpdateCaret(pData);
				}
			}												// Marke lschen
		else if(pData->iMarkMode && U(iCurSelNew)==pData->uSelectedLine)
			{
			SaveMark(pData);

			pData->iCursorX    = iCursorNew;
			pData->iCursorY    = iCurSelNew;

			pData->iMarkStopX  = 0;
			pData->iMarkStartX = 0;
			pData->iMarkStopY  = 0;
			pData->iMarkStartY = 0;
			pData->iMarkMode   = 0;

			UpdateMark(pData);
			}
		else{
			pData->iCursorX    = iCursorNew;
			pData->iCursorY    = iCurSelNew;
			UpdateCaret(pData);
			}
															// X-Scroll-Position nach fhren
		iTemp = PointOfRow(pData,pData->iCursorY,pData->iCursorX);

		if(iTemp<(int)pData->uScrollX)
			{
			iScrollNewX = iTemp;
			}
		else if(iTemp+12>=(int)(pData->uSizeX+pData->uScrollX))
			{
			iScrollNewX = iTemp+12-pData->uSizeX;
			}

		if(pData->uScrollX<0)pData->uScrollX=0;


		if(U(iScrollNewX)!=pData->uScrollX)
			{
			ColorListBoxProc(pData->hWnd,WM_HSCROLL,MAKEWPARAM(SB_THUMBTRACK,iScrollNewX),0);
			}
		}


//*****************************************************************************


	if(U(iCurSelNew)!=pData->uSelectedLine)
		{
		if(pData->iMarkMode!=0 && !iShift)					// Den markierten Bereich lschen
			{
			SaveMark(pData);

			pData->iMarkStopX  = 0;
			pData->iMarkStartX = 0;
			pData->iMarkStopY  = 0;
			pData->iMarkStartY = 0;
			pData->iMarkMode   = 0;

			UpdateMark(pData);
			}

		if(iCurSelNew< 0)iCurSelNew=0;
		if(iCurSelNew>=(int)pData->uListItemsCount)
			{
			iCurSelNew = pData->uListItemsCount-1;
			}

		if(iCurSelNew>=0)
			{
			if(iScrollNewY                              > iCurSelNew          )iScrollNewY=iCurSelNew;
			if(iScrollNewY+ (int) pData->uPageEnties    <=iCurSelNew          )iScrollNewY=iCurSelNew-pData->uPageEnties+1;
			if(iScrollNewY>=(int)(pData->uListItemsCount-pData->uPageEnties/3))iScrollNewY=pData->uListItemsCount-pData->uPageEnties/3-1;
			if(iScrollNewY<0)iScrollNewY=0;

			if(U(iScrollNewY)!=pData->uScrollY)
				{
				ColorListBoxProc(pData->hWnd,WM_VSCROLL,MAKEWPARAM(SB_THUMBTRACK,iScrollNewY),0);
				}

			ColorListBoxSelectItem(pData,iCurSelNew);
			}
		}


return iRet;
}

//*****************************************************************************
//*
//*		ColorListBoxSortItems
//*
//*****************************************************************************
//	Sortiert die Kindereintrge eines Eintrages via Text
//	pData		: Zeiger auf die Fensterdaten
//	pSortData	: Ist ein Zeiger auf die Sortiertaten
//	iMode		: 1=Rekursiv sortieren
//	Ergibt 1 wenn ok sonst 0
static int ColorListBoxSortItems(ListBoxData *pData,LPARAM lSortParam,PFCLBCOMPARE pCompare)
{
#define XCHANGE_MEM(a,b)	{																			\
							DWORD *pPtr1=(DWORD*)a,*pPtr2=(DWORD*)b,uTemp,uOff;							\
							for(uOff=0;uOff<sizeof(BaseItem)/4;uOff++)									\
								{																		\
								uTemp=pPtr1[uOff];pPtr1[uOff]=pPtr2[uOff];pPtr2[uOff]=uTemp;			\
								}																		\
							}




BaseItem	   *pList;
BaseItem	   *pTemp;
BaseItem	   *pEntry;
LPARAM			lParam;
int				aMemL[30];
int				aMemU[30];
int				iLower;
int				iUpper;
int				iMiddle;
int				iStart;
int				iLevel;
int				iLast;
int				iCmp;




	pData->cLockChanges=1;
	UNLOCK(pData);

	pList		= pData->pListItems;
	iLast		= pData->uListItemsCount-1;
	iStart		= 0;
	iLevel		= 0;

	for(;;)
		{
		iLower	=  iStart;
		iMiddle = (iStart+iLast)>>1;						// Mitte berechnen
		iUpper	=  iLast +1;


		XCHANGE_MEM(iMiddle,iLower);

		pTemp   = pList+iStart;
		lParam	= pTemp->lParam;


		for(;;)
			{
			do  {
				iLower++;
				if(iLower>iLast)break;
				pEntry = pList+iLower;
				iCmp   = pCompare(pEntry->lParam,lParam,lSortParam);
				}while(iCmp<=0);

			do  {
				iUpper--;
				if(iUpper<=iStart)break;
				pEntry = pList+iUpper;
				iCmp   = pCompare(pEntry->lParam,lParam,lSortParam);
				}while(iCmp>=0);

			if(iUpper<iLower)break;

			XCHANGE_MEM(iUpper,iLower);
			}


		XCHANGE_MEM(iStart,iUpper);


		if(iUpper-1-iStart >= iLast-iLower )
			{
			if(iStart+1<iUpper)
				{
				aMemL[iLevel] = iStart;
				aMemU[iLevel] = iUpper-1;
				iLevel++;
				}

			if(iLower<iLast)
				{
				iStart = iLower;
				continue;
				}
			}
		else{
			if(iLower<iLast)
				{
				aMemL[iLevel] = iLower;
				aMemU[iLevel] = iLast;
				iLevel++;
				}

			if(iStart+1<iUpper)
				{
				iLast = iUpper-1;
				continue;
				}
			}
															// Eine Ebene absteigen
		iLevel--;

		if(iLevel>=0)										// Noch Ebenen vorhanden
			{
			iStart = aMemL[iLevel];
			iLast  = aMemU[iLevel];
			continue;
			}

		break;
		}


	LOCK(pData);
	pData->cLockChanges=0;
	UpdateView(pData);


return 1;
}

//*****************************************************************************
//*
//*		ColorListBoxSetTab
//*
//*****************************************************************************
//	Stellt die Tabulatorabstnde fr die Zeilen ein.
//	pData		: Zeiger auf die Fensterdaten
//	iTabs		: Ist die Anzahl der Tabulatoren im Feld.
//				  Wenn dieser Wert 1 ist werden konstante Abstnde eingestellt.
//	pTabStops	: Zeiger auf das Feld mit den Tabulatoren.
//				  Die Werte im Feld mssen aufsteigent sein.
//	bRedraw		: Muss das Fenster neugezeichnet werden
//	Ergibt 0 bei einem Fehler oder:
//					2 wenn die Tabulatorabstnde gesetzt wurden und es nderungen gab
//					1 wenn die Tabulatorabstnde gesetzt wurden ohne nderungen
static int ColorListBoxSetTab(ListBoxData *pData,int iTabs,INT *pTabStops)
{
int		iChg;
int		iOff;
int		iPos;
int		iNum;
int		iOld;
int		iTab;


	if(iTabs<1)return 0;


	if(iTabs==1)											// Gleiche Tabbreite fr alle Schritte
		{
		iTab			  = -pTabStops[0];
		iOld			  =  pData->iTabSize;
		pData->iTabSteps  =  iTab;
		pData->iTabSize   = (pData->iTabSteps<0)? -pData->iTabSteps:pData->iTabSteps*pData->iFontWidth;
		pData->aTabPos[0] =  pData->iTabSize;
		pData->iTabCount  =  1;
		if(pData->iTabSize<8)pData->iTabSize=8;				// Ist die Tab-Breite in Pixel

		if(iOld==pData->iTabSize)return 1;

		UpdateTextSize(pData);
		UpdateView(pData);


		return 2;
		}


	iOff = 0;
	iPos = 0;
	iChg = (pData->iTabCount==CLB_MAX_TABS)? 1:2;


	for(iNum=0;iNum<iTabs;iNum++)							// bergebene Tabs einfgen
		{
		if(pTabStops[iNum]<iOff)continue;
		if(pData->aTabPos[iPos]!=pTabStops[iNum])
			{
			pData->aTabPos[iPos] = pTabStops[iNum];
			iChg = 2;
			}

		iOff	= pTabStops[iNum];
		iPos   += 1;

		if(iPos>=CLB_MAX_TABS)break;
		}

	for(;iPos>=CLB_MAX_TABS;iPos++)							// Restliche Tabs auffllen
		{
		iOff += 64;
		iOff /= 64;
		iOff *= 64;

		if(pData->aTabPos[iPos]!=iOff)
			{
			pData->aTabPos[iPos] = iOff;
			iChg = 2;
			}
		}

	pData->iTabCount = CLB_MAX_TABS;
	pData->iTabSize	 = pData->iFontWidth*8;

	if(iChg==2)
		{
		UpdateTextSize(pData);
		UpdateView(pData);
		}



return iChg;
}

//*****************************************************************************
//*
//*		ColorListBoxFromPoint
//*
//*****************************************************************************
//	Sucht den nhesten Eintrag in der Listbox bei einem Punkt.
//	pData		: Zeiger auf die Fensterdaten
//	sPoint		: Sind die Koordinaten des Puntkes relativ zur linken
//				  oberen Ecke der Listbox.
//	pOutside	: Wenn dieser Zeiger ungleich 0 ist wird hier gespeichert
//				  ob der Punkt im Client-Bereich der Listbox ist.
//	Ergibt die Zeile des nhesten Eintrags.
static int ColorListBoxFromPoint(ListBoxData *pData,int iPosX,int iPosY,BOOL *pOutside)
{
int		 iLine;
RECT	 sRect;


	if(pOutside)											// Auerhalb des Fensters
		{
		sRect.top	 = 0;
		sRect.left	 = 0;
		sRect.right	 = pData->uSizeX;
		sRect.bottom = pData->uSizeY;

		if(iPosX<sRect.left || iPosX>=sRect.right ||
		   iPosY<sRect.top  || iPosY>=sRect.bottom)
			{
			*pOutside = TRUE;
			}
		else{
			*pOutside = FALSE;
			}
		}

   	iLine = iPosY/pData->iRowHeight;

	if(iLine<0)												// Ober dem Fenster
		{
		iLine = pData->uScrollY;
		}
	else if(iLine>=(int)pData->uPageEnties)					// Unter dem Fenster
		{
		iLine = pData->uScrollY+pData->uPageEnties-1;
		}
	else{
		iLine += pData->uScrollY;
		}

	if(iLine>=(int)pData->uListItemsCount)					// ber Zeilenanzahl
		{
		return pData->uListItemsCount-1;
		}


return iLine;
}

//*****************************************************************************
//*
//*		ColorListBoxMarkAll
//*
//*****************************************************************************
//	Markiert das ganze Fenster
//	pData		: Zeiger auf die Fensterdaten
static void ColorListBoxMarkAll(ListBoxData *pData)
{

	if(pData->iMarkStartY==0 && U(pData->iMarkStopY)==pData->uListItemsCount)
	if(pData->iMarkStartX==0 &&   pData->iMarkStopX==0)
		{
		return;
		}

	pData->iMarkMode   = (pData->uListItemsCount>0)? 2:0;
	pData->iMarkStopY  =  pData->uListItemsCount;
	pData->iMarkStartY =  0;
	pData->iMarkStopX  =  0;
	pData->iMarkStartX =  0;

	UpdateView(pData);

}

//*****************************************************************************
//*
//*		ColorListBoxCursorMark
//*
//*****************************************************************************
//	Setzt den Cursor oder die Marke
static int ColorListBoxCursorMark(ListBoxData *pData,int iPosX,int iPosY,int iMode=0)
{
CLB_ENTRY		sData;


	if(iPosY>=(int)pData->uListItemsCount)					// Prfe die Koordinaten
		{
		iPosX = 0;
		iPosY = pData->uListItemsCount;
		}

	if(iPosY<0)
		{
		iPosX = 0;
		iPosY = 0;
		}

	if(iPosX>0)
		{
		GetDataEntry(pData,iPosY,&sData);
		if(iPosX>sData.iSize)iPosX=sData.iSize;
		}

	if(iMode==1)											// Marke setzen
		{
		if(pData->iMarkStopX==iPosX)
		if(pData->iMarkStopY==iPosY)
			{
			return TRUE;
			}

		SaveMark(pData);

		pData->iMarkStartX = pData->iCursorX;
		pData->iMarkStartY = pData->iCursorY;
		pData->iMarkStopX  = iPosX;
		pData->iMarkStopY  = iPosY;

		if(pData->iMarkStartX!=pData->iMarkStopX || pData->iMarkStartY!=pData->iMarkStopY)
			{
			pData->iMarkMode = 2;
			}
		else{
			pData->iMarkMode = 0;
			}

		UpdateMark(pData);

		return TRUE;
		}
	else if(iMode==0)										// Cursor setzen
		{
		if(pData->iCursorX==iPosX)
		if(pData->iCursorY==iPosY)
			{
			return TRUE;
			}

		SaveMark(pData);

		pData->iCursorX    = iPosX;
		pData->iCursorY    = iPosY;
		pData->iMarkStartX = 0;
		pData->iMarkStartY = 0;
		pData->iMarkStopX  = 0;
		pData->iMarkStopY  = 0;
		pData->iMarkMode   = 0;

		if(pData->cHasCaret)
			{
			if(pData->uSelectedLine!=U(pData->iCursorY))
				{
				pData->iCursorY = pData->uSelectedLine;
				}

			UpdateCaret(pData);
			}

		if(pData->uSelectedLine^iPosY)
			{
			ColorListBoxSelectItem(pData,iPosY,TRUE);
			}

		UpdateMark(pData);

		return TRUE;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		ColorListBoxProc
//*
//*****************************************************************************
//	Ist die Fensterfunktion fr das ColorListBox Fenster
static LRESULT CALLBACK ColorListBoxProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
MSG			   *pMsg;
ListBoxData    *pData;
BaseItem	   *pEntry;
RECT			sRect;
CLB_ENTRY		sData;
SCROLLINFO		sScroll;
unsigned		uChange;
unsigned		uDelta;
unsigned		uFlag;
unsigned		uLine;
unsigned		uVal;
unsigned		uOld;
LPARAM			lRet;
int				iPos;
int				iMax;
HDC				hDc;




	switch(uMsg)
		{
	case WM_CREATE:												// Das Fenster erzeugen

				pData= new ListBoxData;
			if(!pData)return -1;

			memset(pData,0,sizeof(ListBoxData));


				pData->pListItems = new BaseItem[1];
			if(!pData->pListItems)
				{
				delete pData;
				return -1;
				}

			GlobalInit();

			SYNCH_CREATE(pData);
			SetWindowLongPtr(hWnd,0,(LONG_PTR)pData);
			pData->hWnd			 = hWnd;
			pData->uStyle		 = GetWindowLong(hWnd,GWL_STYLE);
			pData->uStyleEx		 = CLB_EX_MARKMULTI|CLB_EX_STEPMISC|CLB_EX_SELMARK;
			pData->aFonts[0]	 = hDefaultFontN;
			pData->aFonts[1]	 = hDefaultFontN;
			pData->aFonts[2]	 = hDefaultFontN;
			pData->aFonts[3]	 = hDefaultFontN;
			pData->cIsEnabled	 = (char)IsWindowEnabled(hWnd);
			pData->iKeyStepMask	 = iKeyMaskDefault;				// Maske fr Wortsprnge mit der Tastatur
			pData->uSelectedLine = 0xFFFFFFFF;
			pData->uCheckedLine	 = 0xFFFFFFFF;
			pData->aTabPos[0]	 = 32;
			pData->iTabSize		 = 32;
			pData->iTabSteps	 = 4;							// Ist die Tab-Breite in Buchstaben
			pData->iTabCount	 = 1;
			pData->iFontWidth	 = 1;
			pData->iFontHeight	 = 1;
			pData->iRowHeight	 = 1;
			pData->cIconFrame	 = 2;							// Extra Hhe fr Icons
			pData->cStateFrame	 = 2;							// Extra Hhe fr State und Overlay Bilder
			pData->iMarkType	 = 2;
			pData->cIconColor	 =-1;
			pData->cStateColor	 =-1;

			InitColors    (pData);
			UpdateFont    (pData);
			UpdateHeight  (pData);
			UpdateColors  (pData);
			UpdateScrollY (pData);
			UpdateTextSize(pData);

			return 0;

	case WM_DESTROY:											// Das Fenster zerstren

			pData = GetHandle(hWnd);

			LOCK(pData);

			ColorListBoxDeleteItems(pData,0,0x7FFFFFFF);

			if(!(pData->uStyleEx&CLB_EX_SHAREIMAGELISTS))
				{
				if(pData->hStates && pData->cStatesMode)
					{
					ImageList_Destroy(pData->hStates);
					}

				if(pData->hIcons && pData->cIconsMode)
					{
					ImageList_Destroy(pData->hIcons);
					}
				}

			if(pData->hTheme)
				{
				pCloseThemeData(pData->hTheme);
				}

			if(pData->iCaretOn)
				{
				HideCaret(hWnd);
				pData->iCaretOn=0;
				}

			if(pData->iCaretExist)
				{
				DestroyCaret();
				pData->iCaretExist=0;
				}

			delete pData->pListItems;

			UNLOCK(pData);

			SYNCH_DELETE(pData);
			memset(pData,0,sizeof(ListBoxData));
			delete pData;

			GlobalDeinit();


			return 0;

	case WM_SHOWWINDOW:											// Fenster ein/ausblenden

			if(wParam)
				{
				pData = GetHandle(hWnd);

				sScroll.cbSize		= sizeof(SCROLLINFO);
				sScroll.fMask		= SIF_ALL;
				sScroll.nMin		= 0;
				sScroll.nMax  		= 0;
				sScroll.nPage 		= 0;
				sScroll.nPos  		= 0;
				sScroll.nTrackPos	= 0;

				SetScrollInfo(pData->hWnd,SB_VERT,&sScroll,TRUE);
				SetScrollInfo(pData->hWnd,SB_HORZ,&sScroll,TRUE);

				LOCK(pData);

				pData->uOldXCount	= 0xFFFF;
				pData->uOldYCount	= 0xFFFF;

				UpdateScrollY(pData);
				UpdateScrollX(pData);

				UNLOCK(pData);
				}

			return 0;

	case WM_SIZE:												// Die Fenstergre wurde verndert

			pData = GetHandle(hWnd);
			uFlag = 0;

			LOCK(pData);

			   uVal = LOWORD(lParam);
			if(uVal && uVal!=pData->uSizeX)
				{
				uOld			= pData->uSizeX;
				pData->uSizeX	= uVal;

				if(uVal>uOld)									// Hat sich die Breite vergrert
					{
					GetClientRect (hWnd,&sRect);
					sRect.right  = uVal;
					sRect.left   = uOld;
					InvalidateRect(hWnd,&sRect,FALSE);
					}

				   iPos=uVal/pData->iFontWidth;
				if(iPos<=0)iPos=1;

				pData->uSizeCharX = iPos;

				UpdateScrollX(pData);
				}

			GetClientRect (hWnd,&sRect);
			uVal = sRect.bottom-sRect.top;

			if(uVal && uVal!=pData->uSizeY)
				{
				if(uVal>pData->uSizeY)							// Hat sich die Hhe vergrert
					{
					uFlag		  = 1;
					sRect.bottom  = uVal;
					sRect.top     = pData->uSizeY;
					InvalidateRect(hWnd,&sRect,FALSE);
					}

				pData->uSizeY		 =  uVal;
				pData->uMaxEnties	 = pData->uSizeY;
				pData->uMaxEnties	-= pData->uStartPixel;
				pData->uPageEnties	 = pData->uMaxEnties;
				pData->uMaxEnties	+= pData->iRowHeight-1;
				pData->uMaxEnties   /= pData->iRowHeight;
				pData->uPageEnties  /= pData->iRowHeight;
																// Wenn Hhe vergrert dann Scroll-Position prfen
				if(uFlag && pData->uPageEnties>2 && pData->uScrollY)
					{
					if(pData->uScrollY+pData->uPageEnties+1>pData->uListItemsCount)
						{
						   iPos = pData->uListItemsCount-pData->uPageEnties+1;
						if(iPos<0)iPos=0;
						if(U(iPos)!=pData->uScrollY)
							{
							pData->uScrollY = iPos;
							UpdateView(pData);
							}
						}
					}

				UpdateScrollY(pData);
				UpdateHeight (pData);
				}


			UNLOCK(pData);

			return 0;

	case WM_ENABLE:												// Das Fenster freischalten oder sperren

			   pData = GetHandle(hWnd);
			if(pData->cIsEnabled!=((U(wParam))? 1:0))
				{
				uVal  = GetWindowLong(hWnd,GWL_STYLE);
				uVal &= (0xFFFF0000&~WS_DISABLED);
				uVal |= (0x0000FFFF| WS_DISABLED)&(pData->uStyle^WS_DISABLED);
				SetWindowLong(hWnd,GWL_STYLE,uVal);
				}

			return 0;

	case WM_SETFOCUS:											// Das Fenster bekommt den Fokus

				pData = GetHandle(hWnd);
			if(!pData->cHasFocus)
				{
				LOCK(pData);

				pData->cHasFocus=1;

				if(pData->cHasCaret)							// Caret sichtbar machen
					{
					UpdateCaret(pData);
					}

				UpdateRows(pData,pData->uSelectedLine);

				UNLOCK(pData);

				uVal = (WORD)GetWindowLong(pData->hWnd,GWL_ID);
				SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(uVal,LBN_SETFOCUS),(LPARAM)pData->hWnd);
				}


			return 0;

	case WM_KILLFOCUS:											// Das Fenster bekommt den Fokus

			   pData = GetHandle(hWnd);
			if(pData->cHasFocus)
				{
				LOCK(pData);
				pData->cHasFocus=0;

				if(pData->cHasCaret)
					{
					UpdateCaret(pData);

					if(pData->iCaretExist)						// Caret lschen bei Focusverlust
						{
						pData->iCaretExist = 0;
						DestroyCaret();
						}
					}

				UpdateRows(pData,pData->uSelectedLine);

				UNLOCK(pData);

				uVal = (WORD)GetWindowLong(pData->hWnd,GWL_ID);
				SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(uVal,LBN_KILLFOCUS),(LPARAM)pData->hWnd);
				}

			return 0;

	case WM_COMMAND:
																// Nachrichtenempfang
			switch(LOWORD(wParam))
				{
			case ID_COPY:

				pData = GetHandle(hWnd);
				LOCK(pData);
				ColorListBoxCopy(pData);
				UNLOCK(pData);
				return 0;

			case ID_MARKALL:

				pData = GetHandle(hWnd);
				LOCK(pData);
				ColorListBoxMarkAll(pData);
				UNLOCK(pData);
				return 0;

			case ID_SELALL:

				pData = GetHandle(hWnd);
				LOCK(pData);

				for(uLine=0;uLine<pData->uListItemsCount;uLine++)
					{
					ColorListBoxSetSel(pData,uLine,1,TRUE);
					}

				UNLOCK(pData);
				return 0;

			case ID_SELNONE:

				pData = GetHandle(hWnd);
				LOCK(pData);

				for(uLine=0;uLine<pData->uListItemsCount;uLine++)
					{
					ColorListBoxSetSel(pData,uLine,0,TRUE);
					}

				UNLOCK(pData);
				return 0;

			case ID_CLEARALL:

				pData = GetHandle(hWnd);
				LOCK(pData);
				ColorListBoxDeleteItems(pData,0,0x7FFFFFFF);
				UNLOCK(pData);
				return 0;

			case ID_UPDATE:

				pData = GetHandle(hWnd);
				LOCK(pData);

				pData->cUpdateTimer = 1;
				SetTimer(pData->hWnd,ID_UPDATE,200,NULL);

				UNLOCK(pData);
				return 0;
				}

			break;

	case WM_MOUSELEAVE:											// Die Maus hat das Fenster verlassen

			wParam = 0x00000000;
			lParam = 0xFFFF0000;

	case WM_MOUSEMOVE:											// Gab es eine Mausbewegung

			pData = GetHandle(hWnd);
			LOCK(pData);
			ColorListBoxMouseMove(pData,U(wParam),GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);
			return 0;

	case WM_LBUTTONUP:											// Linken Mausklick aufheben

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(pData->cDragFlag)								// Down-Klick fr Drag nachholen
				{
				ColorListBoxLButtonDown(pData,pData->iDragFlags,pData->iDragPosX,pData->iDragPosY);
				}

			if(pData->uCheckedDown)
				{
				pData->uCheckedDown = 0;

				if(pData->uCheckedLine!=0xFFFFFFFF)
					{
					UpdateBox(pData,pData->uCheckedLine);
					}
				}

			ColorListBoxLButtonUp(pData,U(wParam),GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);
			return 0;

	case WM_LBUTTONDOWN:										// Klick mit linker Maustaste

			pData = GetHandle(hWnd);
			LOCK(pData);
			ColorListBoxLButtonDown(pData,U(wParam),GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);
			return 0;

	case WM_LBUTTONDBLCLK:										// Doppelklick mit linker Maustaste

			pData = GetHandle(hWnd);
			LOCK(pData);
			ColorListBoxLButtonDblClk(pData,U(wParam),GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);
			return 0;

	case WM_RBUTTONDOWN:										// Klick mit linker Maustaste

			pData	= GetHandle(hWnd);
			LOCK(pData);
			lRet = ColorListBoxRButtonDown(pData,U(wParam),GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);
			if(!lRet)break;
			return 0;

	case WM_KEYDOWN:											// Tastendruck

			pData	= GetHandle(hWnd);
			LOCK(pData);
			lRet    = ColorListBoxKeyDown(pData,wParam,lParam);
			UNLOCK(pData);

			if(lRet)DefWindowProc(hWnd,uMsg,wParam,lParam);
			return lRet;

	case WM_KEYUP:												// Tastendruck

			DefWindowProc(hWnd,uMsg,wParam,lParam);
			return 1;

	case WM_CHAR:												// Zeicheneingabe

			if((GetAsyncKeyState(VK_CONTROL)&0x8000))
				{
				DefWindowProc(hWnd,uMsg,wParam,lParam);
				return 1;
				}

			pData	= GetHandle(hWnd);
			iMax	= lParam&0xFFFF;
			LOCK(pData);

			for(iPos=0;iPos<iMax;iPos++)
				{
				ColorListBoxChar(pData,(UINT)wParam);
				}

			UNLOCK(pData);

			return 0;

	case WM_SETREDRAW:											// Das Zeichnen sperren

			pData	= GetHandle(hWnd);

			LOCK(pData);

			if(U(wParam)==0)
				{
				pData->cRedrawFlag = 1;
				UNLOCK(pData);
				break;
				}
			else if(pData->cRedrawFlag==0)
				{
				UNLOCK(pData);
				break;
				}

			pData->cRedrawFlag = 0;

			UNLOCK(pData);

			lRet = DefWindowProc(hWnd,uMsg,wParam,lParam);

			UpdateView      (pData);
			UpdateScrollBars(pData);

			return lRet;


	case WM_MOUSEWHEEL:											// Scrollen mit dem Mausrad

			pData	= GetHandle(hWnd);
			lRet	= ColorListBoxMouseWheel(pData,GET_X_LPARAM(wParam),GET_Y_LPARAM(wParam));

			return 0;

	case WM_HSCROLL:											// Vertikales scrollen

			pData	= GetHandle(hWnd);
			iPos	= pData->uScrollX;

			switch(LOWORD(wParam))
				{
			case SB_LINEDOWN:		iPos+=16;				break;
			case SB_LINEUP:			iPos-=16;				break;
			case SB_PAGEDOWN:		iPos+=pData->uSizeX;	break;
			case SB_PAGEUP:			iPos-=pData->uSizeX;	break;
			case SB_THUMBPOSITION:	iPos =HIWORD(wParam);  	break;
			case SB_THUMBTRACK:		iPos =HIWORD(wParam);  	break;
				}

			iMax  = pData->iMaxSizeX;
			iMax -=	pData->uSizeX/2;

			if(iPos>=iMax)iPos=iMax;
			if(iPos<0 || (pData->uStyle&CLBS_FIXEDSCROLL))iPos=0;
			if(iPos!=(int)pData->uScrollX)
				{
				pData->uScrollX = iPos;
				SetScrollPos(hWnd,SB_HORZ,iPos,TRUE);
				UpdateView(pData);
				}

			return 0;

	case WM_VSCROLL:											// Vertikales scrollen

			pData	= GetHandle(hWnd);
			iPos	= pData->uScrollY;

			switch(LOWORD(wParam))
				{
			case SB_LINEDOWN:		iPos++;													break;
			case SB_LINEUP:			iPos--;													break;
			case SB_PAGEDOWN:		iPos+=(pData->uPageEnties>1)? pData->uPageEnties-1:1;	break;
			case SB_PAGEUP:			iPos-=(pData->uPageEnties>1)? pData->uPageEnties-1:1;	break;
			case SB_THUMBPOSITION:
			case SB_THUMBTRACK:		if(pData->uListItemsCount<0x7F00)
										{
										iPos = HIWORD(wParam);
										}
									else{
										sScroll.cbSize = sizeof(SCROLLINFO);
										sScroll.fMask  = SIF_TRACKPOS;
										GetScrollInfo(hWnd,SB_VERT,&sScroll);
										iPos = sScroll.nTrackPos;
										}
				}

			iMax  = pData->uListItemsCount;
			iMax -=	pData->uPageEnties-1;

			if(iPos>=iMax)iPos=iMax;
			if(iPos<0 || (pData->uStyle&CLBS_FIXEDSCROLL))iPos=0;
			if(iPos!=(int)pData->uScrollY)
				{
				pData->uScrollY = iPos;
				SetScrollPos(hWnd,SB_VERT,iPos,TRUE);
				UpdateView(pData);
				}

			return 0;

	case WM_SYSCOLORCHANGE:										// Wurden die Systemfarben verndert

			pData	= GetHandle(hWnd);
			LOCK(pData);
			pData->cUseTheme=0;
			UpdateHeight(pData);
			UpdateColors(pData);
			UNLOCK(pData);
			return 0;

	case WM_SETFONT:											// Einen neuen Font zuweisen

			pData	= GetHandle(hWnd);

			if(pData->uStyleEx&CLB_EX_MULITFONT)				// Bei Multi-Fonts kann die Schrift nicht gendert werden
				{
				return 0;
				}

			LOCK(pData);

			if(wParam==1)										// Variabler Font
				{
				wParam = (WPARAM)hDefaultFontN;
				}
			else if(wParam==0)									// Font mit fixer Breite
				{
				if(hDefaultFontF==NULL)
					{
					hDefaultFontF = GetDefaultFont(2);
					}

				wParam = (WPARAM)hDefaultFontF;
				}

			pData->aFonts[0] 		= (HFONT)wParam;
			pData->aFonts[1] 		= (HFONT)wParam;
			pData->aFonts[2]		= (HFONT)wParam;
			pData->aFonts[3]		= (HFONT)wParam;
			pData->aFontOpaque[0]	= 0;
			pData->aFontOpaque[1]	= 0;
			pData->aFontOpaque[2]	= 0;
			pData->aFontOpaque[3]	= 0;
			pData->aFontAscent[0]	= 0;
			pData->aFontAscent[1]	= 0;
			pData->aFontAscent[2]	= 0;
			pData->aFontAscent[3]	= 0;

			if(UpdateFont(pData) && LOWORD(lParam))				// Schriftgre aktualisieren
				{
				pData->iRowHeight = 0;
				UpdateHeight(pData);
				UpdateView  (pData);
				}

			if(pData->iTabCount>1)
				{
				ColorListBoxSetTab(pData,pData->iTabCount,pData->aTabPos);
				}
			else if(pData->iTabSteps>0)
				{
				iPos = -pData->iTabSteps;
				ColorListBoxSetTab(pData,1,&iPos);
				}

			UNLOCK(pData);

			return 1;

	case WM_STYLECHANGED:										// Hat sich der Fensterstil gendert

			if(wParam==GWL_STYLE)
				{
				pData	= GetHandle(hWnd);
				lParam	= ((STYLESTRUCT*)lParam)->styleNew;

				LOCK(pData);
				uChange		   = U(lParam)^pData->uStyle;
				pData->uStyle  = U(lParam);

				if((uChange&CLBS_FIXEDSCROLL) && (U(lParam)&CLBS_FIXEDSCROLL))
				if(pData->uScrollX || pData->uScrollY)
					{
					pData->uScrollX = 0;
					pData->uScrollY = 0;

					UpdateScrollX(pData);
					UpdateScrollY(pData);
					UpdateView   (pData);
					}

				if(!(uChange&CLBS_MULTIPLESEL))
					{
					UpdateView(pData);
					}

				if(uChange&CLBS_NOSCROLL)
					{
					pData->uOldYCount = 0xFFFF;
					pData->uOldXCount = 0xFFFF;

					UpdateScrollX(pData);
					UpdateScrollY(pData);
					}

				if(uChange&WS_DISABLED)
					{
					pData->cIsEnabled=(char)((pData->uStyle&WS_DISABLED)? 0:1);
					UpdateView(pData);
					}

				UNLOCK(pData);
				}

			return 0;

	case WM_PAINT:												// Das Fenster zeichnen

			if(wParam)
				{
				ColorListBoxDraw(hWnd,(HDC)wParam,NULL);
				}
			else{
				PAINTSTRUCT sPaint;

				hDc = BeginPaint(hWnd,&sPaint);

				if(pBeginBufferedPt && pEndBufferedPt)
					{
					RECT	sRect;
					HDC		hDcBuffer;
					HANDLE	hBufferedPaint;

					GetClientRect(hWnd,&sRect);
					hBufferedPaint = pBeginBufferedPt(hDc,&sRect,BPBF_COMPATIBLEBM,NULL,&hDcBuffer);

					if(hBufferedPaint)
						{
						ColorListBoxDraw(hWnd,hDcBuffer,&sRect);
						pEndBufferedPt(hBufferedPaint,TRUE);
						}
					else{
						ColorListBoxDraw(hWnd,hDc,&sRect);
						}
					}
				else{
					ColorListBoxDraw(hWnd,hDc,&sPaint.rcPaint);
					}

				EndPaint(hWnd,&sPaint);
				}

			return 0;

	case WM_TIMER:												// Timer Funktion

			if(wParam==ID_SCROLL)
				{
				pData = GetHandle(hWnd);
				ColorListBoxScrollTimer(pData);
				return 0;
				}

			if(wParam==ID_UPDATE)
				{
				pData = GetHandle(hWnd);
				LOCK(pData);

				if(pData->cFastFlag)
					{
					pData->cFastFlag	  = 0;
					pData->cUpdateScrollY = 1;
					pData->uScrollY		  = pData->uScrollYtmp;
					}

				if(pData->cUpdateScrollX)
					{
					pData->cUpdateScrollX = 0;
					UpdateScrollX(pData);
					}

				if(pData->cUpdateScrollY)
					{
					pData->cUpdateScrollY = 0;
					UpdateScrollY(pData,TRUE);
					}

				if(pData->cUpdateCaret)
					{
					pData->cUpdateCaret   = 0;
					UpdateCaret(pData);
					}

				if(pData->cUpdateView)
					{
					pData->cUpdateView    = 0;
					UpdateView(pData);
					}

				UNLOCK(pData);
				return 0;
				}

			return 0;

	case WM_GETDLGCODE:											// Welche Tasten werden im Dialog benutzt

			pData = GetHandle(hWnd);

			if(pData->uStyle&CLBS_NOKEYBOARD)					// Keine Tastatureingaben erlauben
				{
				return 0;
				}

			   pMsg=(MSG*)lParam;
			if(pMsg && (pData->uStyleEx&CLB_EX_SELRETURN))
			if(pMsg->message==WM_KEYDOWN && pMsg->wParam==VK_RETURN)
				{
				return DLGC_WANTALLKEYS;
				}

			return DLGC_WANTARROWS|DLGC_WANTCHARS;

	case WM_COPY:												// Text kopieren

			pData = GetHandle(hWnd);
			LOCK(pData);
			ColorListBoxCopy(pData);
			UNLOCK(pData);
			return 0;

	case LB_ADDSTRING:											// Eine Zeile anhngen

			pData	= GetHandle(hWnd);
			wParam  = pData->uListItemsCount;

	case LB_INSERTSTRING:										// Eine Zeile einfgen

			pData	= GetHandle(hWnd);
			LOCK(pData);
			{
				CLB_ITEM	sItem;

				sItem.uMask		= CLBF_TEXT;
				sItem.iItem		= U(wParam);
				sItem.pText		= (LPTSTR)lParam;
				sItem.iTextMax	= -1;

				lRet = ColorListBoxInsertItem(pData,&sItem,0);
			}
			UNLOCK(pData);
			return lRet;

	case CLB_INSERTITEM:										// Eine Zeile einfgen mit Farbe

			pData	= GetHandle(hWnd);
			LOCK(pData);
			lRet	= ColorListBoxInsertItem(pData,(CLB_ITEM*)lParam,U(wParam));
			UNLOCK(pData);
			return lRet;

	case LB_SETCURSEL:											// Aktuelle Auswahl einstellen

			pData	= GetHandle(hWnd);
			LOCK(pData);
			lRet	= ColorListBoxSelectItem(pData,U(wParam));
			UNLOCK(pData);
			return lRet;

	case LB_SETSEL:												// Aktuelle Auswahl einstellen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxSetSel(pData,U(lParam),LOWORD(wParam),HIWORD(wParam));
			UNLOCK(pData);
			return lRet;

	case LB_SETTABSTOPS:										// Die Tabbreiten einstellen

			if(wParam<1)return FALSE;
			pData	= GetHandle(hWnd);
			LOCK(pData);
			lRet    = ColorListBoxSetTab(pData,U(wParam),(INT*)lParam);
			UNLOCK(pData);
			return TRUE;

	case CLB_SELECTALL:											// Alles ab- oder auswhlen

			pData	= GetHandle(hWnd);
			LOCK(pData);

			for(uLine=0;uLine<pData->uListItemsCount;uLine++)
				{
				ColorListBoxSetSel(pData,uLine,(int)wParam,TRUE);
				}

			UNLOCK(pData);
			return TRUE;

	case CLB_SETIMAGELIST:										// Die Image-Liste einstellen

			pData	= GetHandle(hWnd);
			uFlag   = U(wParam)&CLBIL_NOTDEL;

			LOCK(pData);

			switch(U(wParam)&~CLBIL_NOTDEL)
				{
			case CLBIL_STATE:	   lRet=(LPARAM)pData->hStates;
								if(lRet==lParam)break;

								if(pData->hStates && pData->cStatesMode)
									{
									ImageList_Destroy(pData->hStates);
									}

									pData->hStates=(HIMAGELIST)lParam;
								if(!pData->hStates)
									{
									pData->iStatesXsize = 0;
									pData->iStatesYsize = 0;
									pData->cStatesMode  = 0;
									}
								else{
									IMAGEINFO sInfo;
									ImageList_GetImageInfo(pData->hStates,0,&sInfo);
									pData->iStatesXsize = sInfo.rcImage.right -sInfo.rcImage.left;
									pData->iStatesYsize = sInfo.rcImage.bottom-sInfo.rcImage.top;
									pData->cStatesMode  = (uFlag)? 0:1;
									}

								pData->iRowHeight=1;
								UpdateHeight (pData);
								UpdateScrollY(pData);

								break;

			case CLBIL_ICONS:	   lRet=(LPARAM)pData->hIcons;
								if(lRet==lParam)break;

								if(pData->hIcons && pData->cIconsMode)
									{
									ImageList_Destroy(pData->hIcons);
									}

									pData->hIcons=(HIMAGELIST)lParam;
								if(!pData->hIcons)
									{
									pData->iIconsXsize = 0;
									pData->iIconsYsize = 0;
									pData->cIconsMode  = 0;
									}
								else{
									IMAGEINFO sInfo;
									ImageList_GetImageInfo(pData->hIcons,0,&sInfo);
									pData->iIconsXsize = sInfo.rcImage.right -sInfo.rcImage.left+ICONS_ADD;
									pData->iIconsYsize = sInfo.rcImage.bottom-sInfo.rcImage.top;
									pData->cIconsMode  = (uFlag)? 0:1;
									}

								pData->uStartText = (pData->uStyleEx&CLB_EX_CHECKBOXES)? SELECT_SIZE:0;
								if(pData->hIcons)pData->uStartText+=pData->iIconsXsize;

								pData->iRowHeight=1;
								UpdateHeight (pData);
								UpdateScrollY(pData);

								break;

			default:			lRet=0;
				}


			UNLOCK(pData);

			return lRet;


	case CLB_MARKALL:											// Alles markieren

			pData = GetHandle(hWnd);
			LOCK(pData);
			ColorListBoxMarkAll(pData);
			UNLOCK(pData);
			return 0;

	case CLB_GETCURSOR:											// Den Cursor holen

			pData = GetHandle(hWnd);
			LOCK(pData);
			if(wParam)*(LPINT)wParam=pData->iCursorX;
			if(lParam)*(LPINT)lParam=pData->iCursorY;
			UNLOCK(pData);
			return 1;

	case CLB_SETCURSOR:											// Den Cursor setzen
	case CLB_SETSELMARK:

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxCursorMark(pData,U(wParam),U(lParam),(uMsg==CLB_SETSELMARK)? 1:0);
			UNLOCK(pData);
			return lRet;

	case CLB_GETMARK:											// Die Marke abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet = ColorListBoxGetMark(pData,(int*)wParam,(int*)lParam);
			UNLOCK(pData);
			return lRet;

	case CLB_GETMARK_EX:										// Die Marke abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);
			{
			int *pMarks=(int*)lParam;
			lRet = ColorListBoxGetMark(pData,pMarks[0],pMarks[1],pMarks[2],pMarks[3]);
			}
			UNLOCK(pData);
			return lRet;

	case CLB_GETITEMIDENT:										// Die Texteinrckung abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)<pData->uListItemsCount)
				{
				pEntry = pData->pListItems+U(wParam);
				lRet   = pEntry->wIdent;
				}
			else{
				lRet   = 0;
				}

			UNLOCK(pData);

			return lRet;

	case CLB_SETITEMIDENT:										// Die Texteinrckung einstellen

			pData = GetHandle(hWnd);
			LOCK(pData);

			uLine = U(wParam);

			if(uLine<pData->uListItemsCount)
				{
				pEntry = pData->pListItems+uLine;

				if(pEntry->wIdent!=(WORD)lParam)
					{
					pEntry->wIdent = (WORD)lParam;

					if(!(U(lParam)&CLB_NOUPDATE))				// Sofort neuzeichnen
						{
						UpdateRows(pData,uLine,1);
						}										// Ist die Zeile sichtbar
					else if(uLine>=pData->uScrollY && uLine<pData->uListItemsCount+pData->uMaxEnties/2)
						{
						pData->cUpdateView	= 1;
						UpdateStart(pData);
						}
					}

				lRet = 1;
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case CLB_CALCSIZE:											// Die Breite eines Textes berechnen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ColorListBoxCalcSize(pData,U(wParam),(CLB_ITEM*)lParam);

			UNLOCK(pData);

			return 0;

	case CLB_GETITEMSTATE:										// Die Statusbits eines Eintrags abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)<pData->uListItemsCount)
				{
				pEntry = pData->pListItems+U(wParam);
				lRet   = pEntry->wLineSel;
				}
			else{
				lRet   = 0;
				}

			UNLOCK(pData);

			return lRet;

	case CLB_SETITEMSTATE:										// Die Statusbits eines Eintrags einstellen

			pData = GetHandle(hWnd);
			LOCK(pData);
			{
				CLB_ITEM	sItem;

				sItem.uMask		 = CLBF_STATE;
				sItem.uState	 = LOWORD(lParam);
				sItem.uStateMask = HIWORD(lParam);
				sItem.iItem		 = U(wParam);
				lRet = ColorListBoxSetItem(pData,&sItem,0);
			}
			UNLOCK(pData);
			return lRet;

	case CLB_ITEMFROMPOINT:										// Die Zeile von einem Punkt abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxFromPoint(pData,GET_X_LPARAM(wParam),GET_Y_LPARAM(wParam),(BOOL*)lParam);
			UNLOCK(pData);
			return lRet;

	case CLB_GETSELECTEDCOUNT:									// Anzahls der ausgewhlten Eintrge

			pData = GetHandle(hWnd);
			return pData->uSelectedCount;

	case CLB_GETSTRINGWIDTH:									// Abfrage der Breite eines Textes

			pData = GetHandle(hWnd);
			LOCK(pData);
			if(U(lParam)<pData->uListItemsCount)
					lRet = PointOfRow(pData,U(lParam));
			else	lRet = 0;
			UNLOCK(pData);
			return lRet;

	case CLB_GETNEXTITEM:										// Einen Eintrag suchen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet=ColorListBoxGetNextItem(pData,(int)wParam,U(lParam));
			UNLOCK(pData);
			return lRet;

	case CLB_GETITEM:											// Eintrge abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxGetItem(pData,(CLB_ITEM*)lParam);
			UNLOCK(pData);
			return lRet;

	case CLB_SETITEM:											// Eintrge einfgen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxSetItem(pData,(CLB_ITEM*)lParam,U(wParam));
			UNLOCK(pData);
			return lRet;

	case CLB_DELETEITEM:										// Einen Eintrag lschen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxDeleteItems(pData,U(wParam),1);
			UNLOCK(pData);
			return lRet;

	case CLB_DELETEALLITEMS:									// Alle Eintrge lschen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxDeleteItems(pData,0,0x7FFFFFFF);
			UNLOCK(pData);
			return lRet;

	case LB_RESETCONTENT:										// Alle Eintrge lschen

			lParam = 0xBDFFFFFF;
			wParam = 0x00000000;

	case LB_DELETESTRING:										// Eintrge lschen

			pData = GetHandle(hWnd);
			LOCK(pData);
			if((lParam&0xFF000000)!=0xBD000000)lParam=1;
			lRet  = ColorListBoxDeleteItems(pData,U(wParam),U(lParam)&0x00FFFFFF);
			UNLOCK(pData);
			return lRet;

	case LB_SELECTSTRING:										// Einen Eintrag via Text auswhlen
	case CLB_SELECTSTRINGEXACT:

			pData = GetHandle(hWnd);
			LOCK(pData);
			{
				CLB_FIND	sFind;
				sFind.uFlags = (uMsg==LB_SELECTSTRING)? CLB_FIND_TEXT|CLB_FIND_CASE:CLB_FIND_TEXT;
				sFind.pText	 = (LPCTSTR)lParam;
				lRet		 = ColorListBoxFindItem(pData,U(wParam),&sFind);
				if(lRet>=0)ColorListBoxSelectItem(pData,U(lRet));
			}
			UNLOCK(pData);
			return lRet;

	case LB_FINDSTRING:											// Nach einen Text suchen
	case LB_FINDSTRINGEXACT:

			pData = GetHandle(hWnd);
			LOCK(pData);
			{
				CLB_FIND	sFind;
				sFind.uFlags = (uMsg==LB_FINDSTRING)? CLB_FIND_TEXT|CLB_FIND_CASE:CLB_FIND_TEXT;
				sFind.pText	 = (LPCTSTR)lParam;
				lRet		 = ColorListBoxFindItem(pData,U(wParam),&sFind);
			}
			UNLOCK(pData);
			return lRet;

	case CLB_FINDITEM:											// Eintrge suchen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet  = ColorListBoxFindItem(pData,U(wParam),(CLB_FIND*)lParam);
			UNLOCK(pData);
			return lRet;

	case LB_SELITEMRANGE:										// Einen Bereich auswhlen
	case LB_SELITEMRANGEEX:

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(uMsg==LB_SELITEMRANGE)
				{
				iPos   = LOWORD(lParam);
				iMax   = LOWORD(lParam);
				}
			else{
				iPos   = U(wParam);
				iMax   = U(lParam);
				wParam = 1;
				}

			if(U(iPos)>=pData->uListItemsCount)
				{
				UNLOCK(pData);
				return -1;
				}

			if(U(iMax)>=pData->uListItemsCount)
				{
				iMax = pData->uListItemsCount-1;
				}

			pEntry = pData->pListItems+iPos;
			lRet   = 0;

			if(U(wParam))										// An- oder abwhlen
				{
				for(iPos=0;iPos<=iMax;iPos++,pEntry++)
					{
					if(pEntry->wLineSel&CLBS_SELECTED)continue;
					ColorListBoxSetSel(pData,iPos,1,TRUE);
					lRet++;
					}
				}
			else{
				for(iPos=0;iPos<=iMax;iPos++,pEntry++)
					{
					if(!(pEntry->wLineSel&CLBS_SELECTED))continue;
					ColorListBoxSetSel(pData,iPos,0,TRUE);
					lRet++;
					}
				}

			UNLOCK(pData);

			return lRet;

	case CLB_GETEXTENDEDSTYLE:									// Abfragen der erweiterten Style-Bits

			pData = GetHandle(hWnd);
			return pData->uStyleEx;

	case CLB_SETEXTENDEDSTYLE:									// Einstellen der erweiterten Style-Bits

				pData = GetHandle(hWnd);
			if(!wParam)wParam=0xFFFFFFFF;

			uVal  = pData->uStyleEx &~U(wParam);
			uVal |=	U(lParam)& U(wParam);

			if(pData->uStyleEx!=uVal)							// Gab es nderungen
				{
				LOCK(pData);
				uChange = pData->uStyleEx^uVal;
				pData->uStyleEx = uVal;

				if(uChange&CLB_EX_STEPALL)
					{
					if(uVal&CLB_EX_STEPUNTER)
						{
												pData->iKeyStepMask =iKeyMaskDefaultUl;
						if(uVal&CLB_EX_STEPMISC)pData->iKeyStepMask|=iKeyMaskExtentedUl;
						}
					else{
												pData->iKeyStepMask =iKeyMaskDefault;
						if(uVal&CLB_EX_STEPMISC)pData->iKeyStepMask|=iKeyMaskExtented;
						}

					if(uVal&CLB_EX_STEPUPPER)pData->iKeyStepMask |= iKeyMaskUpperCase;
					if(uVal&CLB_EX_STEPFIRST)pData->iKeyStepMask &= iKeyMaskFirst;
					}

				if(uChange&CLB_EX_GRIDLINES)
					{
					UpdateHeight (pData);
					UpdateScrollY(pData);
					}

				if(uChange&CLB_EX_NOTHEMES)
					{
					if(pData->uStyleEx&CLB_EX_CHECKBOXES)
						{
						UpdateHeight(pData);
						UpdateView  (pData);
						}
					}

				if(uChange&CLB_EX_CHECKBOXES)
					{
					pData->uStartText = (uVal&CLB_EX_CHECKBOXES)? SELECT_SIZE:0;
					if(pData->hIcons)pData->uStartText+=pData->iIconsXsize;

					UpdateScrollX(pData);
					UpdateHeight (pData);
					UpdateView   (pData);
					}

				if(uChange&CLB_EX_MARKMASK)
					{
					pData->iMarkType = pData->uStyleEx&CLB_EX_MARKMASK;

					if(pData->iMarkMode)
						{
						SaveMark(pData);

						pData->iMarkStartX = 0;
						pData->iMarkStartY = 0;
						pData->iMarkStopX  = 0;
						pData->iMarkStopY  = 0;
						pData->iMarkMode   = 0;

						UpdateMark(pData);
						}
					}

				if(uChange&CLB_EX_ALTERNATECOLOR)
					{
					UpdateView(pData);
					}

				if(uChange&CLB_EX_SELMARK)
					{
					if(pData->iMarkMode && pData->iMarkType)
						{
						if(pData->iMarkStartY<pData->iMarkStopY)
								UpdateRows(pData,pData->iMarkStartY,1+pData->iMarkStopY -pData->iMarkStartY);
						else	UpdateRows(pData,pData->iMarkStopY ,1+pData->iMarkStartY-pData->iMarkStopY );
						}
					}

				if(uChange&CLB_EX_CURSOR)
					{
					pData->cHasCaret = (uVal&CLB_EX_CURSOR)? 1:0;

					if(pData->cHasCaret)
					if(pData->uSelectedLine!=0xFFFFFFFF)
					if(pData->uSelectedLine!=U(pData->iCursorY))
						{
						pData->iCursorY = pData->uSelectedLine;
						}

					UpdateCaret(pData);
					}

				if(uChange&CLB_EX_MULITFONT)
					{
					if(pData->uStyleEx&CLB_EX_MULITFONT)
						{
						if(!hDefaultFontN)hDefaultFontN	= GetDefaultFont(0);
						if(!hDefaultFontF)hDefaultFontF = GetDefaultFont(2);
						
						pData->aFonts[0] = hDefaultFontN;
						pData->aFonts[1] = GetDefaultFont(1);
						pData->aFonts[2] = hDefaultFontF;
						pData->aFonts[3] = GetDefaultFont(3);
						
						UpdateAscent(pData);
						UpdateFont  (pData);
						UpdateHeight(pData);
						UpdateView  (pData);
						}
					else{
						pData->aFonts[1]        = pData->aFonts[0];
						pData->aFonts[2]        = pData->aFonts[0];
						pData->aFonts[3]        = pData->aFonts[0];
						pData->aFontOpaque[0]	= 0;
						pData->aFontOpaque[1]	= 0;
						pData->aFontOpaque[2]	= 0;
						pData->aFontOpaque[3]	= 0;
						pData->aFontAscent[0]	= 0;
						pData->aFontAscent[1]	= 0;
						pData->aFontAscent[2]	= 0;
						pData->aFontAscent[3]	= 0;
						}
					}

				UNLOCK(pData);
				}

			return uVal;

	case LB_SETCARETINDEX:										// Den Eintrag mit dem Focus auswhlen

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				UNLOCK(pData);
				return -1;
				}

			ColorListBoxSelectItem(pData,U(wParam));
			lRet = ColorListBoxEnsureVisible(pData,U(wParam),(LOWORD(lParam))? 1:0);
			UNLOCK(pData);
			return lRet;

	case LB_SETTOPINDEX:										// Ersten sichtbaren Eintrag bestimmen

			ColorListBoxProc(hWnd,WM_VSCROLL,MAKELONG(SB_THUMBTRACK,wParam),0);
			return 0;

	case CLB_GETCOLOR:											// Eine Farbe abfragen

			if(wParam<0 || wParam>=MAX_COLORS*2)return 0xFFFFFFFF;
			pData = GetHandle(hWnd);
			return pData->aColors[U(wParam)];

	case CLB_SETCOLOR:

			if(U(wParam)>=MAX_COLORS*2)return 0xFFFFFFFF;
			pData = GetHandle(hWnd);
			LOCK(pData);

			lRet  = pData->aColors[U(wParam)];

			if(U(lParam)==CLB_DEFCOLOR || U(lParam)==0xFFFFFFFF)// Standartfarbe einstellen
				{
				if(pData->cColorChanged[U(wParam)])
					{
					pData->cColorChanged[U(wParam)]=0;
					UpdateColors(pData);
					UpdateView  (pData);
					}
				}
			else{
				pData->cColorChanged[U(wParam)]=1;
				pData->aColors      [U(wParam)]=(COLORREF)lParam;

				if(lRet!=lParam)UpdateView(pData);
				}

			UNLOCK(pData);
			return lRet;

	case CLB_HITTEST:											// Abfragen von Koordinatenpositionen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet=ColorListBoxHitTest(pData,(CLB_HITTESTINFO*)lParam);
			UNLOCK(pData);
			return lRet;

	case LB_ITEMFROMPOINT:

			pData = GetHandle(hWnd);
			LOCK(pData);
			{
				CLB_HITTESTINFO	sInfo;

				sInfo.sPoint.x	= GET_X_LPARAM(lParam);
				sInfo.sPoint.y	= GET_Y_LPARAM(lParam);
				sInfo.uFlags	= U(wParam);

				lRet=ColorListBoxHitTest(pData,&sInfo);
			}
			UNLOCK(pData);
			return lRet;

	case LB_GETCOUNT:											// Anzahl der Eintrge abfragen

			pData = GetHandle(hWnd);
			return pData->uListItemsCount;

	case LB_GETCURSEL:											// Gewhlte Zeile abfragen

			pData = GetHandle(hWnd);
			return pData->uSelectedLine;

	case LB_GETCARETINDEX:										// Die Zeile mit dem Focus abfragen

			pData = GetHandle(hWnd);
			return pData->uSelectedLine;

	case LB_GETHORIZONTALEXTENT:								// X-Scroll-Bereich abfragen

			pData = GetHandle(hWnd);
			GetScrollInfo(hWnd,SB_HORZ,&sScroll);
			return sScroll.nMax;

	case LB_GETITEMDATA:										// Datenwert eines Eintrages abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				lRet   = -1;
				}
			else{
				pEntry = pData->pListItems+U(wParam);
				lRet   = pEntry->lParam;
				}

			UNLOCK(pData);
			return lRet;

	case LB_SETITEMDATA:										// Datenwert eines Eintrages einstellen

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				lRet   = -1;
				}
			else{
				lRet   =  0;
				pEntry = pData->pListItems+U(wParam);
				pEntry->lParam = lParam;
				}

			UNLOCK(pData);
			return lRet;

	case LB_GETITEMHEIGHT:										// Zeilenhhe abfragen

			pData = GetHandle(hWnd);
			return pData->iRowHeight;

	case LB_SETITEMHEIGHT:										// Zeilenhhe abfragen

			if(lParam!=-1)return LB_ERR;

			pData = GetHandle(hWnd);
			lRet  = pData->iRowHeight;

			if(wParam==-1)
				{
				LOCK(pData);
				UpdateHeight (pData);
				UpdateScrollY(pData);
				UNLOCK(pData);
				return lRet;
				}

			if(wParam<  1)wParam=  1;
			if(wParam>256)wParam=256;

			if(lRet!=(LPARAM)wParam)
				{
				LOCK(pData);
				pData->iRowHeight = (int)wParam;
				UpdateView(pData);
				UNLOCK(pData);
				}

			return lRet;

	case CLB_GETVISIBLECOUNT:									// Abfragen der sichtbaren Zeilen (ohne halbe Zeilen)

			pData = GetHandle(hWnd);
			return pData->uPageEnties;

	case CLB_ENSUREVISIBLE:										// Einen Eintrag ins Sichtfenster legen

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet=ColorListBoxEnsureVisible(pData,(int)wParam,U(lParam));
			UNLOCK(pData);
			return lRet;

	case CLB_ISITEMVISIBLE:										// Ist ein Eintrag sichtbar

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet=ColorListBoxIsVisible(pData,U(lParam));
			UNLOCK(pData);
			return lRet;

	case LB_GETITEMRECT:										// Das Rechteck eines Eintrages abfragen

			pData = GetHandle(hWnd);
			LOCK(pData);
			uVal=(!(U(wParam)&0xFC000000))? U(wParam)>>24:1;
			lRet=ColorListBoxGetItemRect(pData,U(wParam)&0x00FFFFFF,uVal,(RECT*)lParam);
			UNLOCK(pData);
			return lRet;

	case LB_GETSEL:												// Abfragen ob ein Eintrag ausgewhlt ist

			pData = GetHandle(hWnd);
			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				lRet   = -1;
				}
			else{
				pEntry = pData->pListItems+U(wParam);
				lRet   = pEntry->wLineSel&CLBS_SELECTED;
				}

			UNLOCK(pData);
			return lRet;

	case LB_GETSELCOUNT:										// Abfragen der ausgewhlten Eintrge

			pData = GetHandle(hWnd);
			if(pData->uStyle&CLBS_MULTIPLESEL)
				{
				return pData->uSelectedCount;
				}
			return -1;

	case LB_GETSELITEMS:										// Abfragen der gewhlten Eintrge

			pData = GetHandle(hWnd);

			if(!(pData->uStyle&CLBS_MULTIPLESEL))
				{
				return -1;
				}

			LOCK(pData);
			{
			unsigned   *pItems;
			unsigned	uLine;

				pItems	= (unsigned*)lParam;
				pEntry	= pData->pListItems;
				iMax	= U(wParam);
				iPos	= 0;
																// Suche alle selektierten Eintrge
				for(uLine=0;uLine<pData->uListItemsCount;uLine++,pEntry++)
					{
					if(!(pEntry->wLineSel&CLBS_SELECTED))continue;
					if(iPos>=iMax)break;
					pItems[iPos]=uLine;
					iPos++;
					}
			}
			UNLOCK(pData);

			return iPos;

	case LB_GETTEXT:											// Einen Text auslesen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				lRet= -1;
				memcpy((void*)lParam,"\x00",sizeof(TCHAR));
				}
			else{
				GetDataEntry(pData,U(wParam),&sData);
				memcpy((void*)lParam,sData.pText,sData.iSize*sizeof(TCHAR));
				((LPTSTR)lParam)[sData.iSize]=0;
				lRet = sData.iSize;
				}

			UNLOCK(pData);

			return lRet;

	case LB_GETTEXTLEN:											// Lnge eines Textes auslesen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(U(wParam)>=pData->uListItemsCount)
				{
				lRet= -1;
				}
			else{
				GetDataEntry(pData,U(wParam),&sData);
				lRet = sData.iSize;
				}

			UNLOCK(pData);

			return lRet;

	case CLB_SORTITEMS:											// Sortieren mit Funktion

			pData = GetHandle(hWnd);
			LOCK(pData);
			lRet=ColorListBoxSortItems(pData,wParam,(PFCLBCOMPARE)lParam);
			UNLOCK(pData);
			return lRet;

	case CLB_GETCOUNTPERPAGE:									// Hole die Anzahl der darstellbaren Reihen (mit halben Zeilen)

			pData = GetHandle(hWnd);
			return pData->uMaxEnties;

	case LB_GETTOPINDEX:										// Dan ersten sichtbaren Eintrag abfragen

			pData = GetHandle(hWnd);
			return (LRESULT)pData->uScrollY;

	case CLB_REDRAWITEMS:										// Eintrge neu zeichnen

			pData = GetHandle(hWnd);
			LOCK(pData);

			lRet = U(lParam)-U(wParam);

			if(lRet>0)
					{uVal=U(wParam);}
			else	{uVal=U(lParam);lRet=-lRet;}


			if(uVal<pData->uListItemsCount)
				{
				UpdateRows(pData,uVal,U(lRet));
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);
			return (lRet==0)? 0:1;

	case CLB_SCROLL:											// Das Fenster scrollen

			pData = GetHandle(hWnd);
			LOCK(pData);
			if(wParam)ColorListBoxProc(hWnd,WM_HSCROLL,MAKELONG(SB_THUMBPOSITION,pData->uScrollX+wParam),0);
			if(lParam)ColorListBoxProc(hWnd,WM_VSCROLL,MAKELONG(SB_THUMBPOSITION,pData->uScrollY+lParam),0);
			UNLOCK(pData);
			return 1;

	case CLB_GETSETOPTION:										// Diverse Optionen einstellen

			pData	= GetHandle(hWnd);
			lRet	= 0;

			switch(U(wParam)&~CLBO_WRITEOPTION)
				{
			case CLBO_WINDOWPROC:								// Fenster Funktion

				lRet = (LPARAM)ColorListBoxProc;
				break;

			case CLBO_FONTHEIGHT:								// Hhe des Fonts

				lRet = (LPARAM)pData->iFontHeight;
				break;

			case CLBO_FONTWIDTH:								// Breite des Fonts

				lRet = (LPARAM)pData->iFontWidth;
				break;

			case CLBO_TABSTEPS:									// Eingestellte Tabulatorbreite

				lRet = (LPARAM)pData->iTabSteps;
				break;

			case CLBO_STATEFRAME:								// Extra Hhe fr State-Bilder

				lRet = (LPARAM)pData->cStateFrame;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->cStateFrame = (char)lParam;
					if(pData->hStates)UpdateHeight(pData);

					UNLOCK(pData);
					}

				break;

			case CLBO_ICONEFRAME:								// Extra Hhe fr Icons

				lRet = (LPARAM)pData->cIconFrame;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->cIconFrame = (char)lParam;
					if(pData->hIcons)UpdateHeight(pData);

					UNLOCK(pData);
					}

				break;

			case CLBO_LINECALLBACK:								// Callback fr Daten

				lRet = (LPARAM)pData->pLineCallback;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->pLineCallback = (PFCLBLINECB)lParam;

					if(pData->pLineCallback)
						{
						UpdateView(pData);
						}

					UNLOCK(pData);
					}

				break;

			case CLBO_LINECBPARAM:								// Parameter fr Callback

				lRet = (LPARAM)pData->pLineCbParam;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->pLineCbParam = (LPVOID)lParam;

					UNLOCK(pData);
					}

				break;

			case CLBO_ICONCOLOR:								// Hintergund fr Icons

				lRet = (LPARAM)pData->cIconColor;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->cIconColor = (CHAR)lParam;

					UNLOCK(pData);
					}

				break;

			case CLBO_STATECOLOR:								// Hintergund fr Overlay-Icons

				lRet = (LPARAM)pData->cStateColor;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->cStateColor = (CHAR)lParam;

					UNLOCK(pData);
					}

				break;

			case CLBO_BASEIDENT:								// Standart-Text-Einrckung 

				lRet = (LPARAM)pData->wIdent;

				if((wParam&CLBO_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->wIdent = (WORD)lParam;

					UNLOCK(pData);
					}

				break;
				}

			return lRet;

	case CLB_GETISEARCHSTRING:									// Holt den aktuellen Suchtext

			uDelta = GetTickCount()-uKeyLast;

			if(!lParam)
				{
				return (uDelta<=750)? uKeyPos:0;
				}

			if(uDelta>750)
				{
				((TCHAR*)lParam)[0]=0;
				return FALSE;
				}

			memcpy((TCHAR*)lParam,cKeyData,uKeyPos*sizeof(TCHAR));
			((TCHAR*)lParam)[uKeyPos]=0;
			return TRUE;

	case CLB_GETUNICODEFORMAT:									// Wird gerade UNI-Code verwendet

			#if UNICODE
			return 1;
			#else
			return 0;
			#endif

		}


return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


//*****************************************************************************
//*
//*		ColorListBoxDraw
//*
//*****************************************************************************
//	Zeichnet das Fenster
static void ColorListBoxDraw(HWND hWnd,HDC hDc,RECT *pRect)
{
HRGN			hRgnMain;
DWORD			uSize;
RECT			sRect;
RECT			sArea;
BYTE			bBack;
BYTE			bText;
BYTE			bDone;
BYTE			bMask;
BYTE			bBits;
BYTE			bUsed;
ListBoxData    *pData;
BaseItem	   *pEntry;
CLB_ENTRY		sData;
COLORREF		uColor;
COLORREF		uColText;
unsigned		uStartCol;
unsigned		uStyleEx;
unsigned		uEndCol;
unsigned		uStyle;
unsigned		uState;
unsigned 		uLine;
unsigned 		uFont;
unsigned		uPos;
unsigned		uMax;
unsigned		uNum;
int				iXscroll;
int				iHeight;
int				iStartX;
int				iStopX;
int				iStartY;
int				iStopY;
int				iPosY;
int				iMaxX;
int				iPosX;
int				iDiff;
int				iValT;
int				iValB;
int				iNum;
int				iOff;
int				iPos;
int				iAdd;
int				i;



	pData=GetHandle(hWnd);

	LOCK(pData);

	GetClientRect(hWnd,&sRect);

	if(!pRect)pRect=&sRect;


	hRgnMain	=  CreateRectRgn(pRect->left,pRect->top,pRect->right,pRect->bottom);
	iXscroll	=  -(int)pData->uScrollX;
	iHeight		=  pData->iRowHeight;
	uStyleEx 	=  pData->uStyleEx;
	uStyle	 	=  pData->uStyle;
	uPos	 	=  pData->uScrollY;
	uMax	 	=  pData->uMaxEnties+uPos;
	iMaxX		=  pData->iMaxSizeX;
	uStartCol	=  pRect->top/pData->iRowHeight;
	uEndCol		= (pRect->bottom+pData->iRowHeight-1)/pData->iRowHeight;


	if(uStyleEx&CLB_EX_GRIDLINES)
		{
		iHeight--;
		}

	if(uMax>pData->uListItemsCount)
		{
		uMax  = pData->uListItemsCount;
		}

	if(uMax>pData->uScrollY+uEndCol)
		{
		uMax  = pData->uScrollY+uEndCol;
		}



	SelectObject(hDc,pData->aFonts[0]);
	SetBkMode   (hDc,OPAQUE);

	uFont     = 0;


	if(pData->iMarkMode)										// Daten der Markierung
		{
		if(pData->iMarkStartY<pData->iMarkStopY || (pData->iMarkStartY==pData->iMarkStopY && pData->iMarkStartX<=pData->iMarkStopX))
			{
			iStartX = pData->iMarkStartX;
			iStartY = pData->iMarkStartY;
			iStopX  = pData->iMarkStopX;
			iStopY  = pData->iMarkStopY;
			}
		else{
			iStartX = pData->iMarkStopX;
			iStartY = pData->iMarkStopY;
			iStopX  = pData->iMarkStartX;
			iStopY  = pData->iMarkStartY;
			}
		}
	else{
		iStartX		= 0;
		iStopX 		= 0;
		iStartY		= 0;
		iStopY 		= 0;
		}



//******************** Eintrge zeichnen **************************************

	bMask   =(pData->uStyleEx&CLB_EX_MULITFONT)? 0xFF:0x3F;
	uLine	= pData->uScrollY+uStartCol;
	pEntry	= pData->pListItems+uLine;
	iPosY	= pData->iRowHeight*uStartCol;
	iOff	= pData->uStartText-pData->uScrollX;
	bDone	= 0;

	if(pData->uStyleEx&CLB_EX_CHECKBOXES)
		{
		SelectObject(hDc,GetStockObject(BLACK_PEN));
		SelectObject(hDc,GetStockObject(WHITE_BRUSH));
		}

	for(;uLine<uMax;uLine++,pEntry++)							// Alle sichtbaren Zeilen durchlaufen
		{
		GetDataEntry(pData,pEntry,&sData);

		iPosX	= 0;
		bBack	= 0;
		bText	= 0;

		if((int)uLine==pData->iCursorY)							// Den Cursor ausblenden
			{
			bDone = 1;

			if(pData->iCaretOn)
				{
				pData->iCaretOn = 0;
				HideCaret(hWnd);
				}
			}

		if(sData.iSize<=0)										// Kein Zeichen in der Zeile
			{
			sData.iSize  = 1;
			sData.pText  = _T(" ");
			sData.pColor = &bBack;
			}


		//************ Icon zeichnen ******************************************


		if(pData->hIcons)
			{
			bBack = (pData->cIconColor>=0)? pData->cIconColor:sData.pColor[0];

			if(pData->uSelectedLine==uLine && (pData->uStyleEx&CLB_EX_SELMARK))
				{
				bBack &= bMask&0xE0;
				bBack |= (pData->cHasFocus)? 1:2;
				}

			   uColor = pData->aColors[MAX_COLORS+bBack];
			if(uColor==0xFFFFFFFF)								// Standart Hintergrundfarbe whlen
				{
				   uNum = bBack*2;
				if(uNum>= 6)uNum=0;
				if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
					{
					uNum += uLine&1;
					}

				uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
				}

			SetBkColor(hDc,uColor);
			sArea.left	 = 0-pData->uScrollX;
			sArea.right	 = 0-pData->uScrollX+pData->iIconsXsize+1;
			sArea.top	 = iPosY;
			sArea.bottom = iPosY+pData->iRowHeight;


			if(pData->uStyleEx&CLB_EX_GRIDLINES)sArea.bottom--;

			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

			i = CLBS_ICON_GET(pEntry->wLineSel);
			ImageList_Draw(pData->hIcons,i,hDc,sArea.left+ICONS_ADD/2,iPosY+1,ILD_TRANSPARENT);
			}


		//************ Auswahlbox zeichnen ************************************


		if(pData->uStyleEx&CLB_EX_CHECKBOXES)
		if(pData->uScrollX<pData->uStartText)
			{
			bBack = (pData->cStateColor>=0)? pData->cStateColor:sData.pColor[0];

			if(pData->uSelectedLine==uLine && (pData->uStyleEx&CLB_EX_SELMARK))
				{
				bMask &= bMask&0xE0;
				bBack |= (pData->cHasFocus)? 1:2;
				}

			   uColor = pData->aColors[MAX_COLORS+bBack];
			if(uColor==0xFFFFFFFF)								// Standart Hintergrundfarbe whlen
				{
				   uNum = bBack*2;
				if(uNum>= 6)uNum=0;
				if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
					{
					uNum += uLine&1;
					}

				uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
				}

			SetBkColor(hDc,uColor);
			sArea.left	 = 0-pData->uScrollX;
			sArea.right	 = 0-pData->uScrollX+SELECT_SIZE+1;
			sArea.top	 = iPosY;
			sArea.bottom = iPosY+pData->iRowHeight;

			if(pData->hIcons)
				{
				sArea.left  += pData->iIconsXsize;
				sArea.right += pData->iIconsXsize;
				}
			else if(pData->uStyleEx&CLB_EX_GRIDLINES)
				{
				sArea.bottom--;
				}

			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

			if(!(pEntry->wLineSel&CLBS_NOBOX))					// Box zeichnen ?
				{
				if(pData->hStates)								// Icon zeichnen
					{
					i = pEntry->wLineSel&CLBS_OVERLAY;
					ImageList_Draw(pData->hStates,i,hDc,sArea.left+3,iPosY,ILD_TRANSPARENT);
					}
				else if(pData->cUseTheme==2)					// Mit Thema zeichnen
					{
					if(pData->uCheckedLine!=uLine)
						{
						uState = (pEntry->wLineSel&CLBS_SELECTED)?  CBS_CHECKEDNORMAL:CBS_UNCHECKEDNORMAL;
						}
					else if(pData->uCheckedDown)
						{
						uState = (pEntry->wLineSel&CLBS_SELECTED)?  CBS_CHECKEDPRESSED:CBS_UNCHECKEDPRESSED;
						}
					else{
						uState = (pEntry->wLineSel&CLBS_SELECTED)?  CBS_CHECKEDHOT:CBS_UNCHECKEDHOT;
						}

					   iDiff = sArea.bottom-sArea.top;
					   iDiff-= sThemeSize.cy;
					if(iDiff > 0)
						{
						iValT	      = sArea.top;
						iValB		  = sArea.bottom;
						sArea.top	 += (iDiff+1)/2;
						sArea.bottom  = sArea.top+sThemeSize.cy;

						pDrawThemeBackg(pData->hTheme,hDc,BP_CHECKBOX,uState,&sArea,0);

						sArea.top	  = iValT;
						sArea.bottom  = iValB;
						}
					else{
						pDrawThemeBackg(pData->hTheme,hDc,BP_CHECKBOX,uState,&sArea,0);
						}
					}
				else{
					Rectangle(hDc,0-pData->uScrollX+3,iPosY+1,sArea.left+3+11,iPosY+1+11);

					if(pEntry->wLineSel&CLBS_SELECTED)			// Hkchen zeichnen
						{
						SelectObject(hDc,hPen1);
						MoveToEx(hDc,sArea.left+4-2 ,iPosY+2+3,NULL);
						LineTo  (hDc,sArea.left+4+2 ,iPosY+2+7);
						LineTo  (hDc,sArea.left+4+12,iPosY+2+0);

						SelectObject(hDc,hPen2);
						MoveToEx(hDc,sArea.left+4-1 ,iPosY+2+2,NULL);
						LineTo  (hDc,sArea.left+4+2 ,iPosY+2+5);
						LineTo  (hDc,sArea.left+4+12,iPosY+2+0);
						SelectObject(hDc,GetStockObject(BLACK_PEN));
						}
					}
				}
			}


		iPosY += pData->iRowPre;
		
		//************ Text einrcken *****************************************
		

		if(sData.iIdent)
			{
			sArea.left	 = iPosX;
			sArea.right  = iPosX+pEntry->wIdent;
			sArea.top	 = iPosY;
			sArea.bottom = iPosY+pData->iRowHeight;

			if(pData->uSelectedLine==uLine && (pData->uStyleEx&CLB_EX_SELMARK))
				{
				bBack |= (pData->cHasFocus)? 1:2;

				   uColor = pData->aColors[MAX_COLORS+(bBack&CLB_COLOR_MASK)];
				if(uColor==0xFFFFFFFF)							// Standart Hintergrundfarbe whlen
					{
					   uNum = (bBack&CLB_COLOR_MASK)*2;
					if(uNum>= 6)uNum=0;
					if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
						{
						uNum += uLine&1;
						}

					uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
					}
				}
			else{												// Zeile ist nicht gewhlt
				if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
					{
					uNum += uLine&1;
					}
				else{
					uNum  = 0;
					}

				uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
				}
					
			SetBkColor (hDc,uColor);															
			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

			iPosX += pEntry->wIdent;
			}



		//************ Text ausgeben ******************************************

		
		for(i=0;i<sData.iSize;i+=iNum)							// Zeichne die Zeile
			{
			bBack  = sData.pColor[i];
			bText  = sData.pColor[i];
			uColor = pData->aColors[MAX_COLORS+(bBack&CLB_COLOR_MASK)];


			if(pData->uSelectedLine==uLine && (pData->uStyleEx&CLB_EX_SELMARK))
				{
				bUsed  = bMask;
				bBack &= bMask&0xE0;
				bBack |= (pData->cHasFocus)? 1:2;
				
																
				if(!(pData->aColors[bBack&CLB_COLOR_MASK]>>24))	// Gleiche Textfarbe fr ganze Auswahl
					{
					bText = bBack;
					bUsed = bMask&0xE0;
					}
				
				for(iNum=1;i+iNum<sData.iSize;iNum++)			// Zhle gleiche Farben
					{
					bBits  = sData.pColor[i+iNum]^bText;
					bBits &= bUsed;
					
					if(bBits==0x00)continue;
					if(bBits &0xE0)break;			

					if(sData.pText[i+iNum]!=' ')
					if(sData.pText[i+iNum]!='\t')
					if(sData.pText[i+iNum]!=_T('\xA0'))
						{
						break;									// Wenn kein ' ' sofort stoppen
						}
					}
				}
			else for(iNum=1;i+iNum<sData.iSize;iNum++)			// Zhle gleiche Farben
				{
				bBits  = sData.pColor[i+iNum]^bText;
				bBits &= bMask;

				if(bBits==0x00)continue;
				if(bBits &0xE0)break;			

				if(sData.pText[i+iNum]!=' ')
				if(sData.pText[i+iNum]!='\t')
				if(sData.pText[i+iNum]!=_T('\xA0'))
					{
					break;										// Wenn kein ' ' sofort stoppen
					}

				bUsed = sData.pColor[i+iNum]&CLB_COLOR_MASK;
				
				if(pData->aColors[MAX_COLORS+bUsed]!=uColor)	// Ist der Hintergrund anderst
					{
					break;
					}
				}


			if(pData->iMarkMode)								// Ist ein Text ausgewhlt
				{
				if(iStartY<(int)uLine && iStopY>(int)uLine)
					{
					iNum   = sData.iSize-i;
					bBack &=0xE0;
					bBack |= 3;
					}
				else if(iStartY==(int)uLine && iStopY==(int)uLine)
					{
					if(i<iStartX && i+iNum>iStartX)
						{
						iNum = iStartX-i;
						}

					if(i>=iStartX && i<iStopX )
						{
						if(iStopX<sData.iSize)
								iNum=iStopX-i;
						else	iNum=sData.iSize-i;

						bBack &=0xE0;
						bBack |= 3;
						}
					}
				else if(iStartY==(int)uLine)
					{
					if(i<iStartX && i+iNum>iStartX)
						{
						iNum = iStartX-i;
						}
					else if(i>=iStartX)
						{
						iNum   = sData.iSize-iStartX;
						bBack &=0xE0;
						bBack |= 3;
						}
					}
				else if(iStopY==(int)uLine && i<iStopX)
					{
					if(iStopX<sData.iSize)
							iNum=iStopX-i;
					else	iNum=sData.iSize-i;

					bBack &=0xE0;
					bBack |= 3;
					}

				if((bBack&0x1F)==3)								// Wurde etwas ausgewhlt
					{
					bUsed = bMask;

					if(pData->aColors[3]<=0x00FFFFFF)		
						{						
						bUsed &= 0xE0;
						bText &= 0xE0;
						bText |= 3;
						}

					for(iAdd=1;iAdd<iNum;iAdd++)				// Zhle gleiche Farben
						{
						if(i+iAdd>=sData.iSize)break;

						bBits  = sData.pColor[i+iAdd]^bText;
						bBits &= bUsed;

						if(sData.pText[i+iAdd]==' '       )bBits&=0xE0;
						if(sData.pText[i+iAdd]=='\t'      )bBits&=0xE0;
						if(sData.pText[i+iAdd]==_T('\xA0'))bBits&=0xE0;

						if(bBits)break;			
						}
					
					iNum = iAdd;
					}

				if(i+iNum>sData.iSize)
					{
					iNum = sData.iSize-i;
					}
				}


			   uColor = pData->aColors[MAX_COLORS+(bBack&CLB_COLOR_MASK)];
			if(uColor==0xFFFFFFFF)								// Standart Hintergrundfarbe whlen
				{
				   uNum = (bBack&CLB_COLOR_MASK)*2;
				if(uNum>= 6)uNum=0;
				if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
					{
					uNum += uLine&1;
					}

				uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
				}

			
			uColText = pData->aColors[bText&CLB_COLOR_MASK];
			SetTextColor(hDc,uColText);
			SetBkColor  (hDc,uColor  );


			if(i==0 && sData.pText[0]=='\t' && iOff>0 && !pData->hIcons &&!(pData->uStyleEx&CLB_EX_CHECKBOXES))
				{												// Erster Pixel hat ein Problem mit TAB's
				sArea.left	 = iPosX+iOff-1;
				sArea.right	 = iPosX+iOff+1;
				sArea.top	 = iPosY;
				sArea.bottom = iPosY+pData->iRowHeight;

				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
				}

			if(iNum<=0)iNum=1;


			if((UINT)((bText&bMask)>>6)!=uFont)					// Eine andere Schrift benutzen
				{
				uFont = bText>>6;
				SelectObject(hDc,pData->aFonts[uFont]);
				}

			if(pData->aFontOpaque[uFont])						// Text Tranparent zeichnen
				{
				SetBkMode(hDc,TRANSPARENT);

				uSize  = GetTabbedSizeEx(hDc,iPosX+iOff,sData.pText+i,sData.pColor+i,iNum,pData->iTabCount,pData->aTabPos,iOff,pData->aFonts);

				sArea.left	  = iPosX+iOff+1;
				sArea.right   = iPosX+iOff+1+uSize;
				sArea.top     = iPosY;
				sArea.bottom  = iPosY+pData->iFontHeight+1;
																// Hintergrund ausmalen
				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
				
				uSize  = TabbedTextOut(hDc,iPosX+iOff,iPosY-pData->aFontAscent[uFont],sData.pText+i,iNum,pData->iTabCount,pData->aTabPos,iOff);
				uSize &= 0xFFFF;
				
				SetBkMode(hDc,OPAQUE);
				}
			else{												// Text mit Hintergrund zeichnen
				uSize  = TabbedTextOut(hDc,iPosX+iOff,iPosY,sData.pText+i,iNum,pData->iTabCount,pData->aTabPos,iOff);
				uSize &= 0xFFFF;
				}

			if(bBack&CLB_UNTERLINE)								// Text unterstreichen
				{
				sArea.left	  = iPosX+iOff-1;
				sArea.right   = iPosX+iOff+uSize;
				sArea.top     = iPosY+pData->iFontLine;
				sArea.bottom  = sArea.top+1;

				SetBkColor(hDc,uColText);
				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
				}

			if(pData->iRowPost)									// Restliche Zeile fllen falls der Font niedrig ist
				{
				sArea.left	 = iPosX+iOff;
				sArea.right	 = iPosX+iOff+uSize+1;
				sArea.top	 = iPosY+pData->iFontHeight;
				sArea.bottom = iPosY+pData->iFontHeight+pData->iRowPost;

				if(sData.pText[i]!='\t')
					{
					sArea.left	+= pData->iFontOff;
					sArea.right	+= pData->iFontOff;
					}

				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
				}

			if(pData->iRowPre)									// Reihen vor dem Text fllen falls der Font zu niedrig ist
				{
				sArea.left	 = iPosX+iOff;
				sArea.right	 = iPosX+iOff+uSize+1;
				sArea.top	 = iPosY-pData->iRowPre;
				sArea.bottom = iPosY;

				if(sData.pText[i]!='\t')
					{
					sArea.left	+= pData->iFontOff;
					sArea.right	+= pData->iFontOff;
					}

				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
				}


			iPosX += uSize;

			if(iPosX>pRect->right-iOff)break;
			}

		if(iPosX<pRect->right-iOff)								// Restliche Zeile ausmahlen
			{
			if(pData->iMarkType!=3)
				{
				if(!(pData->uStyleEx&CLB_EX_COLORCONTINUE))
					{
					bBack = 0;
					}
				else if(pData->uSelectedLine==uLine && (pData->uStyleEx&CLB_EX_SELMARK))
					{
					bBack=(pData->cHasFocus)? 1:2;
					}
				}

			   uColor = pData->aColors[MAX_COLORS+(bBack&CLB_COLOR_MASK)];
			if(uColor>=0xFF000000)								// Standart Hintergrundfarbe whlen
				{
				   uNum = (bBack&CLB_COLOR_MASK)*2;
				if(uNum>4)uNum=0;
				if(pData->uStyleEx&CLB_EX_ALTERNATECOLOR)
					{
					uNum += uLine&1;
					}

				uColor = pData->aColors[CLB_COLOR_EVENWND+uNum];
				}

			SetBkColor(hDc,uColor);

			sArea.left	 = iPosX+iOff;
			sArea.right	 = sRect.right+1;
			sArea.top	 = iPosY-pData->iRowPre;
			sArea.bottom = iPosY+pData->iFontHeight+pData->iRowPost;

			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
			}


																// Den Cursor einblenden
		if((int)uLine==pData->iCursorY && pData->cHasCaret && pData->cHasFocus)
			{
			pData->iCaretX  = pData->uStartText;
			pData->iCaretX += PointOfRow(pData,uLine,pData->iCursorX,hDc);
			pData->iCaretY  = iPosY+pData->uScrollY*pData->iRowHeight;

			if(!pData->iCaretExist)
				{
				pData->iCaretExist = 1;
				CreateCaret(hWnd,NULL,2,pData->iFontHeight-1);
				}

			if(pData->iCaretX>=(int)pData->uScrollX)
			if(pData->iCaretX <(int)pData->uScrollX+(int)pData->uSizeX)
				{
				iPos = pData->iCaretX-pData->uScrollX;
				SetCaretPos(iPos,iPosY+1+pData->iRowPre);

				if(!pData->iCaretOn)
					{
					pData->iCaretOn=1;
					ShowCaret(hWnd);
					}
				}
			}

		iPosY += pData->iFontHeight;
		iPosY += pData->iRowPost;


		//************ Zeile abschlieen **************************************


		if(iPosY>=pRect->bottom)break;

		if(pData->uStyleEx&CLB_EX_GRIDLINES)					// Linien nach jeder Zeile
			{
			if((pData->uStyleEx&CLB_EX_ALTERNATECOLOR) && (uLine&1))
					uColor = pData->aColors[CLB_COLOR_FRAME_ODD ];
			else	uColor = pData->aColors[CLB_COLOR_FRAME_EVEN];

			SetBkColor(hDc,uColor);

			sArea.left	 =  0;
			sArea.right	 =  sRect.right+1;
			sArea.bottom =  iPosY+1;
			sArea.top	 =  iPosY;

			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

			iPosY++;
			}


		if(iPosY>=pRect->bottom)break;
		}


//******************** Unteres Ende zeichnen **********************************


	if(!bDone && pData->iCaretOn)								// Cursor ausblenden wenn er nicht im Bild ist
		{
		pData->iCaretOn=0;
		HideCaret(hWnd);
		}

	if(iPosY<pRect->bottom)										// Restliches Fenster ausmahlen
		{
		   uColor = pData->aColors[MAX_COLORS];
		if(uColor>=0xFF000000)									// Standart Hintergrundfarbe whlen
			{
			uColor = pData->aColors[CLB_COLOR_EVENWND];
			}

		SetBkColor(hDc,uColor);

		sArea.left	 =  0;
		sArea.right	 =  pRect->right +1;
		sArea.bottom =  pRect->bottom+1;
		sArea.top	 =  iPosY;

		ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);
		}


	if(pData->iMaxSizeX!=iMaxX)									// X-Scrollbar aktualisieren
		{
		   pData->iMaxSizeX=iMaxX;
		if(pData->iMaxSizeX!=(int)pData->uOldXCount)
			{
			UpdateScrollX(pData);
			}
		}


	UNLOCK(pData);

 	DeleteObject(hRgnMain);

}



